/**
 * \file gre/container.h
 * \brief GRE::CONTAINER class definition
 *
 * \if NODOC
 * $Id: container.h_v 1.6 2016/01/14 16:41:39 bcolombe Exp $
 *
 * $Log: container.h_v $
 * Revision 1.6  2016/01/14 16:41:39  bcolombe
 * Remove qualification on method
 *
 * Revision 1.5  2013/10/23 21:37:07  mju
 * Add updateRgnMgrNode and use treeviewnode instead of folder.
 *
 * Revision 1.4  2013/10/22 20:35:26  mju
 * Add accessRgnMgrFolder.
 *
 * Revision 1.3  2013/10/21 22:41:20  mju
 * Add treeviewfolder.
 *
 * Revision 1.2  2013/02/15 21:30:42  mju
 * Add findChildByName.
 *
 * Revision 1.1  2013/02/14 22:42:34  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_CONTAINER_H
#define	INC_GRE_CONTAINER_H

#ifndef	INC_GRE_OBJECTBASE_H
	#include <gre/objectbase.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif

namespace GRE {

//===================================================================================================================

//! General container class to manage list of objects.
class CONTAINER : public GRE::OBJECT {
	public:

		//! Destructor.
		virtual ~CONTAINER ();

		//! Access "region manager" node for container.
		//! @return Pointer to node if available, NULL if not.
		//! Will create for non-temporary container if does not already exist.
		TREEVIEWNODE* AccessRgnMgrNode ();

		//! Destroy all child objects.
		void DestroyAllChildren ();

		//! Find child with specified name and type(s).
		OBJECT* FindChildByName (
			const MISTRING& name,				//!< Object name
			OBJECT::TYPE type,					//!< Object type(s)
			const OBJECT *pObjectSkip = 0		//!< Object to skip
			) const;

		//! Find first layer in depth-first order.
		LAYER* FindFirstLayer () const;

		//! Find first object of specified type.
		OBJECT* FindFirstObjectOfType (
			OBJECT::TYPE type,					//!< Object type(s)
			bool recursive = false				//!< Whether to perform recursive (depth-first) search (for TYPE_Layer only)
			) const;

		//! Find last layer in depth-first order.
		LAYER* FindLastLayer () const;

		//! Find last object of specified type.
		OBJECT* FindLastObjectOfType (
			OBJECT::TYPE type,					//!< Object type(s)
			bool recursive = false				//!< Whether to perform recursive (depth-first) search (for TYPE_Layer only)
			) const;

		//! Find first or last layer selected by specified delegate.
		LAYER* FindLayer (
			DELEGATE_BOOL_LAYER testLayer,	//!< Delegate to test if layer is suitable
			bool uselast = false					//!< Use last layer instead of first
			) const;

		//! Find layer using unique ID.
		LAYER* FindLayerByID (
			UINT32 LayerID							//!< Layer ID
			) const;

		//! Find all layers selected by specified delegate.
		void FindLayers (
			DELEGATE_BOOL_LAYER testLayer,	//!< Delegate to test if layer is suitable
			SIMPLE_ARRAY<LAYER*>& layers		//!< Array of layer pointers returned
			) const;

		//! Get active object for container.
		OBJECT* GetActiveChild () const
			{ return (m_pActiveChild); }

		//! Get active layer.
		LAYER* GetActiveLayer () const;

		//! Get first object in container.
		OBJECT* GetFirstChild () const
			{ return (m_pFirstChild); }

		//! Get last object in container.
		OBJECT* GetLastChild () const
			{ return (m_pLastChild); }

		//! Get next available unique layer ID.
		UINT32 GetNextLayerID () const;

		//! Get number of objects in container.
		UINT32 GetNumObjects () const;

		//! Determine if has a child object.
		bool HasChild () const
			{ return (m_pFirstChild != 0); }

		//! Determine if has any layers.
		bool HasLayers () const;

		//! Determine if list contains specified object.
		bool ListContains (
			const OBJECT *pObject
			) const;

		//! Process all layers using specified delegate.
		ERRVALUE ProcessLayers (
			DELEGATE_ERRVALUE_LAYER procLayer	//!< Delegate to process layer
			) const;

		//! Set "active" object.
		//! Will send notification to previous and new active objects.
		void SetActiveChild (
			OBJECT *pObject							//!< Object to set as active.
			);

		//! Determine if object contained anywhere in container's subtree.
		bool TreeContains (
			const OBJECT *pObject,					//!< Object to check, will not be dereferenced
			OBJECT::TYPE objtype						//!< Object type
			) const;

		//! Determine if has any temporary objects in hierarchy.
		bool TreeHasTemporary () const;

		//! Update region manager node.
		//! Will create if does not exist or send change notification if already exists.
		void UpdateRgnMgrNode (bool bValid);

	protected:

		//! Constructor.
		CONTAINER (
			OBJECT::TYPE type,
			CONTAINER *pContainer,
			LISTPOS listpos = LISTPOS_Last,
			OBJECT *pRefObject = 0
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		friend class GRE::OBJECT;

		OBJECT *m_pFirstChild;
		OBJECT *m_pLastChild;
		OBJECT *m_pActiveChild;
		TREEVIEWNODE *m_pRgnMgrNode;

		void AddChild (OBJECT *pChild, LISTPOS listpos, OBJECT *pRefObject);
		void FindLayersLow (DELEGATE_BOOL_LAYER testLayer, SIMPLE_ARRAY<LAYER*>& layers) const;
		void RemoveChild (OBJECT *pChild);
		ERRVALUE ReservePrivDataLow (int NewNumAlloc);

		virtual TREEVIEWNODE* v_CreateRgnMgrNode (TREEVIEWNODE& nodeParent, TREEVIEWNODE *pNodeInsertBefore);

		#endif
	};

//===================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_CONTAINER_H
