/**
 * \file gre/datatipspec.h
 * \brief GRE::DATATIPSPEC class definition.
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_GRE_DATATIPSPEC_H
#define	INC_GRE_DATATIPSPEC_H

#ifndef	INC_RVC_OBJECTNAME_H
	#include <rvc/objectname.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
	#include <mi32/serializeritem.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
namespace GRE {
	class LAYER;
	}
namespace RVC {
	class DBASE;
	class DBTABLE;
	}
namespace SPATMOD {
	namespace IMAGE {
		class STAGE_ARRAY;
		class STAGE_AUTOPTR;
		}
	}
#endif


namespace GRE {

//=====================================================================================================================

//! DataTip specification.
class DATATIPSPEC {
	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		class FORM;
		#endif

		//! Field specification.
		class FIELD {
			public:

				//! BuiltIn built-in values.
				enum BUILTIN {
					BUILTIN_None =								0,
					BUILTIN_ImageCellValue =				1,
					BUILTIN_ImageLinkTileName =			2,
					BUILTIN_LidarClassName =				3,
					BUILTIN_LidarReturnNumberCount =		4,
					BUILTIN_ComponentFileName =			5,
					BUILTIN_ComponentObjectName =			6,
					// Values stored with serialization, so cannot change.
					};

				//! Field type for formatting.
				enum TYPE {
					TYPE_Unspecified = 0,
					TYPE_String,
					TYPE_Integer,
					TYPE_Float,
					TYPE_Image
					};

				//! Get serialization item definition array.
				static const SERIALIZERITEM* SerialGetItemDef();

				//! Default constructor.
				FIELD ();

				//! Destructor.
				~FIELD ();

				// \if NODOC
				bool CanSerializeLegacy () const;
				// \endif

				//! Clear field.
				void Clear ();

				//! Format numeric value.
				MISTRING FormatValue (
					double value,						//!< Value to format
					UINT8 unitnum						//!< Unit of passed value
					) const;

				//! Get "builtin" setting.
				BUILTIN GetBuiltIn () const
					{ return (m_BuiltIn); }

				//! Get number of decimal places for numeric values.
				UINT8 GetDecPlaces () const
					{ return (m_DecPlaces); }

				//! Get field name.
				const MISTRING& GetFieldName () const
					{ return (m_FieldName); }

				//! Get Field number.
				INT32 GetFieldNum () const
					{ return (m_FieldNum); }

				//! Get table name.
				const MISTRING& GetTableName () const
					{ return (m_TableName); }

				//! Get field type.
				TYPE GetType () const
					{ return (m_Type); }

				//! Get unit ID within type.
				UINT8 GetUnitShown () const
					{ return (m_UnitShown); }

				//! Get unit type ID.
				UINT8 GetUnitType () const
					{ return (m_UnitType); }

				//! Determine if field is builtin (predefined, builtin, etc).
				bool IsBuiltIn () const
					{ return (m_BuiltIn != BUILTIN_None); }

				//! Determine if field is numeric.
				bool IsNumeric () const
					{ return (m_Type == TYPE_Integer || m_Type == TYPE_Float); }

				//! Read string or image from table.
				//! @return TRUE if have value, FALSE if not, < 0 if error.
				int ReadValue (
					RVC::DBTABLE &ObjTable,				//!< Table to read from
					ELEMTYPE ElemType,					//!< Element type
					ELEMNUM ElemNum,						//!< Element number
					MISTRING &string,						//!< String to contain result, will be empty if is image or no value
					SPATMOD::IMAGE::STAGE_AUTOPTR &PtrImage,	//!< Pointer to image returned, if any
					RVC::DBTABLE *pSourceTable = 0	//!< Source table if required
					) const;

				//! Set as builtin field.
				void SetBuiltIn (
					BUILTIN builtin
					);

				//! Set number of decimal places for numeric field.
				void SetDecPlaces (
					UINT8 DecPlaces
					) { m_DecPlaces = DecPlaces; }

				//! Set field name.
				//! This will reset the field number, use Validate after setting name.
				void SetFieldName (
					const MISTRING &FieldName
					);

				//! Set allowed image size range.
				void SetImageSizeRange (
					UINT32 MinColumns,					//!< Minimum columns, 0 if no minimum
					UINT32 MinRows,						//!< Minimum rows, 0 if no minimum
					UINT32 MaxColumns,					//!< Maximum columns, 0 if no maximum
					UINT32 MaxRows							//!< Maximum rows, 0 if no maximum
					);

				//! Set table name.
				//! This will reset the field number, use Validate after setting name.
				void SetTableName (
					const MISTRING &TableName
					);

				//! Set field type.
				//! Usually this is done via Validate, use caution when setting directly.
				void SetType (
					TYPE Type
					);

				//! Set unit ID (within type).
				void SetUnitShown (
					UINT8 UnitShown
					) { m_UnitShown = UnitShown; }

				//! Set unit type ID.
				void SetUnitType (
					UINT8 UnitType
					) { m_UnitType = UnitType; }

				//! Validate field description.
				//! @return TRUE if changed, FALSE if not, < 0 if error.
				int Validate (
					RVC::DBASE &ObjDb
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				typedef DATATIPSPEC::FIELD THISCLASS;

				MISTRING m_TableName;
				MISTRING m_FieldName;
				INT32 m_FieldNum;
				UINT32 m_ImageMinColumns;
				UINT32 m_ImageMinRows;
				UINT32 m_ImageMaxColumns;
				UINT32 m_ImageMaxRows;
				TYPE m_Type;
				BUILTIN m_BuiltIn;
				UINT8 m_DecPlaces;
				UINT8 m_UnitType;
				UINT8 m_UnitShown;
				bool m_bIsValid;
				#endif
			};

		typedef MILIST<FIELD> FIELDLIST;

		//! Get serialization item definition array.
		static const SERIALIZERITEM* SerialGetItemDef();

		//! Default constructor.
		DATATIPSPEC ();

		//! Copy constructor.
		DATATIPSPEC (
			const DATATIPSPEC &rhs
			);

		//! Assignment.
		DATATIPSPEC& operator= (
			const DATATIPSPEC &rhs
			);

		//! Add object name prefix if specified.
		void AddObjNamePrefix (
			MISTRING &string,						//!< String to add to
			const MISTRING &name					//!< Object name
			) const;

		//! Clear all values.
		void Clear ();

		//! Format values for layer component.
		ERRVALUE FormatValues (
			LAYER *layer,									//!< Layer
			int component,									//!< Layer component
			ELEMNUM ElemNum,								//!< Element number
			MISTRING &string,								//!< String to contain results
			SPATMOD::IMAGE::STAGE_ARRAY &images,	//!< Image array (appended to)
			RVC::DBTABLE *pSourceTable = 0			//!< Source table
			) const;

		//! Get maximum visible (reciprocal) scale.
		double GetScaleVisibleMax () const
			{ return (m_ScaleVisibleMax); }

		//! Get minimum visible (reciprocal) scale.
		double GetScaleVisibleMin () const
			{ return (m_ScaleVisibleMin); }

		//! Determine if datatip is defined.
		bool IsDefined () const;

		//! Determine if datatip is enabled for display.
		bool IsEnabled () const
			{ return (m_bEnabled); }

		//! Determine if is enabled at specified scale.
		bool IsEnabledAtScale (
			double scale
			) const;

		//! Set whether enabled for display.
		void SetEnabled (
			bool bEnabled
			) { m_bEnabled = bEnabled; }

		//! Set whether to highlight corresponding element.
		void SetHighlightElem (
			bool bHighlightElem
			) { m_bHighlightElem = bHighlightElem; }

		//! Set visible scale range.
		void SetScaleVisibleRange (
			double ScaleVisibleMin,
			double ScaleVisibleMax
			) { m_ScaleVisibleMin = ScaleVisibleMin; m_ScaleVisibleMax = ScaleVisibleMax; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DATATIPSPEC THISCLASS;

		MISTRING m_Template;					// Template string used to generate datatip
		FIELDLIST m_FieldList;
		double m_ScaleVisibleMin;
		double m_ScaleVisibleMax;
		bool m_bEnabled;
		bool m_bHighlightElem;

		static ERRVALUE SerializerCB (
			SERIALIZER &serializer,
			const SERIALIZERITEM *pItem,
			void *vpInst,
			SERIALIZERITEM::ACTION action
			);

		ERRVALUE SerialReadWrite (
			SERIALIZER &serializer,
			const SERIALIZERITEM &item,
			SERIALIZERITEM::ACTION action
			);

		#endif

	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_DATATIPSPEC_H
