/**
 * \file gre/legend.h
 * \brief GRE::LEGEND base class definitions
 *
 * \if NODOC
 * $Id: legend.h_v 1.30 2016/11/04 16:57:31 mju Exp $
 *
 * $Log: legend.h_v $
 * Revision 1.30  2016/11/04 16:57:31  mju
 * Make getStyleObjItem const.
 *
 * Revision 1.29  2015/06/08 20:53:57  mju
 * Add parms for setting sortstr.
 *
 * Revision 1.28  2015/05/04 16:34:25  ldyas
 * y
 *
 * Revision 1.27  2015/05/04 16:31:59  ldyas
 * added method to get geometric objitem.
 *
 * Revision 1.26  2015/04/30 13:37:03  mju
 * Use elemidx hdr.
 *
 * Revision 1.25  2014/12/08 17:26:35  mju
 * Use rect hdr.
 *
 * Revision 1.24  2014/06/23 20:56:53  mju
 * Add usesDesignScale.
 *
 * Revision 1.23  2014/06/10 14:36:05  mju
 * Remove unused defns.
 *
 * Revision 1.22  2011/10/20 12:49:26  mju
 * Remove redundant hdr.
 *
 * Revision 1.21  2010/08/19 17:40:22  mju
 * Use elemnum type.
 *
 * Revision 1.20  2010/06/18 18:27:45  mju
 * Add colorscale::setLength and move colorscale::item defn to implementation file.
 *
 * Revision 1.19  2010/06/17 18:01:37  mju
 * Add colorspread ctor that does not use cbparms.
 *
 * Revision 1.18  2009/09/01 14:34:37  mju
 * Add getNeedStyleObject.
 *
 * Revision 1.15  2007/07/05 20:36:02  mju
 * Add setDftBackgroundColor and corresponding static member.
 *
 * Revision 1.14  2007/07/02 22:32:59  mju
 * Add get/setBackgroundColor.
 *
 * Revision 1.1  2004/03/04 19:00:34  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_LEGEND_H
#define	INC_GRE_LEGEND_H

#ifndef	INC_GRE_ELEMSAMPLEDC_H
	#include <gre/elemsampledc.h>
#endif

#ifndef	INC_RVC_ELEMIDX_H
	#include <rvc/elemidx.h>
#endif

#ifndef	INC_MI32_ELEMSTYL_H
	#include <mi32/elemstyl.h>
#endif

#ifndef	INC_MI32_COLORARRAY_H
	#include <mi32/colorarray.h>
#endif

#ifndef	INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif

#if !defined(INC_MGUI_TREE_H) && defined(USE_MGUI)
	#include <mgui/tree.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT

struct CBPARMS;

namespace MGUI {
	class CUSTOMDRAW_TREE;
	};

namespace RVC {
	class DBASE_ELEMENT;
	class DBTABLE;
	class DBTABLE_STYLE;
	class STYLE;
	};

#endif // GENERATING_DOXYGEN_OUTPUT


namespace GRE {

//=====================================================================================================================

//! Legend base class.
class LEGEND {
	public:

		class DRAWCONTEXT;
		class ITEMBASE;
		typedef MILIST<ITEMBASE*> ITEMLIST;

		//! Set default background color.
		static void SetDftBackgroundColor (
			const COLOR &color
			) { s_DftBackgroundColor = color; }

		//! Constructor
		explicit LEGEND (
			GRE::LAYER *layer,
			ELEMTYPE ElemType
			);

		//! Destructor.
		virtual ~LEGEND ();

	#ifdef USE_MGUI
		//! Add elemidx to legend 'element table' with prompt for label(s) (MGUI only).
		ERRVALUE AddElemPrompted (
			MDLGPARENT dlgparent,					//!< Parent for dialog
			const RVC::ELEMIDX &elemidx			//!< Element to add
			) { return (v_AddElemPrompted(dlgparent,elemidx)); }
	#endif

		//! Determine if legend 'element table' contains specified elemidx.
		bool ContainsElem (
			const RVC::ELEMIDX &elemidx			//!< Element
			) const;

		//! Create 'element table' if don't already have one attached.
		ERRVALUE CreateElemTable (
			const RVC::DESCRIPTOR &descriptor	//!< Table descriptor
			);

		//! Delete all items from the legend.
		void DeleteAllItems ();

		//! Delete elemidx from legend 'element table'.
		ERRVALUE DeleteElem (
			const RVC::ELEMIDX &elemidx			//!< Element
			);

		//! Draw entire legend.
		void Draw (
			const DRAWCONTEXT &dc,					//!< Legend drawing context
			LPOINT2D &origin							//!< Origin passed / returned
			) const;

		const COLOR& GetBackgroundColor (
			) const { return (m_ColorBG); }

		//! Return element type legend is for.
		ELEMTYPE GetElemType (
			) const { return (m_ElemType); }

		//! Get reference to geometric object item referred to by legend.
		const RVC::OBJITEM& GetGeoObjItem (
			) {return (m_GeoObjItem); }

		//! Get legend item list.
		const ITEMLIST& GetItemList (
			) const { return (m_ItemList); }

		//! Get "preferred" item size in pixels.
		//! This is used for rendering legend in user interface (ie. LegendView).
		void GetItemSizePixels (
			LPOINT2D& size								//!< Size in pixels returned, not including labels
			) const { v_GetItemSizePixels(size); }

		//! Return layer legend is associated with.
		GRE::LAYER* GetLayer (
			) const { return (m_layer); }

		//! Determine if set as needing style object.
		bool GetNeedStyleObject (
			) const { return (m_NeedStyleObj); }

		//! Return flags for sample style rendering.
		//! @return Flags for sample style rendering.
		MGD::STYLESAMPLEFLAGS GetSampleFlags (
			) const { return (m_SampleFlags); }

		//! Get size of legend given sample item height and width.
		//! @return Size of legend given sample item height and width.
		LPOINT2D GetSize (
			const DRAWCONTEXT &dc					//!< Legend drawing context
			) const;

		//! Get reference to style object item used by legend.
		const RVC::OBJITEM& GetStyleObjItem () const
			{ return (m_StyleObjItem); }

		//! Determine if legend has associated 'element table'.
		bool HasElemTable () const;

		//! Determine if legend has any items.
		bool HasItems (
			) const { return (!m_ItemList.IsEmpty()); }

		//! Determine if duplicate styles are being merged as items are added.
		bool IsMergingDuplicateStyles (
			) const { return (m_MergeDupStyles); }

		//! Determine if legend is sorted by label strings.
		bool IsSorted (
			) const { return (m_Sorted); }

		//! Set background color
		void SetBackgroundColor (
			const COLOR &color
			) { m_ColorBG = color; }

		//! Set whether to merge duplicate styles. 
		void SetMergeDuplicateStyles (
			bool MergeDupStyles = true
			) { m_MergeDupStyles = MergeDupStyles; }

		//! Set whether a style object is needed.
		void SetNeedStyleObject (
			bool NeedStyleObj = true
			) { m_NeedStyleObj = NeedStyleObj; }

		//! Set sample flags.
		void SetSampleFlags (
			MGD::STYLESAMPLEFLAGS newflags
			) { m_SampleFlags = newflags | MGD::STYLESAMPLE_ShrinkToFit; }

		//! Set whether should be sorted by labels.
		//! Must be done before adding items.
		void SetSorted (						
			bool sorted = true
			) { m_Sorted = sorted; }

		//! Set style object to use for legend using RVC::OBJITEM reference.
		void SetStyleObject (
			const RVC::OBJITEM &StyleObj
			);

	#ifdef USE_MGUI
		//! Add all legend items to TreeView (MGUI only).
		//! @return Handle for last item inserted.
		MGUI::CTRL_TREE::ITEMHND TreeViewInsertItems (
			GRE::CTRL_TREEVIEW &TreeView,
			MGUI::CTRL_TREE::ITEMHND hParent,
			MGUI::CTRL_TREE::ITEMHND hInsertAfter
			);
	#endif

		//! Determine if any legend item uses "design scale" in styling.
		bool UsesDesignScale () const
			{ return (m_bUsesDesignScale); }

	protected:

		ITEMLIST m_ItemList;						//!< List of legend items
		RVC::OBJITEM m_GeoObjItem;
		RVC::DBTABLE *m_pElemTable;

		ERRVALUE AddElemRecord (
			const RVC::ELEMIDX &elemidx,
			const MISTRING &prilabel,
			const MISTRING &seclabel
			);

		void InsertItem (
			ITEMBASE *item 								//!< Item to insert
			);

		ERRVALUE OpenElemTable (
			const RVC::OBJECTNAME &ElemTableName
			);

		ERRVALUE OpenStyle (
			RVC::DBTABLE_STYLE &StyleTable,
			const RVC::OBJITEM &StyleObjItem,
			const RVC::OBJITEM &GeoObjItem,
			RVC::STYLE &StyleObj
			);

		void SetHasMergedStyles ()
			{ m_HasMergedStyles = true; }

		void SetUsesDesignScale ()
			{ m_bUsesDesignScale = true; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static COLOR s_DftBackgroundColor;

		RVC::OBJITEM m_StyleObjItem;
		COLOR m_ColorBG;
		GRE::LAYER *m_layer;							// Layer legend is associated with
		ELEMTYPE m_ElemType;							// Element type legend is for
		MGD::STYLESAMPLEFLAGS m_SampleFlags;	// Flags for sample style drawing
		bool m_Sorted;									// Set if sorted
		bool m_NeedStyleObj;							// Set if need style object for patterns
		bool m_MergeDupStyles;						// Merge duplicate styles when adding
		bool m_HasMergedStyles;						// Duplicate styles have been merged
		bool m_bUsesDesignScale;

		LEGEND (const LEGEND&);
		LEGEND& operator= (const LEGEND&);

		friend class DRAWCONTEXT;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

	#ifdef USE_MGUI
		//! Add elemidx to legend 'element table' with prompt for label(s) (MGUI only).
		//! Default implementation returns EProgramError.
		virtual ERRVALUE v_AddElemPrompted (
			MDLGPARENT dlgparent,					//!< Parent for dialog
			const RVC::ELEMIDX &elemidx			//!< Element to add
			);
	#endif

		//! Get "preferred" item size in pixels.
		//! This is used for rendering legend in user interface (ie. LegendView).
		virtual void v_GetItemSizePixels (
			LPOINT2D &size								//!< Size in pixels returned, not including labels
			) const;

	};

//=====================================================================================================================

//! Context for legend item drawing.
class LEGEND::DRAWCONTEXT : public ELEMSAMPLE_DC {
	public:

		//! Orientation for drawing if supported by legend type.
		//! This is used by "color scale" legend.
		enum ORIENTATION {
			ORIENTATION_Vertical = 		0,		//!< Draw items in "vertical" orientation (default)
			ORIENTATION_Horizonatal =	1		//!< Draw items in "horizontal" orientation
			};

		GRE::LAYER *m_layer;
		ELEMTYPE m_ElemType;
		INT32 m_itemheight;
		INT32 m_samplewidth;
		INT32 m_labelspace;
		INT32 m_itemspace;
		LRECT2D m_cliprect;
		bool m_DrawLabels;
		bool m_DrawLabelsOnColorScales;		//!< Special case override for above flag.
		bool m_HasTextStyle;
		TEXTSTYLE m_TextStyle;
		ORIENTATION m_Orientation;

		//! Constructor for use in user interface
		DRAWCONTEXT (
			const LEGEND &legend,
			MGD::CONTEXT *gc,
			INT32 labelspace,						//!< Spacing between sample and label in pixels
			INT32 itemspace,						//!< Spacing between items in pixels
			const LRECT2D &cliprect				//!< Clipping rectangle
			);

		virtual ~DRAWCONTEXT ();

		//! Determine if legend has vertical orientation.
		bool IsVertical (
			) const { return (m_Orientation == ORIENTATION_Vertical); }

		//! Set whether or not to draw labels for legend items.
		void SetDrawLabels (
			bool DrawLabels
			) { m_DrawLabels = DrawLabels; }

	};	//! End of LEGEND::DRAWCONTEXT class.

//=====================================================================================================================

//! Base class for legend item defining virtual methods for drawing
class LEGEND::ITEMBASE {
	public:

		//! Construct item for legend.
		ITEMBASE (
			LEGEND &legend
			);

		//! Construct with no specified element
		ITEMBASE (
			LEGEND &legend,
			const MISTRING &label,					//!< Label
			const MISTRING &sortstr					//!< String for sorting
			);

		//! Construct with elemidx and label.
		ITEMBASE (
			LEGEND &legend,
			const RVC::ELEMIDX &elemidx,		//!< Element from associated object
			const MISTRING &label				//!< Label
			);

		//! Destructor.
		virtual ~ITEMBASE ();

		//! Draw item.
		void Draw (
			const DRAWCONTEXT &dc,				//!< Legend drawing context
			const LRECT2D &samplerect			//!< Rectangle for sample symbol, not including label
			) const { v_Draw(dc,samplerect); }

		//! Draw label.
		void DrawLabel (
			const DRAWCONTEXT &dc,				//!< Legend drawing context
			const LRECT2D &samplerect			//!< Rectangle for sample symbol, not including label
			) const;

		//! Get elemidx used by item.
		const RVC::ELEMIDX& GetElement (
			) const { return (m_elemidx); }

		//! Get element number.
		ELEMNUM GetElemNum (
			) const { return (m_elemidx.GetNumber()); }

		//! Get the label.
		const MISTRING& GetLabel (
			) const { return (m_label); }

		//! Get size of label.
		void GetLabelSize (
			const DRAWCONTEXT &dc,				//!< Legend drawing context
			LPOINT2D &size							//!< Size in "display pixels" returned
			) const;

		//! Get reference to legend.
		LEGEND& GetLegend (
			) { return (m_legend); }

		//! Get reference to legend (const).
		const LEGEND& GetLegend (
			) const { return (m_legend); }

		//! Get string used for sorting.
		const MISTRING& GetSortString () const
			{ return (m_sortstr); }

		//! Determine if item requires transparency support.
		bool HasTransparency (
			) const { return (v_HasTransparency()); };

		//! Merge label with current label.
		void MergeLabel (
			const MISTRING &label
			);

		//! Set label.
		void SetLabel (
			const MISTRING &label
			) { m_label = label; }

	#ifdef USE_MGUI
		//! Insert item into TreeView (MGUI only).
		MGUI::CTRL_TREE::ITEMHND TreeViewInsertItem (
			GRE::CTRL_TREEVIEW &TreeView,
			MGUI::CTRL_TREE::ITEMHND hndParent,
			MGUI::CTRL_TREE::ITEMHND hndInsertAfter
			);
	#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LEGEND &m_legend;
		RVC::ELEMIDX m_elemidx;
		MISTRING m_label;
		MISTRING m_sortstr;

	#ifdef USE_MGUI
		//! Draw item with label in TreeView (MGUI only).
		MGUI::CUSTOMDRAW::RETVALUE TreeViewOnCustomDraw (
			GRE::CTRL_TREEVIEW &TreeView,
			MGUI::CUSTOMDRAW_TREE &CustomDraw	//!< Custom drawing information from CTRL_TREE
			);
	#endif

		ITEMBASE (const ITEMBASE&);
		ITEMBASE& operator= (const ITEMBASE&);
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Draw item.
		virtual void v_Draw (
			const DRAWCONTEXT &dc,				//!< Legend drawing context
			const LRECT2D &samplerect			//!< Rectangle for sample symbol, not including label
			) const = 0;

		//! Determine if item requires transparency support.
		virtual bool v_HasTransparency (
			) const;

	#ifdef USE_MGUI
		//! Draw item in TreeView (MGUI only).
		virtual void v_TreeViewCustomDraw (
			GRE::CTRL_TREEVIEW &TreeView,
			MGUI::CUSTOMDRAW_TREE &CustomDraw	//!< Custom drawing information from CTRL_TREE
			) = 0;
	#endif

	};	// End of LEGEND::ITEMBASE class.

//===================================================================================================================

//! Point Feature legend subclass.
class LEGEND_POINT : public LEGEND {
	public:

		class ITEM;

		//! Construct legend for layer and element type.
		LEGEND_POINT (
			GRE::LAYER *layer,							//!< Layer legend is for
			ELEMTYPE ElemType							//!< Element type
			);

		//! Add item to legend with label.
		void AddItem (
			const POINTSTYLE &style,
			const MISTRING &label
			) { AddItem(style,label,label); }

		//! Add item to legend with label and sort string.
		void AddItem (
			const POINTSTYLE &style,
			const MISTRING &label,
			const MISTRING &sortstr						//!< String for sorting
			);

		//! Add item to legend.
		void AddItem (
			const RVC::ELEMIDX &elemidx,				//!< Element to add
			const MISTRING &label
			);

		ERRVALUE Setup (
			const POINTPARMS &parms,
			const RVC::OBJITEM &GeoObjItem,
			const RVC::OBJITEM &StyleObjItem
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// LEGEND overrides.
	#ifdef USE_MGUI
		virtual ERRVALUE v_AddElemPrompted (MDLGPARENT dlgparent, const RVC::ELEMIDX &elemidx);
	#endif

		LEGEND_POINT (const LEGEND_POINT&);
		LEGEND_POINT& operator= (const LEGEND_POINT&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Individual item in point legend.
class LEGEND_POINT::ITEM : public LEGEND::ITEMBASE {
	public:

		//! Constructor using POINTSTYLE and label.
		ITEM (
			LEGEND_POINT &legend,
			const POINTSTYLE &style,
			const MISTRING &label,
			const MISTRING &sortstr
			);

		//! Constructor using element number and label.
		ITEM (
			LEGEND_POINT &legend,
			const RVC::ELEMIDX &elemidx,		//!< Element
			const MISTRING &label				//!< Label
			);

		//! Destructor.
		virtual ~ITEM ();

		//! Get style for item.
		const POINTSTYLE& GetStyle (
			) const { return (m_style); }

		//! Determine if this item has same style.
		bool HasSameStyle (
			const POINTSTYLE &style				//!< Style to compare
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Overrides.
		virtual void v_Draw (const DRAWCONTEXT &dc, const LRECT2D &samplerect) const;
		virtual bool v_HasTransparency () const;
	#ifdef USE_MGUI
		virtual void v_TreeViewCustomDraw (CTRL_TREEVIEW &TreeView, MGUI::CUSTOMDRAW_TREE &CustomDraw);
	#endif

		POINTSTYLE m_style;
		ITEM (const ITEM&);
		ITEM& operator= (const ITEM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Line Feature legend subclass.
class LEGEND_LINE : public LEGEND {
	public:

		class ITEM;

		//! Construct legend for layer and element type.
		LEGEND_LINE (
			GRE::LAYER *layer,							//!< Layer legend is for
			ELEMTYPE ElemType							//!< Element type
			);

		//! Add item to legend with label.
		void AddItem (
			const LINESTYLE &style,
			const MISTRING &label
			) { AddItem(style,label,label); }

		//! Add item to legend with label and sort string.
		void AddItem (
			const LINESTYLE &style,
			const MISTRING &label,
			const MISTRING &sortstr						//!< String for sorting
			);

		//! Add item to legend.
		void AddItem (
			const RVC::ELEMIDX &elemidx,				//!< Element to add
			const MISTRING &label
			);

		ERRVALUE Setup (
			const LINEPARMS &parm,
			const RVC::OBJITEM &GeoObjItem,
			const RVC::OBJITEM &StyleObjItem
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// LEGEND overrides.
	#ifdef USE_MGUI
		virtual ERRVALUE v_AddElemPrompted (MDLGPARENT dlgparent, const RVC::ELEMIDX &elemidx);
	#endif

		LEGEND_LINE (const LEGEND_LINE&);
		LEGEND_LINE& operator= (const LEGEND_LINE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Individual item in line legend.
class LEGEND_LINE::ITEM : public LEGEND::ITEMBASE {
	public:

		//! Constructor.
		ITEM (
			LEGEND_LINE &legend,
			const LINESTYLE &style,
			const MISTRING &label,
			const MISTRING &sortstr
			);

		//! Constructor using element number and label.
		ITEM (
			LEGEND_LINE &legend,
			const RVC::ELEMIDX &elemidx,		//!< Element
			const MISTRING &label				//!< Label
			);

		//! Destructor.
		virtual ~ITEM ();

		//! Get style for item.
		const LINESTYLE& GetStyle (
			) const { return (m_style); };

		//! Determine if this item has same style.
		bool HasSameStyle (
			const LINESTYLE &style				//!< Style to compare
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Overrides.
		virtual void v_Draw (const DRAWCONTEXT &dc, const LRECT2D &samplerect) const;
		virtual bool v_HasTransparency () const;
	#ifdef USE_MGUI
		virtual void v_TreeViewCustomDraw (CTRL_TREEVIEW &TreeView, MGUI::CUSTOMDRAW_TREE &CustomDraw);
	#endif

		LINESTYLE m_style;
		ITEM (const ITEM&);
		ITEM& operator= (const ITEM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Polygon/Area Feature legend subclass.
class LEGEND_POLYGON : public LEGEND {
	public:

		class ITEM;

		//! Construct legend for layer and element type.
		LEGEND_POLYGON (
			GRE::LAYER *layer,								//!< Layer legend is for
			ELEMTYPE ElemType								//!< Element type
			);

		//! Add item to legend with label.
		void AddItem (
			const POLYSTYLE &style,
			const MISTRING &label,						//!< Label to place to right of sample
			const MISTRING &intlabel					//!< Label to place INSIDE sample (blank for none)
			) { AddItem(style,label,intlabel,label); }

		//! Add item to legend with label and sort string.
		void AddItem (
			const POLYSTYLE &style,
			const MISTRING &label,						//!< Label to place to right of sample
			const MISTRING &intlabel,					//!< Label to place INSIDE sample (blank for none)
			const MISTRING &sortstr						//!< String for sorting
			);

		//! Add item to legend.
		void AddItem (
			const RVC::ELEMIDX &elemidx,				//!< Element to add
			const MISTRING &label,						//!< Label to place to right of sample
			const MISTRING &intlabel					//!< Label to place INSIDE sample (blank for none)
			);											

		ERRVALUE Setup (
			const POLYPARMS &parm,
			const RVC::OBJITEM &GeoObjItem,
			const RVC::OBJITEM &StyleObjItem
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// LEGEND overrides.
	#ifdef USE_MGUI
		virtual ERRVALUE v_AddElemPrompted (MDLGPARENT dlgparent, const RVC::ELEMIDX &elemidx);
	#endif

		LEGEND_POLYGON (const LEGEND_POLYGON&);
		LEGEND_POLYGON& operator= (const LEGEND_POLYGON&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Individual item in polygon legend.
class LEGEND_POLYGON::ITEM : public LEGEND::ITEMBASE {
	public:

		//! Constructor.
		ITEM (
			LEGEND_POLYGON &legend,
			const POLYSTYLE &style,				//!< Item style
			const MISTRING &label,				//!< Label to place to right of sample
			const MISTRING &intlabel,			//!< Label to place INSIDE sample (blank string for none)
			const MISTRING &sortstr
			);

		//! Constructor using element number and label.
		ITEM (
			LEGEND_POLYGON &legend,
			const RVC::ELEMIDX &elemidx,		//!< Element
			const MISTRING &label,				//!< Label to place to right of sample
			const MISTRING &intlabel			//!< Label to place INSIDE sample (blank string for none)
			);

		//! Destructor.
		virtual ~ITEM ();

		//! Get style for item.
		const POLYSTYLE& GetStyle (
			) const { return (m_style); }

		//! Determine if this item has same style.
		bool HasSameStyle (
			const POLYSTYLE &style				//!< Style to compare
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		POLYSTYLE m_style;
		MISTRING m_intlabel;		// Label to place INSIDE sample

		// Draw the Interior label (m_intlabel) label
		void DrawInteriorLabel (
			const DRAWCONTEXT &cd,				// Legend drawing context
			const LRECT2D &samplerect			// Rectangle for sample symbol, not including label
			) const;

		// Overrides.
		virtual void v_Draw (const DRAWCONTEXT &dc, const LRECT2D &samplerect) const;
		virtual bool v_HasTransparency () const;
	#ifdef USE_MGUI
		virtual void v_TreeViewCustomDraw (CTRL_TREEVIEW &TreeView, MGUI::CUSTOMDRAW_TREE &CustomDraw);
	#endif

		ITEM (const ITEM&);
		ITEM& operator= (const ITEM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//===================================================================================================================

//! Color-Sample legend subclass.
class LEGEND_COLORSAMPLE : public LEGEND {
	public:

		class ITEM;

		//! Construct legend for layer and element type.
		LEGEND_COLORSAMPLE (
			GRE::LAYER *layer							//!< Layer legend is for
			);

		//! Add item to legend.
		void AddItem (
			const COLOR &color,
			const MISTRING &label
			);											

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		LEGEND_COLORSAMPLE (const LEGEND_COLORSAMPLE&);
		LEGEND_COLORSAMPLE& operator= (const LEGEND_COLORSAMPLE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

class LEGEND_COLORSAMPLE::ITEM : public LEGEND::ITEMBASE {
	public:

		//! Constructor.
		ITEM (
			LEGEND_COLORSAMPLE &legend,
			const COLOR &color,
			const MISTRING &label
			);

		//! Destructor.
		virtual ~ITEM ();

		//! Get color associated with item.
		const COLOR& GetColor (
			) const { return (m_color); }

		//! Determine if this item has same style.
		bool HasSameStyle (
			const COLOR &color
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		COLOR m_color;

		// Overrides.
		virtual void v_Draw (const DRAWCONTEXT &dc, const LRECT2D &samplerect) const;
	#ifdef USE_MGUI
		virtual void v_TreeViewCustomDraw (CTRL_TREEVIEW &TreeView, MGUI::CUSTOMDRAW_TREE &CustomDraw);
	#endif

		ITEM (const ITEM&);
		ITEM& operator= (const ITEM&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

//! Color-Scale legend subclass.
class LEGEND_COLORSCALE : public LEGEND {
	public:

		class ITEM;

		//! Construct legend for layer.
		LEGEND_COLORSCALE (
			GRE::LAYER *layer							//!< Layer legend is for
			);

		//! Set optional label.
		void SetLabel (
			const MISTRING &label
			);

		//! Set length in pixels.
		void SetLength (
			int length
			);

		//! Set legend values with palette.
		//! This will create or update the single legend "item".
		void SetValues (
			const COLOR_ARRAY &palette,
			double MinValue,
			double MaxValue,
			bool VerticalMinAtTop
			);

		//! Set legend values with palette and color balance.
		//! This will create or update the single legend "item".
		void SetValues (
			const COLOR_ARRAY &palette,
			const CBPARMS &ColorBalance,				//!< For adjusting color
			double MinValue,
			double MaxValue,
			bool VerticalMinAtTop
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		// LEGEND overrides.
		virtual void v_GetItemSizePixels (LPOINT2D &size) const;

		LEGEND_COLORSCALE (const LEGEND_COLORSCALE&);
		LEGEND_COLORSCALE& operator= (const LEGEND_COLORSCALE&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_LEGEND_H
