/**
 * \file gre/lrregion.h
 * \brief GRE Region Layer definitions
 *
 * \if NODOC
 * $Id: lrregion.h_v 1.27 2014/08/21 21:43:20 mju Exp $
 *
 * $Log: lrregion.h_v $
 * Revision 1.27  2014/08/21 21:43:20  mju
 * Add line/point legends.
 *
 * Revision 1.26  2014/08/19 20:45:56  mju
 * Make dispparm members private and use multistyle so can draw lines/points.
 *
 * Revision 1.23  2014/06/20 21:23:40  mju
 * Add v_hasProperty.
 *
 * Revision 1.21  2013/10/11 14:05:13  mju
 * Add copySettings.
 *
 * Revision 1.1  2003/06/13 22:09:27  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_GRE_LRREGION_H
#define	INC_GRE_LRREGION_H

#ifndef	INC_GRE_LAYER_H
#include <gre/layer.h>
#endif

#ifndef	INC_GRE_LEGEND_H
#include <gre/legend.h>
#endif

namespace GRE {

//=====================================================================================================================

//! Region layer.
class LAYER_REGION : public GRE::LAYER {
	public:

		// LAYER_REGION::DISPPARM
		class DISPPARM : public GRE::LAYER::DISPPARM {
			public:

				class DLG;

				DISPPARM ();

				DISPPARM (
					const DISPPARM& rhs
					): GRE::LAYER::DISPPARM(rhs) { Copy(rhs); }

				virtual ~DISPPARM (
					) { Free(); }

				DISPPARM& operator= (const DISPPARM& rhs) {
					if (this != &rhs) {
						Free();
						Copy(rhs);
						}
					return (*this);
					}

				//! Access style for direct changing.
				MULTISTYLE& AccessStyle ()
					{ return (m_Style); }

				//! Access style for subregion drawing.
				POLYSTYLE& AccessStylePoly ()
					{ return (m_Style.Poly); }

				//! Copy settings without changing object.
				//! Does not validate.
				void CopySettingsFrom (
					const DISPPARM& other
					);

				//! Get location of region object.
				const RVC::OBJITEM& GetObjItemRegion () const
					{ return (m_ObjItem); }

				//! Get location of style object.
				const RVC::OBJITEM& GetObjItemStyle () const
					{ return (m_ObjItemStyle); }

				//! Get drawing style.
				const MULTISTYLE& GetStyle () const
					{ return (m_Style); }

				//! Get style for subregion drawing.
				const POLYSTYLE& GetStylePoly () const
					{ return (m_Style.Poly); }

				//! Set to use specified object.
				ERRVALUE SetObject (
					const RVC::OBJITEM& objitem,		//!< Object to set
					LAYER_REGION *layer,
					bool keepsettings = false			//!< Retain current display settings
					);

				//! Set style for all components.
				void SetStyle (
					const MULTISTYLE& style				//!< Style to set
					) { m_Style = style; }
					
				//! Set style for subregion drawing.
				void SetStyle (
					const POLYSTYLE& PolyStyle			//!< Style to set
					) { m_Style.Poly = PolyStyle; }
					
			protected:
				virtual const SERIALIZERITEM* SerialGetItemDef (SERIALIZER& serializer) const;

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::OBJITEM m_ObjItem;
				RVC::OBJITEM m_ObjItemStyle;
				MULTISTYLE m_Style;

				static ERRVALUE SerializerCB_StyleFileName (SERIALIZER&, const SERIALIZERITEM *, void *, SERIALIZERITEM::ACTION);

				void Copy (const DISPPARM&);
				void Free ();

				virtual ERRVALUE v_SetObjItem (const RVC::OBJITEM& objitem, bool replace);
				virtual ERRVALUE v_Validate ();

				friend class LAYER_REGION;
				#endif
			};	// End of LAYER_REGION::DISPPARM

		//! Register this layer type
		static void RegisterType ();

		//! Construct layer and insert into list
		explicit LAYER_REGION (
			GRE::GROUP *group,										//!< Group to contain layer
			CREATEFLAGS createflags = CREATEFLAG_None,	//!< Creation flags
			GRE::LISTPOS listpos = GRE::LISTPOS_Last,		//!< Position in list
			GRE::LAYER *reflayer = 0								//!< Reference layer for insertion
			);

		//! Destructor.
		virtual ~LAYER_REGION ();

		//! Clear region.
		void ClearRegion ();

		//! Retrieve layer-specific display parameters.
		const DISPPARM& GetDispParm () const
			{ return (m_dispparm); }

		//! Set layer-specific display parameters.
		//! This is the recommended method for changing how an existing layer is to be
		//! rendered.  Validation will be performed on the specified parameters and
		//! adjustments made to the layer's copy if necessary.  Use of this method
		//! usually result in a ChangeBegin / ChangeEnd message pair, and will also
		//! generate a ChangeObject message if the spatial object used in the layer
		//! is changed.
		ERRVALUE SetDispParm (
			const DISPPARM& dispparm					//!< Parameters to set
			);

		void SetRegion (
			const REGION2D& region
			);

	protected:

		DISPPARM m_dispparm;								//!< Layer-specific display parameters

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef LAYER BASECLASS;

	#ifdef USE_MGUI
		MGUI::DLGSHELL *m_pDlgControls;
	#endif
		GRE::LEGEND_POLYGON m_LegendArea;
		GRE::LEGEND_LINE m_LegendLine;
		GRE::LEGEND_POINT m_LegendPoint;

		int DrawSetup (void*, GRE::LAYERDC&);

		//! Overrides from GRE::LAYER.
		virtual ERRVALUE v_AssignObject (const RVC::OBJITEM& objitem, MDLGPARENT dlgparent);
	#ifdef USE_MGUI
		virtual int v_ControlPanel (MDLGPARENT dlgparent,UINT32 flags = 0);
	#endif
		virtual GRE::LAYER* v_Copy (GRE::GROUP *group) const;
		virtual ERRVALUE v_CopySettingsFrom (const LAYER *other);
		virtual ERRVALUE v_Draw (GRE::LAYERDC& layerdc);
		virtual const GRE::LEGEND* v_GetLegend (ELEMTYPE ElemType);
		virtual const RVC::OBJITEM& v_GetObjItem (int component) const;
		virtual bool v_HasProperty (PROPERTY property) const;
	#ifdef USE_MGUI
		virtual ERRVALUE v_PromptObject (bool OpenCtrlPanel, MDLGPARENT dlgparent);
	#endif
		virtual const char* v_SerialGetTagName () const;
		virtual void v_SetLegendSampleFlags (MGD::STYLESAMPLEFLAGS SampleFlags);
		virtual ERRVALUE v_SetStyleObjItem (const RVC::OBJITEM& ObjItem);
	#ifdef USE_MGUI
		virtual MGUI::CTRL_TREE::ITEMHND v_TreeViewAddUpdateLegend (GRE::CTRL_TREEVIEW& TreeView, MGUI::CTRL_TREE::ITEMHND hndParent, MGUI::CTRL_TREE::ITEMHND hndInsertAfter);
	#endif
		virtual int v_UpdateExtents (bool forceupdate);
		virtual bool v_UsesTransparency (const MGD::DEVICE *pDevice = 0) const;

		LAYER_REGION (const LAYER_REGION&);
		LAYER_REGION& operator= (const LAYER_REGION&);
		
		class PRIVDRAWDATA;
		friend class PRIVDRAWDATA;
		friend class DISPPARM;

		#endif
	};

//=====================================================================================================================

}	// End namespace GRE

#endif	// INC_GRE_LRREGION_H
