/**
 * \file gre/system.h
 * \brief GRE::SYSTEM class definition.
 *
 * \if NODOC
 * $Id: system.h_v 1.34 2014/06/12 12:57:36 mju Exp $
 *
 * $Log: system.h_v $
 * Revision 1.34  2014/06/12 12:57:36  mju
 * Combine methods for setting delegates to create group as need both.
 *
 * Revision 1.33  2014/06/11 22:07:19  mju
 * Add delegate to end group creation.
 *
 * Revision 1.32  2014/06/09 22:06:09  mju
 * Remove unused member.
 *
 * Revision 1.31  2014/06/09 17:26:00  mju
 * Add createGroup and setDelegateCreateGroup.
 *
 * Revision 1.30  2014/05/14 14:19:50  mju
 * Remove legacy region manager.
 *
 * Revision 1.29  2013/02/15 21:32:20  mju
 * Change v_destroy.
 *
 * Revision 1.28  2013/02/14 22:43:48  mju
 * Use container base class.
 *
 * Revision 1.27  2012/01/27 22:27:21  mju
 * Remove unused parm from initX.
 *
 * Revision 1.25  2011/10/25 20:51:49  mju
 * Add getLog and setLogOwnFile.
 *
 * Revision 1.24  2011/10/20 13:36:07  mju
 * Remove win32-native stuff.
 *
 * Revision 1.22  2008/05/05 20:53:09  mju
 * Add threadpool for web.
 *
 * Revision 1.20  2008/03/20 13:14:23  mju
 * Add getThreadPool.
 *
 * Revision 1.19  2008/02/01 15:02:59  mju
 * Add setting for whether process has own customization settings.
 *
 * Revision 1.18  2007/02/05 14:23:32  mju
 * Use mithread namespace on criticalsection.
 *
 * Revision 1.15  2006/10/10 13:47:27  mju
 * Use miconstruct.
 *
 * Revision 1.14  2006/09/21 21:59:25  mju
 * Use rvc objitem hdr as need for objitemlist until can forward-declare.
 *
 * Revision 1.13  2006/09/18 20:04:03  mju
 * Fwd declare rvc:objitemlist.
 *
 * Revision 1.10  2005/12/08 19:05:35  mju
 * Add setGlobalIniGroup.
 *
 * Revision 1.7  2005/09/16 15:03:15  mju
 * Make setActiveView/Viewable non-inline.
 *
 * Revision 1.6  2005/09/16 14:43:06  mju
 * Add get/setActiveView/Viewable methods.
 *
 * Revision 1.5  2005/05/13 14:38:05  scowan
 * Moved global lists to here.
 *
 * Revision 1.4  2004/12/16 14:58:33  mju
 * Add GetAutoOpenList.
 *
 * Revision 1.1  2003/06/16 19:40:12  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_GRE_SYSTEM_H
#define	INC_GRE_SYSTEM_H

#ifndef	INC_GRE_CONTAINER_H
	#include <gre/container.h>
#endif

#ifndef	INC_MI32_CRITSEC_H
	#include <mi32/critsec.h>
#endif

#ifndef	INC_MI32_SIMPLEAR_H
	#include <mi32/simplear.h>
#endif


#ifndef	GENERATING_DOXYGEN_OUTPUT
class MILOG;
namespace RVC {
	class OBJITEMLIST;
	}
namespace MITHREAD {
	class POOL;
	}
#endif


namespace GRE {
//===================================================================================================================

//! GRE "system" container object.
//!
//! A single instance of this object is automatically created by the GRE system.  This
//! object is used as a container for all other top-level GRE::VIEWABLE objects.
//! This provides a convenient place to attach callbacks where notification is desired
//! for all layouts and top-level groups.
class SYSTEM : public GRE::CONTAINER {
	public:

		enum AUTOOPEN {
			AUTOOPEN_None,
			AUTOOPEN_Layout,
			AUTOOPEN_Group,
			AUTOOPEN_Layer
			};

		//! Create group using previously set delegate.
		//! Typically called in UI handler for layer creation.
		static GROUP* CreateGroup ()
			{ if (GetRef().m_DelegateCreateGroup) return (GetRef().m_DelegateCreateGroup()); else return (NULL); }

		//! Create group using previously set delegate.
		//! Typically called in UI handler for layer creation.
		static void CreateGroupFinish (GROUP *group)
			{ if (GetRef().m_DelegateCreateGroupFinish) GetRef().m_DelegateCreateGroupFinish(group); }

		//! Get active group based on type of active 'viewable'.
		//! @return Pointer to active group, NULL if none.
		static GRE::GROUP* GetActiveGroup ();

		//! Get active layer based on active viewable/group.
		//! @return Pointer to active layer, NULL if none.
		static LAYER* GetActiveLayer ()
			{ return (static_cast<CONTAINER&>(GetRef()).GetActiveLayer()); }

		//! Get active layout.
		//! @return Pointer to active layout, NULL if active viewable is not a layout.
		static GRE::LAYOUT* GetActiveLayout ();

		//! Get active view.
		static GRE::VIEW* GetActiveView (
			) { return (GetRef().m_ActiveView); }

		//! Get active viewable.
		static GRE::VIEWABLE* GetActiveViewable ();

		//! Get list of objects to auto-open via command line.
		//! @return Type of auto-open to be performed.
		static AUTOOPEN GetAutoOpenList (
			MDLGPARENT dlgparent,								//!< Parent for dialog if need to prompt user for anything
			const SIMPLE_ARRAY<const char*>& cmdparms,	//!< Command-line parameters
			RVC::OBJITEMLIST& ObjItemList						//!< Object item list returned
			);
			
		//! Get first view in GRE system.
		//! To access this data you should get a lock on the layout critical section
		static GRE::VIEW* GetFirstView ()
			{ return (GetRef().m_FirstView); }

		//! Get global INI group for use when saving/restoring global settings.
		//! Note return value will be NULL if for this process only.
		static const char* GetGlobalIniGroup ()
			{ return (GetRef().m_GlobalIniGroup); }

		//! Get reference to the 'standalone' group list critical section
		static MITHREAD::CRITICALSECTION& AccessCriticalSection ()
			{ return (GetRef().m_CritSec); }

		//! Get log reference.
		static MILOG& GetLog ()
			{ return (GetRef().PrivGetLog()); }

		//! Get reference to the single GRE::SYSTEM instance.
		static SYSTEM& GetRef ();

		//! Determine whether to return or continue on serial read error.
		static bool GetReturnSerialReadErr (
			) { return (GetRef().m_ReturnSerialReadErr); }

		//! Get thread pool for running background web jobs.
		static MITHREAD::POOL& GetThreadPoolWeb ()
			{ return (GetRef().PrivGetThreadPoolWeb()); }

		//! Check if have displayed warning about mixing georef/ungeoref layers.
		static bool HasDoneWarningGeoMixed (
			) { return (GetRef().m_DoneWarningGeoMixed); }

		//! Check if process set to have own customization settings.
		static bool HasOwnCustomization ()
			{ return (GetRef().m_HasOwnCustomization); }

		//! Initialize GRE system for non-interactive use.
		//! This will automatically register the standard geospatial layer types.
		//! At program exit, GRE::SYSTEM::Stop() should be called for proper cleanup.
		static void Init ();

	#ifdef X_NATIVE
		//! Initialize GRE system for interactive use in X Windows.
		//! This will automatically register the standard geospatial layer types.
		//! At program exit, GRE::SYSTEM::StopX() should be called for proper cleanup.
		//! This method is only available when compiling for X.
		static void InitX ();
	#endif

		//! Register layout-specific layer types (text, scalebar, legend).
		//! By default only the standard geospatial layer are registered.
		static void RegisterLayoutLayerTypes ();

		//! Set active view.
		//! Will also set active viewable to view's object if view is not NULL.
		static void SetActiveView (
			GRE::VIEW *view								//!< View to set
			) { GetRef().PrivSetActiveView(view); }

		//! Set active viewable.
		//! Will set active view to NULL if viewable is NULL or if active view not for viewable.
		static void SetActiveViewable (
			GRE::VIEWABLE *viewable					//!< Viewable to set
			) { GetRef().PrivSetActiveViewable(viewable); }

		//! Set delegates to call to create a new group.
		//! Delegates invoked by CreateGroup and CreateGroupFinish methods.
		static void SetDelegatesCreateGroup (
			DELEGATE_GROUP delegateCreate,
			DELEGATE_VOID_GROUP delegateFinish
			) { GetRef().m_DelegateCreateGroup = delegateCreate; GetRef().m_DelegateCreateGroupFinish = delegateFinish; }

		//! Set whether have displayed warning about mixing georef/ungeoref layers.
		//!
		//! Normally a warning dialog is displayed the first time a user attempts to place
		//! both georeferenced and non-georeferenced spatial objects in the same GRE::GROUP.
		//! In some processes, such as georeference editing, this is necessary, so this
		//! method is provided to avoid the unwanted warning dialog.
		static void SetDoneWarningGeoMixed (
			bool set = true
			) { GetRef().m_DoneWarningGeoMixed = set; }

		//! Set first view in GRE system.
		//! To access this data you should get a lock on the layout critical section
		static void SetFirstView (
			GRE::VIEW* view
			) { GetRef().m_FirstView = view; }

		//! Set global INI group for use when saving/restoring global settings.
		//! By default, the INI group will be "display-common".
		static void SetGlobalIniGroup (
			const char *IniGroup				//!< INI group name, NULL to use process name
			) { GetRef().PrivSetGlobalIniGroup(IniGroup); }

		//! Set log (if extended logging is on) to use process instance specific file.
		//! By default logging is to shared session log file.
		static void SetLogOwnFile (
			bool LogOwnFile = true
			) { GetRef().PrivSetLogOwnFile(LogOwnFile); }

		//! Set whether to return serial read error or continue.
		static void SetReturnSerialReadErr (
			bool set = true
			) { GetRef().m_ReturnSerialReadErr = set; };

		//! Terminate GRE system at program exit.
		//! This method should only be called if GRE::SYSTEM::Init() was used.
		static void Stop ()
			{ GetRef().PrivStop(); }

	#ifdef X_NATIVE
		//! Terminate GRE system at program exit.
		//! This method should only be called if GRE::SYSTEM::InitX() was used.
		static void StopX ();
	#endif

	#ifdef USE_MGUI
		//! Get TreeView pointer.
		static GRE::CTRL_TREEVIEW* TreeViewGetPtr ()
			{ return (GetRef().m_pTreeView); }
	#endif

	#ifdef USE_MGUI
		//! Set TreeView pointer.
		//! Used by GRE CTRL_TREEVIEW implementation, should not be called directly.
		static void TreeViewSetPtr (
			GRE::CTRL_TREEVIEW *pTreeView
			) { GetRef().m_pTreeView = pTreeView; }
	#endif

		#ifndef GENERATING_DOXYGEN_OUTPUT
		~SYSTEM ();
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		static bool s_InitDone;

	#ifdef USE_MGUI
		GRE::CTRL_TREEVIEW *m_pTreeView;
	#endif
		bool m_DoneWarningGeoMixed;
		bool m_ReturnSerialReadErr;					// Return error when loading group/layout instead of continuing
		bool m_HasOwnCustomization;
		MITHREAD::CRITICALSECTION m_CritSec;
		GRE::VIEW *m_FirstView;
		GRE::VIEW *m_ActiveView;
		char *m_GlobalIniGroup;
		MITHREAD::POOL *m_pThreadPoolWeb;			// Thread pool for web layers
		MILOG *m_pLog;
		DELEGATE_GROUP m_DelegateCreateGroup;
		DELEGATE_VOID_GROUP m_DelegateCreateGroupFinish;

		SYSTEM ();

		void PrivDestroyAllViews ();
		MILOG& PrivGetLog ();
		MITHREAD::POOL& PrivGetThreadPoolWeb ();
		void PrivSetActiveView (GRE::VIEW *view);
		void PrivSetActiveViewable (GRE::VIEWABLE *viewable);
		void PrivSetGlobalIniGroup (const char *IniGroup);
		void PrivSetLogOwnFile (bool LogOwnFile);
		void PrivStop ();

		// OBJECT overrides.
		virtual void v_Destroy (bool NotifyParent);
		virtual MICONSTRUCT* v_GetIcon () const;
		virtual const char* v_GetTypeName () const;

		SYSTEM (const SYSTEM&);
		SYSTEM& operator= (const SYSTEM&);

		#endif
	};

//===================================================================================================================
}	// End namespace GRE


#endif	// INC_GRE_SYSTEM_H
