/**
 * \file mgui/dlgwizard.h
 * \brief MGUI::DLGWIZARD class definition.
 *
 * \if NODOC
 * $Id: dlgwizard.h_v 1.14 2015/06/04 20:13:20 mju Exp $
 *
 * $Log: dlgwizard.h_v $
 * Revision 1.14  2015/06/04 20:13:20  mju
 * Redesign to simplify and avoid use of page class.
 *
 * Revision 1.1  2006/04/14 17:41:29  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_DLGWIZARD_H
#define	INC_MGUI_DLGWIZARD_H

#ifndef	INC_MGUI_SHELL_H
#include <mgui/shell.h>
#endif

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif

#ifndef	INC_MGUI_FORMBOOK_H
#include <mgui/formbook.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Base class for "wizard" dialog.
//! A "wizard" consists of a set of pages that are traversed in sequence.
//! Depending on user selections, some pages may not be traversed, however, pages
//! must be traversed in increasing order of creation.
class DLGWIZARD : public MGUI::SHELL {
	public:

		//! Dialog style settings.
		enum DLGSTYLE {
			DLGSTYLE_Default =			0x0000,		//!< Default style
			DLGSTYLE_Resizable =			0x0001,		//!< Dialog is resizable
			};

		//! Destructor.
		virtual ~DLGWIZARD () = 0;

		//! Open dialog in modal state and wait for user to dismiss via Finish/Cancel.
		//! @return 0 if OK, EUserCancel if Cancel, < 0 if error
		ERRVALUE DoModal (
			MDLGPARENT dlgparent							//!< Parent for modal dialog
			);

	protected:

		//! Special return values from v_OnPageBack/Next.
		enum PAGE {
			PAGE_Same = -2,				//!< Stay on same page
			PAGE_Next = -1,				//!< Go to next sequential page (for OnNext only)
			PAGE_Back = PAGE_Next		//!< Go to previous sequential page (for OnBack only)
			};

		//! Delegate for Back/Next/Finish handling.
		//! May return appropriate PAGE value, ID of specific page to go to, or error code.
		typedef fastdelegate::FastDelegate<ID()> DELEGATE;

		//! Constructor, initializes only.
		DLGWIZARD (
			const MISTRING& title,						//!< Title string, can use TEXTID also
			HELPID HelpID,									//!< Help ID, use HELPID__None if no 'help' available
			DLGSTYLE style = DLGSTYLE_Default		//!< Dialog style settings
			);

		//! Add page.
		//! @return Pane to use for adding controls.
		//! Should be done in v_CreateContent only.
		LAYOUT_PANE& AddPage (
			ID PageID,										//!< Unique page ID, must be >= 0
			LAYOUT_ORIENTATION orientation = LAYOUT_ORIENTATION_Vertical,
			FORM_BOOK::PAGESTYLE PageStyle = FORM_BOOK::PAGESTYLE_Default,
			int ChildSpacing = 4,
			int ExtraBorder = 0
			);

		//! Get ID of active page.
		//! @return ID of active page, -1 if none created.
		ID GetActivePageID () const
			{ return (m_Book.GetPageID(m_Book.GetActivePage())); }

		//! Send 'Cancel' notification to dialog.
		void NotifyCancel ()
			{ v_OnCancel(); }

		//! Set next / finish button status.
		void SetNextEnabled (
			bool bEnabled,						//!< Whether button is enabled or not
			bool bShowFinishNonLast = false		//!< Force to display "Finish" instead of "Next" even if not last page
			);

		//! Set value to return from dialog.
		void SetReturnValue (
			int RetValue
			) { m_RetValue = RetValue; }

		//! Set margin for root pane.
		//! This must be called before DoModal() and is usually done in the subclass constructor.
		void SetRootMargin (
			int RootMargin
			) { m_RootMargin = RootMargin; }

		//! Set spacing for root pane.
		//! This must be called before DoModal() and is usually done in the subclass constructor.
		void SetRootSpacing (
			int RootSpacing
			) { m_RootSpacing = RootSpacing; }

		// Overridables that may be called from derived classes.

		//! Called when 'Cancel' button is pressed.
		//! Derived class must call base class v_OnCancel() method AFTER performing its own processing
		//! to actually close the dialog.  In some cases the user may be given an opportunity to 
		//! verify that they intended to press the Cancel button.  If they elect not to cancel then
		//! the base class v_OnCancel() method should not be called.  This will cause the dialog to
		//! remain open and allow the user to retain the settings.
		//! Derived classes should generally declare this method as private.
		virtual void v_OnCancel ();

		//! Called when Finish button pressed.
		//! Derived class must call base class v_OnFinish() method AFTER performing its own processing
		//! to actually close the dialog.
		//! Derived classes should generally declare this method as private.
		virtual void v_OnFinish ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef DLGWIZARD THISCLASS;

		DLGSTYLE m_Style;
		ERRVALUE m_RetValue;
		int m_RootSpacing;
		int m_RootMargin;

		FORM_BOOK m_Book;
		LAYOUT_PANE m_ButtonPane;
		CTRL_PUSHBUTTON m_BtnBack;
		CTRL_PUSHBUTTON m_BtnNext;
		CTRL_PUSHBUTTON m_BtnFinish;
		CTRL_PUSHBUTTON m_BtnCancel;
		CTRL_PUSHBUTTON m_BtnHelp;
		SIMPLE_ARRAY<int> m_PageIdxStack;

		void Close (bool notify = true);

		ERRVALUE CreateDlg (MDLGPARENT dlgparent);

		void OnCmdBack ();
		void OnCmdNext ();

		DLGWIZARD (const DLGWIZARD&);
		DLGWIZARD& operator= (const DLGWIZARD&);

		// Overrides from SHELL.
		void v_Destroy ();
		void v_OnCloseRequest ();

		friend class LAYOUT_WIZARD_PAGE;
		#endif // GENERATING_DOXYGEN_OUTPUT

		// Overridables.

		//! Called when dialog gets closed.
		virtual void v_OnClose ();

		//! Called when dialog is destroyed.
		virtual void v_OnDestroy ();

		//! Called after dialog is actually opened.
		virtual void v_OnOpen ();

		//! Called when page is activated.
		virtual void v_OnPageActivate (
			ID PageID								//!< ID of page being activated
			);

		//! Called when Back button pressed.
		//! May return PAGE_Back, PAGE_Same, or error code.
		//! Default implementation returns PAGE_Back.
		virtual ID v_OnPageBack ();

		//! Called when page is deactivated.
		virtual void v_OnPageDeactivate (
			ID PageID								//!< ID of page being deactivated
			);

		//! Called when Next button pressed.
		//! May return PAGE_Next, PAGE_Same, ID of specific page to go to, or error code.
		//! Default implementation returns PAGE_Next.
		//! If an error code is returned it will be displayed and the active page will remain unchanged.
		virtual ID v_OnPageNext ();

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(DLGWIZARD::DLGSTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI


#endif	// INC_MGUI_DLGWIZARD_H
