/**
 * \file mgui/formselectfont.h
 * \brief MGUI::FORM_SELECT_FONT definitions
 *
 * \if NODOC
 * $Id: formselectfont.h_v 1.3 2014/02/13 15:22:12 mju Exp $
 *
 * $Log: formselectfont.h_v $
 * Revision 1.3  2014/02/13 15:22:12  mju
 * Return whether setSelectedFont successful.
 *
 * Revision 1.2  2014/02/12 14:52:16  mju
 * Add dft parm for getSelectedFont, add formstyle_List.
 *
 * Revision 1.1  2014/02/11 20:45:35  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORM_SELECT_FONT_H
#define	INC_MGUI_FORM_SELECT_FONT_H

#ifndef	INC_MGUI_CTRL_H
#include <mgui/ctrl.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Form to select a font.
//! Displays either as a combobox with optional label or as a list.
//! Fonts may be chosen by "family" or full name (family + face).
class FORM_SELECT_FONT : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style options.
		enum FORMSTYLE {
			FORMSTYLE_ChooseByFamily =		0x00,		//!< Choose font by "family", eg Times, Helvetica, Verdana, etc (default)
			FORMSTYLE_ChooseByFullName =	0x01,		//!< Choose by "full name" eg "Times", "Times Bold", "Times Italic", etc
			FORMSTYLE_AllowLegacy =			0x02,		//!< Allow legacy outline font selection
			FORMSTYLE_List =					0x04,		//!< Use list control instead of combobox (label ignored)
			};

		enum NAMETYPE {
			NAMETYPE_File,
			NAMETYPE_Family,
			NAMETYPE_Full
			};

		//! Default constructor.
		FORM_SELECT_FONT ();

		//! Destructor.
		virtual ~FORM_SELECT_FONT ();

		//! Create form with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			FORMSTYLE formstyle = FORMSTYLE_ChooseByFamily,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create form with label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			FORMSTYLE formstyle = FORMSTYLE_ChooseByFamily,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get label control if any.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel ();

		//! Get selected font.
		MISTRING GetSelectedFont (
			NAMETYPE nametype = NAMETYPE_File		//!< Name type
			) const;

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set selected item to item prior to current selection.
		//! Intended to be used in delegate to revert selection on failure.
		void SetSelectedPrevious (
			bool notify = false							//!< Notify delegate if selection changed
			);

		//! Set selected font.
		//! @return true if specified font is in list, false if not
		bool SetSelectedFont (
			const MISTRING& fontname,					//!< Font name, may be file, family or full name
			bool notify = true							//!< Notify delegate if selection changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_SELECT_FONT (const FORM_SELECT_FONT&);
		FORM_SELECT_FONT& operator= (const FORM_SELECT_FONT&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FORM_SELECT_FONT::FORMSTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORM_SELECT_FONT_H
