/**
 * \file mgui/formselectunit.h
 * \brief MGUI::FORM_SELECT_UNIT definitions
 *
 * \if NODOC
 * $Id: formselectunit.h_v 1.2 2014/08/08 21:39:24 bcolombe Exp $
 *
 * $Log: formselectunit.h_v $
 * Revision 1.2  2014/08/08 21:39:24  bcolombe
 * Add InsertItem
 *
 * Revision 1.1  2013/04/11 20:47:49  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MGUI_FORM_SELECT_UNIT_H
#define	INC_MGUI_FORM_SELECT_UNIT_H

#ifndef	INC_MGUI_COMBOBOX_H
#include <mgui/combobox.h>
#endif

#ifndef	INC_MI32_UOM_H
#include <mi32/uom.h>
#endif


namespace MGUI {

//=====================================================================================================================

//! Form to select a measurement unit.
//! Provides option for user to add non-typical items to the list.
class FORM_SELECT_UNIT : public MGUI::FORM_COMPOSITE {
	public:

		//! Form style options.
		enum FORMSTYLE {
			FORMSTYLE_NamePlural =			0,		//!< Show plural names (meters, feet, etc), default, cannot combine with NameSingular/Symbol
			FORMSTYLE_NameSingular =	0x01,		//!< Show singular names (meter, foot, etc), cannot combine with NamePlural/Symbol
			FORMSTYLE_Symbol =			0x02,		//!< Show unit symbols (abbreviations), cannot combine with NamePlural/NameSingular
			};

		//! Default constructor.
		FORM_SELECT_UNIT ();

		//! Destructor.
		virtual ~FORM_SELECT_UNIT ();

		//! Add default items to list.
		void AddDefaultItems (
			UOM::USAGE usage,								//!< Unit usage for initial list population
			const char *IniGroup,						//!< Group entry in INI file for keeping track of which items appear in list (0 for current process)
			const char *IniName,							//!< Name entry in INI file for keeping track of which items appear in list (0 to not keep track)
			const int *ExtraUnitCodes = 0,			//!< Array of additional unit codes, 0 for none
			unsigned NumExtraCodes = 0					//!< Number of additional unit codes
			);

		//! Add unit to list.
		//! Unit will not be added if it is already in the list or has a different measurement type.
		//! Will not save the list of items to keep track of.
		void AddUnit (
			const UOM::UNIT& unit						//!< Unit, must have same measurement type as used in Create
			);

		//! Add unit to list using ID code.
		//! Unit will not be added if it is already in the list.
		//! Will not save the list of items to keep track of.
		void AddUnitID (
			int unitcode									//!< Unit code
			);

		//! Create form with no label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			UOM::MEASURE::TYPE MeasureType,			//!< Measurement type
			FORMSTYLE formstyle = FORMSTYLE_NamePlural,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight
			);

		//! Create form with label.
		void Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,		//!< Parent pane
			const MISTRING& label,						//!< Label string
			UOM::MEASURE::TYPE MeasureType,			//!< Measurement type
			FORMSTYLE formstyle = FORMSTYLE_NamePlural,
			MGUI::LAYOUT_SIZEALIGN sizealign = MGUI::LAYOUT_SIZEALIGN_FixedHeight,
			MGUI::CTRL_LABEL::STYLE labelstyle = MGUI::CTRL_LABEL::STYLE_LeftNoWrap
			);

		//! Get label control if any.
		//! Usually the label control is only retrieved for alignment purposes.
		MGUI::CTRL_LABEL& GetLabel ()
			{ return (m_label); }

		//! Get selected unit.
		const UOM::UNIT GetSelectedUnit () const
			{ return (m_unit); }

		//! Get selected unit ID code.
		int GetSelectedUnitID () const;

		//! Add non-unit item to combobox
		//! ID must not conflict with any unit IDs, use >= 256
		//! Must be done after default unit items added
		void InsertItem (
			int index,
			ID id,
			const MISTRING& string
			);

		//! Set delegate to call when user makes a selection.
		void SetDelegateOnSelection (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateOnSelection = delegate; }

		//! Set selected item to item prior to current selection.
		//! Intended to be used in delegate to revert selection on failure.
		void SetSelectedPrevious (
			bool notify = false							//!< Notify delegate if selection changed
			);

		//! Set selected unit.
		//! If unit is not shown in the list it will be added, (this will not save the list of items to keep track of).
		void SetSelectedUnit (
			const UOM::UNIT& unit,						//!< Unit, must have same measurement type as used in Create
			bool notify = true							//!< Notify delegate if selection changed
			);

		//! Set selected unit ID code.
		//! If unit is not shown in the list it will be added, (this will not save the list of items to keep track of).
		void SetSelectedUnitID (
			int unitcode,									//!< Unit code
			bool notify = true							//!< Notify delegate if selection changed
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FORM_SELECT_UNIT THISCLASS;

		MGUI::CTRL_LABEL m_label;
		MGUI::CTRL_COMBOBOX m_cbx;
		DELEGATE_VOID_NOPARMS m_DelegateOnSelection;
		UOM::UNIT m_unit;
		FORMSTYLE m_formstyle;
		char *m_IniGroup;
		char *m_IniName;

		void AddUnitLow (const UOM::UNIT& unit);
		void OnCbx ();

		FORM_SELECT_UNIT (const FORM_SELECT_UNIT&);
		FORM_SELECT_UNIT& operator= (const FORM_SELECT_UNIT&);
		#endif
	};

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_FORM_SELECT_UNIT_H
