/**
 * \file mgui/graph.h
 * \brief MGUI::GRAPH definition
 *
 * \if NODOC
 * $Id: graph.h_v 1.5 2017/03/22 19:21:07 mju Exp $
 *
 * $Log: graph.h_v $
 * Revision 1.5  2017/03/22 19:21:07  mju
 * Add getGraph/ScaleColor.
 *
 * Revision 1.4  2013/08/01 21:48:03  mju
 * Add ability to set extra space at sides.
 *
 * Revision 1.3  2013/07/01 18:53:41  mju
 * Change parm order for Trans fns to put axis first.
 *
 * Revision 1.2  2013/06/27 20:16:14  mju
 * Add more basic methods.
 *
 * Revision 1.1  2013/06/24 19:15:37  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_GRAPH_H
#define	INC_MGUI_GRAPH_H

#ifndef	INC_MGUI_TOOLCANVAS_H
#include <mgui/toolcanvas.h>
#endif

#ifndef	INC_MI32_RANGE_H
#include <mi32/range.h>
#endif

namespace MGUI {

//=====================================================================================================================

//! Base/general graph control.
//! Supports axis labels, linear and logarithmic scales, scale value display with optional pointer tracking.
//! Allows adding multiple scales on either or both axes with control over which values are displayed on.
//! Base class performs drawing via a delegate.
class GRAPH : public FORM_COMPOSITE {
	public:

		typedef fastdelegate::FastDelegate<void(MGD::CONTEXT*)> DELEGATE_DRAW;

		enum AXIS {
			AXIS_X = 0,
			AXIS_Y = 1,
			};

		enum SCALESTEP {
			SCALESTEP_Linear,
			SCALESTEP_Log10,
			SCALESTEP_Log2
			};

		enum SCALESTYLE {
			SCALESTYLE_Tick_None = 			0x000000,	//!< Don't show tick for scale
			SCALESTYLE_Tick_LeftOut =		0x000001,	//!< Show tick at left, outward from frame
			SCALESTYLE_Tick_LeftIn =		0x000002,	//!< Show tick at left, inward from frame
			SCALESTYLE_Tick_BottomOut =	0x000001,	//!< Show tick at bottom, outward from frame
			SCALESTYLE_Tick_BottomIn =		0x000002,	//!< Show tick at bottom, inward from frame
			SCALESTYLE_Tick_RightOut =		0x000004,	//!< Show tick at right, outward from frame
			SCALESTYLE_Tick_RightIn =		0x000008,	//!< Show tick at right, inward from frame
			SCALESTYLE_Tick_TopOut =		0x000004,	//!< Show tick at top, outward from frame
			SCALESTYLE_Tick_TopIn =			0x000008,	//!< Show tick at top, inward from frame
			SCALESTYLE_Tick_MASK =			0x00000F,

			SCALESTYLE_Line_None =			0x000000,	//!< Don't show line within graph (default)
			SCALESTYLE_Line_Auto =			0x000010,	//!< Automatic line intervals with major/minor colors
			SCALESTYLE_Line_Step10 =		0x000020,	//!< Step lines by 10's (major interval)
			SCALESTYLE_Line_Step5 =			0x000030,	//!< Step lines by 5's (1/2 major interval)
			SCALESTYLE_Line_Step2 =			0x000040,	//!< Step lines by 2's (1/5 major interval)
			SCALESTYLE_Line_Step1 =			0x000050,	//!< Step lines by 1's (1/10 major interval)
			SCALESTYLE_Line_MASK =			0x000070,

			SCALESTYLE_Text_Tick =			0x000000,	//!< Show values on same side as ticks (default, if no ticks will have no value text)
			SCALESTYLE_Text_Left =			0x000100,	//!< Show values at left
			SCALESTYLE_Text_Bottom =		0x000100,	//!< Show values at bottom
			SCALESTYLE_Text_Right =			0x000200,	//!< Show values at right
			SCALESTYLE_Text_Top =			0x000200,	//!< Show values at top
			SCALESTYLE_Text_None =			0x000300,	//!< Don't show value text, useful when multiple graphs share a common scale
			SCALESTYLE_Text_MASK =			0x000300,

			SCALESTYLE_Value_Min =			0x001000,	//!< Show value at minimum of overall range
			SCALESTYLE_Value_Max =			0x002000,	//!< Show value at maximum of overall range
			SCALESTYLE_Value_Track =		0x004000,	//!< Show value for pointer when within graph area

			SCALESTYLE_Integer =				0x100000,	//!< Scale values are integers
			};

		enum GRAPHCOLOR {
			GRAPHCOLOR_MarginBG = 0,	//!< Background of margin area (default common background color)
			GRAPHCOLOR_GraphBG,			//!< Background of graph area (default white)
			GRAPHCOLOR_Frame,				//!< Frame around graph area (default black)
			GRAPHCOLOR_COUNT
			};

		enum SCALECOLOR {
			SCALECOLOR_Text = 0,			//!< Text (default black for index 0)
			SCALECOLOR_TickOut,			//!< Tick drawn outside frame (default black for index 0)
			SCALECOLOR_TickIn,			//!< Tick drawn inside frame (default black for index 0)
			SCALECOLOR_Axis,				//!< Axis line
			SCALECOLOR_LineMajor,		//!< Major interval line (default dark gray for index 0)
			SCALECOLOR_LineMinor,		//!< Minor interval line (default light gray for index 0)
			SCALECOLOR_COUNT
			};

		//! Dimension property.
		enum DIMENSION {
			DIMENSION_TickLengthOutside,	//!< Length of tick outside frame
			DIMENSION_TickLengthInside,	//!< Length of tick inside frame
			DIMENSION_MarginInsideFrame,	//!< Margin inside frame
			DIMENSION_TextMargin,			//!< Margin between text and tick or other text
			};

		enum SIDE {
			SIDE_Top,
			SIDE_Left,
			SIDE_Bottom,
			SIDE_Right
			};

		enum GRAPHSTYLE {
			GRAPHSTYLE_Default =				0,
			GRAPHSTYLE_DisableDraw =	0x01,	//!< Create with drawing initially disabled
			};

		//! Default constructor.
		GRAPH ();

		//! Destructor.
		virtual ~GRAPH ();

		//! Access canvas for drawing, tool creation, etc.
		TOOLCANVAS& AccessCanvas ();

		//! Add scale on specified axis.
		//! @return Scale index.
		unsigned AddScale (
			AXIS axis,									//!< Axis
			const MISTRING &label,					//!< Label for axis
			const COLOR &color,						//!< Color for text/ticks
			const DOUBLE_RANGE &range,				//!< Overall and initial visible range of values
			SCALESTYLE style,							//!< Style
			SCALESTEP step = SCALESTEP_Linear	//!< Step
			);

		//! Calculate rectangle for "extra" space on specified side.
		//! @return Rectangle, will be empty if no extra space set.
		LRECT2D CalcRectExtra (
			SIDE side									//!< Side
			) const;

		//! Create graph form.
		//! If no scale has been added for each axis, one will be created at index 0.
		void Create (
			LAYOUT_PANE_BASE &paneParent,
			int width,									//!< Width in pixels
			int height,									//!< Height in pixels
			LAYOUT_SIZEALIGN sizealign,
			GRAPHSTYLE style = GRAPHSTYLE_Default	//!< Graph style
			);

		//! Draw graph.
		void Draw ();

		//! Disable drawing.
		void DrawDisable ();

		//! Enable drawing, will draw graph if no longer disabled.
		void DrawEnable ();

		//! Draw track value for specified axis.
		//! This will not erase any previous track values, allowing multiple values to be displayed.
		//! Use RestoreFromBuffer to erase all tracking values. this is done automatically whenever the pointer moves.
		void DrawTrackTickAndText (
			AXIS axis,									//!< Axis
			double value,								//!< Object value on specified axix
			const COLOR *pColor = 0,				//!< Override color, if NULL will use specified scale colors
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Get color for graph element.
		const COLOR& GetGraphColor (
			GRAPHCOLOR graphColor					//!< Color value
			) const;

		//! Get rectangle on canvas used to draw graph.
		const LRECT2D& GetGraphDeviceRect () const;

		//! Get overall range for specified axis and scale index.
		const DOUBLE_RANGE& GetRange (
			AXIS axis,									//!< Axis
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Get visible range for specified axis and scale index.
		const DOUBLE_RANGE& GetRangeVisible (
			AXIS axis,									//!< Axis
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Get color for scale element.
		const COLOR& GetScaleColor (
			AXIS axis,									//!< Axis
			SCALECOLOR scaleColor,					//!< Color value
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Get step for axis/scale.
		SCALESTEP GetScaleStep (
			AXIS axis,									//!< Axis
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Get style for axis/scale.
		SCALESTYLE GetScaleStyle (
			AXIS axis,									//!< Axis
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Get amount of "extra" space set for side.
		//! @return Space in pixels.
		UINT16 GetSpaceExtra (
			SIDE side									//!< Side
			) const;

		//! Determine if drawing is disabled.
		bool IsDrawDisabled () const;

		//! Restore graph from buffer, erasing any tracking values shown.
		void RestoreFromBuffer ();

		//! Set delegate to call to draw data in graph.
		void SetDelegateDraw (
			DELEGATE_DRAW delegate
			) { m_DelegateDraw = delegate; }

		//! Set delegate to call for updating data in graph.
		void SetDelegateUpdate (
			DELEGATE_VOID_NOPARMS delegate
			) { m_DelegateUpdate = delegate; }

		//! Set graph color.
		void SetColor (
			GRAPHCOLOR graphColor,					//!< Color value
			const COLOR &color						//!< Color to set
			);

		//! Set dimension value.
		void SetDimension (
			DIMENSION dimension,						//!< Dimension to set
			INT16 value									//!< New value
			);

		//! Set label for specified axis and scale index.
		void SetLabel (
			AXIS axis,									//!< Axis
			const MISTRING &label,					//!< Label for axis
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set range for specified axis and scale index.
		void SetRange (
			AXIS axis,									//!< Axis
			const DOUBLE_RANGE &range,				//!< Overall and initial visible range
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set range for scale index 0 on both axes.
		void SetRange (
			const DRECT2D &range						//!< Overall and initial visible ranges
			);

		//! Set range of visible values for specified axis and scale index.
		//! By default, visible and overall ranges are nominally the same
		void SetRangeVisible (
			AXIS axis,									//!< Axis
			const DOUBLE_RANGE &range,				//!< Visible range
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set scale color.
		void SetScaleColor (
			AXIS axis,									//!< Axis
			SCALECOLOR scaleColor,					//!< Color value
			const COLOR &color,						//!< Color to set
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set scale step.
		void SetScaleStep (
			AXIS axis,									//!< Axis
			SCALESTEP step,							//!< Step
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set scale style values.
		void SetScaleStyle (
			AXIS axis,									//!< Axis
			SCALESTYLE style,							//!< Style
			unsigned scaleidx = 0					//!< Scale index
			);

		//! Set amount of "extra" space for custom drawing on specified side.
		void SetSpaceExtra (
			SIDE side,									//!< Side
			UINT16 space								//!< Space in pixels
			);

		//! Translate device coordinates to object coordinates.
		//! @return true if all values in visible range, false if not
		bool TransDeviceToObject (
			AXIS axis,									//!< Axis
			double *values,							//!< Values passed/returned
			UINT32 numValues = 1,					//!< Number of values
			int step = 1,								//!< Step between values (for multi-dimensional arrays)
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Translate object coordinates to device coordinates.
		//! @return true if all values in visible range, false if not
		bool TransObjectToDevice (
			AXIS axis,									//!< Axis
			double *values,							//!< Values passed/returned
			UINT32 numValues = 1,					//!< Number of values
			int step = 1,								//!< Step between values (for multi-dimensional arrays)
			unsigned scaleidx = 0					//!< Scale index
			) const;

		//! Translate object coordinates to device coordinates.
		//! @return true if all values in visible range, false if not
		bool TransObjectToDevice (
			POLYLINE &polyline						//!< Polyline passed/returned
			) const;

		//! Update data in graph.
		//! Calls v_Update followed by Draw.
		void Update ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;
		DELEGATE_DRAW m_DelegateDraw;
		DELEGATE_VOID_NOPARMS m_DelegateUpdate;
		#endif

		//! Draw data in graph.
		//! Default implementation calls delegate.
		virtual void v_Draw (MGD::CONTEXT *gc);

		//! Update data in graph.
		//! Default implementation calls delegate.
		virtual void v_Update ();
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GRAPH::GRAPHSTYLE)
DEFINE_ENUM_OP_BITWISE(GRAPH::SCALESTYLE)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_GRAPH_H
