/**
 * \file mgui/graphhistogram.h
 * \brief MGUI::GRAPH_HISTOGRAM definition
 *
 * \if NODOC
 * $Id: graphhistogram.h_v 1.6 2013/10/31 21:46:09 mju Exp $
 *
 * $Log: graphhistogram.h_v $
 * Revision 1.6  2013/10/31 21:46:09  mju
 * Add mode enum and get/setMode.
 *
 * Revision 1.5  2013/08/02 17:59:29  mju
 * Add delegate to get label.
 *
 * Revision 1.4  2013/08/01 21:48:21  mju
 * Add stat value display methods.
 *
 * Revision 1.3  2013/07/01 19:23:57  mju
 * Add setDelegateGetColorLabel.
 *
 * Revision 1.2  2013/07/01 18:53:57  mju
 * Add onPointerMove.
 *
 * Revision 1.1  2013/06/27 20:16:36  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MGUI_GRAPHHISTOGRAM_H
#define	INC_MGUI_GRAPHHISTOGRAM_H

#ifndef	INC_MGUI_GRAPH_H
#include <mgui/graph.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class STAT_UNIVARIATE;
#endif

namespace MGUI {

//=====================================================================================================================

//! Graph display of one or more histograms from univariate statistics.
//! Access to the actual statistics and colors is done via delegates, this control does not
//! retain the statistics itself. The SetNumItems() method must be called to set the number
//! of histograms to graph.
class GRAPH_HISTOGRAM : public GRAPH {
	public:

		//! Delegate to get statistics for 0-based item index.
		typedef fastdelegate::FastDelegate<const STAT_UNIVARIATE&(UINT32 idx)> DELEGATE_GETSTATS;

		//! Delegate to get color for 0-based item index.
		typedef fastdelegate::FastDelegate<COLOR(UINT32 idx)> DELEGATE_GETCOLOR;

		//! Delegate to get label for 0-based item index.
		typedef fastdelegate::FastDelegate<MISTRING(UINT32 idx)> DELEGATE_GETLABEL;

		enum STAT {
			STAT_None =		0,
			STAT_Median =	0x01,
			STAT_Mean =		0x02,
			STAT_StdDev =	0x04,
			STAT_Count =	0x08,
			STAT_All =		0x0F
			};

		enum MODE {
			MODE_Default =			0,
			MODE_Cumulative =		0x01,
			};

		//! Constructor.
		GRAPH_HISTOGRAM ();

		//! Destructor.
		virtual ~GRAPH_HISTOGRAM ();

		//! Get display mode flags.
		MODE GetMode () const
			{ return (m_Mode); }

		//! Get which statistic values to show.
		STAT GetStatValuesShown () const
			{ return (m_StatValuesShown); }

		//! Set delegate to call to get color for graph.
		//! If no delegate is specified drawing will be in black.
		void SetDelegateGetColorGraph (
			DELEGATE_GETCOLOR delegate
			) { m_DelegateGetColorGraph = delegate; }

		//! Set delegate to call to get color for label.
		//! If no delegate is specified drawing will be in black.
		void SetDelegateGetColorLabel (
			DELEGATE_GETCOLOR delegate
			) { m_DelegateGetColorLabel = delegate; }

		//! Set delegate to call to get label.
		void SetDelegateGetLabel (
			DELEGATE_GETLABEL delegate
			) { m_DelegateGetLabel = delegate; }

		//! Set delegate to call to get stats.
		//! No drawing will be performed unless a delegate is specified.
		void SetDelegateGetStats (
			DELEGATE_GETSTATS delegate
			) { m_DelegateGetStats = delegate; }

		//! Set display mode flags.
		void SetMode (
			MODE mode
			);

		//! Set number of statistic items and update graph.
		void SetNumItems (
			UINT32 NumItems					//!< Number of histograms to display
			);

		//! Set which statistic values to show.
		//! Values will be shown below graph.
		void SetStatValuesShown (
			STAT stats							//!< Statistics to show
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef GRAPH BASECLASS;
		typedef GRAPH_HISTOGRAM THISCLASS;

		DELEGATE_GETSTATS m_DelegateGetStats;
		DELEGATE_GETCOLOR m_DelegateGetColorGraph;
		DELEGATE_GETCOLOR m_DelegateGetColorLabel;
		DELEGATE_GETLABEL m_DelegateGetLabel;
		UINT32 m_NumItems;
		STAT m_StatValuesShown;
		MODE m_Mode;

		bool OnPointerMove (MGUI::POINT ptCanvas, POINTERBUTTON button, KEYSTATE keystate, bool bHasMore);

		// GRAPH overrides.
		virtual void v_Draw (MGD::CONTEXT *gc);
		virtual void v_Update ();
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GRAPH_HISTOGRAM::MODE)
DEFINE_ENUM_OP_BITWISE(GRAPH_HISTOGRAM::STAT)
#endif

//=====================================================================================================================

}	// End namespace MGUI

#endif	// INC_MGUI_GRAPHHISTOGRAM_H
