/**
 * \file colorspc.h <mi32/colorspc.h>
 * \brief Definitions colorspaces
 *
 * \if NODOC
 * $Id: colorspc.h_v 1.23 2015/05/27 20:17:45 mju Exp $
 *
 * $Log: colorspc.h_v $
 * Revision 1.23  2015/05/27 20:17:45  mju
 * Remove hsv/munsell model.
 *
 * Revision 1.22  2014/01/03 21:17:59  mju
 * Remove unused defns.
 *
 * Revision 1.19  2007/01/16 22:30:01  mju
 * Add conversions to/from COLOR.
 *
 * Revision 1.18  2006/12/13 17:29:34  mju
 * Add convert fns to/from HBS.
 *
 * Revision 1.14  2006/01/12 18:13:22  mju
 * Add HIS/RGB methods using doubles and explicit RGB range.
 *
 * Revision 1.11  2003/05/20 15:53:28  dwilliss
 * Added Indexed4 colorspace
 *
 * Revision 1.10  2003/05/16 20:09:02  dwilliss
 * Added 3 new colorspaces for 16 bit componants
 *
 * Revision 1.8  2002/10/17 15:21:32  mju
 * Move COLORSPACE enum here, rename some items therein to be more specific.
 * Add comments describing colorspace enum.
 *
 * Revision 1.1  1999/04/19  17:36:37  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_COLORSPC_H
#define	INC_MI32_COLORSPC_H

#ifndef	INC_MI32_STDDEFNS_H
#include <mi32/stddefns.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct COLOR;
#endif // GENERATING_DOXYGEN_OUTPUT

//===================================================================================================================
//! Color space enumeration, used to described how image/color data is stored in a memory buffer.
//! Some colorspace values such as those having 'alpha' component may be treated identically to
//! the similar values having a 'skip' component in the same position.  In addition, some processes
//! may treat gray and indexed colorspaces identically.
//!
//! In "unpacked" colorspaces, each component is accessed sequentially in the data
//! using data elements of the size (in bits) specified.  For components of more
//! than 8 bits this is assumed to be in the NATIVE byte order of the machine and
//! thus byte swapping may need to be performed when accessing from external sources.
//!
//! In "packed" colorspaces, more than one pixel or component is 'packed' into a data
//! element.  The number after 'Packed' indicates the size of each data element in bits.
//! For elements larger than 8 bits this is assumed to be in the NATIVE byte order of
//! the machine and thus byte swapping may need to be performed when accessing data.
//! When multiple components are packed into a data element they are listed in order
//! from most-significant bits to least.
enum COLORSPACE {
	COLORSPACE_Unknown = 				0,

	// Unpacked colorspaces.

	COLORSPACE_Gray8 =					3,			//!< Grayscale 1 byte/pixel
	COLORSPACE_Gray16 =					4,			//!< Grayscale 2 bytes/pixel
	COLORSPACE_Gray32 =					5,			//!< Grayscale 4 bytes/pixel
// COLORSPACE_Indexed4 below (36)
	COLORSPACE_Indexed8 =				6,			//!< Indexed 1 byte/pixel
	COLORSPACE_Indexed16 =				7,			//!< Indexed 2 bytes/pixel
	COLORSPACE_R8_G8_B8 =				8,			//!< Red, Green, Blue, 1 byte each
	COLORSPACE_B8_G8_R8 =				9,			//!< Blue, Green, Red, 1 byte each
	COLORSPACE_R8_G8_B8_X8 =			16,		//!< Red, Green, Blue, Skip, 1 byte each
	COLORSPACE_X8_R8_G8_B8 =			17,		//!< Skip, Red, Green, Blue, 1 byte each
	COLORSPACE_B8_G8_R8_X8 =			18,		//!< Blue, Green, Red, Skip, 1 byte each
	COLORSPACE_X8_B8_G8_R8 =			19,		//!< Skip, Blue, Green, Red, 1 byte each
	COLORSPACE_R8_G8_B8_A8 =			20,		//!< Red, Green, Blue, Alpha, 1 byte each
	COLORSPACE_A8_R8_G8_B8 =			21,		//!< Alpha, Red, Green, Blue, 1 byte each
	COLORSPACE_B8_G8_R8_A8 =			22,		//!< Blue, Green, Red, Alpha, 1 byte each
	COLORSPACE_A8_B8_G8_R8 =			23,		//!< Alpha, Blue, Green, Red, 1 byte each

	COLORSPACE_C8_M8_Y8 =				31,		//!< Cyan, Magenta, Yellow, 1 byte each
	COLORSPACE_C8_M8_Y8_X8 =			32,		//!< Cyan, Magenta, Yellow, Skip, 1 byte each
	COLORSPACE_C8_M8_Y8_K8 =			33,		//!< Cyan, Magenta, Yellow, Black, 1 byte each
	COLORSPACE_X8_C8_M8_Y8 =			34,		//!< Skip, Cyan, Magenta, Yellow, 1 byte each
	COLORSPACE_K8_C8_M8_Y8 =			35,		//!< Black, Cyan, Magenta, Yellow, 1 byte each
	COLORSPACE_Indexed4 = 				36,		//!< 4-bits per pixel colormapped

	// Packed colorspaces.

	COLORSPACE_Packed8_Binary_LowFirst =		101,		//!< 1 bit/pixel, low bit first along image line
	COLORSPACE_Packed8_Binary_HighFirst =		102,		//!< 1 bit/pixel, high bit first along image line
	COLORSPACE_Packed8_Gray4_LowFirst =			103,		//!< 4 bits/pixel, low nibble first along image line
	COLORSPACE_Packed8_Gray4_HighFirst =		104,		//!< 4 bits/pixel, high nibble first along image line
	COLORSPACE_Packed8_Indexed4_LowFirst =		105,		//!< Indexed 4 bits/pixel, low nibble first along image line
	COLORSPACE_Packed8_Indexed4_HighFirst =	106,		//!< Indexed 4 bits/pixel, high nibble first along image line

	COLORSPACE_Packed16_X1_R5_G5_B5 =			201,		//!< 0RRRRRGGGGGBBBBB
	COLORSPACE_Packed16_X1_B5_G5_R5 =			202,		//!< 0BBBBBGGGGGRRRRR

	COLORSPACE_R16_G16_B16	=						300,		//!< sRGB as 16-bit per sample 
	COLORSPACE_X16_Y16_Z16	=						301,		//!< CIE XYZ as 16-bit per sample fixed point
	COLORSPACE_L16_a16_b16	=						302		//!< CIE Lab as 16-bit per sample fixed point
		
	};	

//===================================================================================================================

//----------------------------------------------------------------------------
//!		Function prototypes
//----------------------------------------------------------------------------

#if defined(__cplusplus)
extern "C" {
#endif

//! Convert Hue-Brightness-Saturation to COLOR.
//! @return true if values are valid, false if not
bool ConvertHBStoColor (
	double hue,					//!< Hue 0-360
	double brightness,		//!< Brightness 0-100
	double saturation,		//!< Saturation 0-100
	COLOR& color				//!< Color returned
	);

//! Convert Hue-Brightness-Saturation to Red-Green-Blue.
//! @return true if values are valid, false if not
bool ConvertHBStoRGB (
	double hue,					//!< Hue 0-360
	double brightness,		//!< Brightness 0-100
	double saturation,		//!< Saturation 0-100
	double MaxRGB,				//!< Range for RGB values
	double& red,				//!< Red returned
	double& green,				//!< Green returned
	double& blue				//!< Blue returned
	);

//! Convert Hue-Intensity-Saturation to COLOR.
//! @return true if values are valid, false if not
bool ConvertHIStoColor (
	double hue,					//!< Hue 0-360
	double intensity,			//!< Intensity 0-100
	double saturation,		//!< Saturation 0-100
	COLOR& color				//!< Color returned
	);

//! Convert Hue-Intensity-Saturation to Red-Green-Blue.
//! @return true if values are valid, false if not
bool ConvertHIStoRGB (
	double hue,					//!< Hue 0-360
	double intensity,			//!< Intensity 0-100
	double saturation,		//!< Saturation 0-100
	double MaxRGB,				//!< Range for RGB values
	double& red,				//!< Red returned
	double& green,				//!< Green returned
	double& blue				//!< Blue returned
	);

//! Convert COLOR to Hue-Brightness-Saturation.
//! @return true if values are valid, false if not
bool ConvertColorToHBS (
	const COLOR& color,		//!< Color to convert
	double& hue,				//!< Hue 0-360 returned
	double& brightness,		//!< Brightness 0-100 returned
	double& saturation		//!< Saturation 0=100 returned
	);

//! Convert COLOR to Hue-Intensity-Saturation.
//! @return true if values are valid, false if not
bool ConvertColorToHIS (
	const COLOR& color,		//!< Color to convert
	double& hue,				//!< Hue 0-360 returned
	double& intensity,		//!< Intensity 0-100 returned
	double& saturation		//!< Saturation 0=100 returned
	);

//! Convert Red-Green-Blue to Hue-Brightness-Saturation.
//! @return true if values are valid, false if not
bool ConvertRGBtoHBS (
	double MaxRGB,				//!< Range for RGB values
	double red,					//!< Red 0-MaxRGB
	double green,				//!< Green 0-MaxRGB
	double blue,				//!< Blue 0-MaxRGB
	double& hue,				//!< Hue 0-360 returned
	double& brightness,		//!< Brightness 0-100 returned
	double& saturation		//!< Saturation 0=100 returned
	);

//! Convert Red-Green-Blue to Hue-Intensity-Saturation.
//! @return true if values are valid, false if not
bool ConvertRGBtoHIS (
	double MaxRGB,				//!< Range for RGB values
	double red,					//!< Red 0-MaxRGB
	double green,				//!< Green 0-MaxRGB
	double blue,				//!< Blue 0-MaxRGB
	double& hue,				//!< Hue 0-360 returned
	double& intensity,		//!< Intensity 0-100 returned
	double& saturation		//!< Saturation 0=100 returned
	);

#if defined(__cplusplus)
}
#endif

#endif	// INC_MI32_COLORSPC_H 
