/**
 * \file mi32/elemnumtrans.h
 * \brief ELEMNUMTRANS definitions
 *
 * \if NODOC
 * $Id: elemnumtrans.h_v 1.8 2015/12/08 21:48:53 mju Exp $
 *
 * $Log: elemnumtrans.h_v $
 * Revision 1.8  2015/12/08 21:48:53  mju
 * Add checkDuplicate for addItem.
 *
 * Revision 1.7  2015/05/18 13:30:52  mju
 * No default for keepExisting on resize.
 *
 * Revision 1.6  2015/05/12 21:42:25  mju
 * Add assign from array.
 *
 * Revision 1.5  2015/05/11 19:01:34  mju
 * Add keepExisting parm to resize.
 *
 * Revision 1.4  2015/05/08 19:04:00  mju
 * Support identity transformation, calcInverse, maxTgtNum.
 *
 * Revision 1.3  2015/05/07 16:14:03  mju
 * Allow single or multiple targets.
 *
 * Revision 1.2  2015/05/07 14:51:43  mju
 * Move elemnumset to own hdr.
 *
 * Revision 1.1  2015/05/07 14:06:31  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_ELEMNUMTRANS_H
#define	INC_MI32_ELEMNUMTRANS_H

#ifndef	INC_MI32_ELEMNUMSET_H
#include <mi32/elemnumset.h>
#endif

//=====================================================================================================================

//! Container for element number translation to new element number(s).
//! Allows for identity transformation, or single or multiple target element numbers for each source element.
//! Uses ELEMNUMSET for efficient storage of single element when multiple targets are allowed.
class ELEMNUMTRANS {
	public:

		//! Default constructor.
		ELEMNUMTRANS ();

		//! Copy constructor.
		ELEMNUMTRANS (
			const ELEMNUMTRANS &rhs
			);

		//! Destructor.
		~ELEMNUMTRANS ();

		//! Assignment.
		ELEMNUMTRANS& operator= (
			const ELEMNUMTRANS &rhs
			);

		//! Add translation item.
		//! Will automatically resize if necessary.
		ERRVALUE AddItem (
			ELEMNUM SrcNum,					//!< Source element number
			ELEMNUM TgtNum,					//!< Target element number
			bool bCheckDuplicate				//!< Check for duplicate (slower)
			);

		//! Assign from array.
		ERRVALUE Assign (
			const INT32 *SrcToTgt,			//!< Lookup array
			UINT32 NumItems					//!< Number of array items
			);

		//! Calculate inverse transformation.
		ERRVALUE CalcInverse (
			ELEMNUMTRANS &InverseTrans		//!< Inverse transformation returned
			) const;

		//! Clear translation table.
		void Clear ();

		//! Copy translation table.
		ERRVALUE CopyTo (
			ELEMNUMTRANS &tgt					//!< Destination for copy
			) const;

		//! Exchange content with another ELEMNUMTRANS.
		void Exchange (
			ELEMNUMTRANS &other
			);

		//! Get maximum target element number.
		ELEMNUM GetMaxTgtNum () const
			{ return (m_MaxTgtNum); }

		//! Get number of source items.
		UINT32 GetNumItems () const
			{ return (m_NumItems); }

		//! Get number of target items for specified source element number.
		UINT32 GetNumTgtItems (
			ELEMNUM SrcNum						//! Source element number
			) const;

		//! Get target element number.
		ELEMNUM GetTgtNum (
			ELEMNUM SrcNum,					//!< Source element number
			UINT32 idx = 0						//!< Index in set, ignored for identity/single
			) const;

		//! Determine whether has multiple sources for any target.
		bool HasMultiSrcPerTgt () const;

		//! Determine whether has multiple target elements per source.
		bool HasMultiTgtPerSrc () const
			{ return (m_TgtSets != 0); }

		//! Determine if empty.
		bool IsEmpty () const
			{ return (m_NumItems == 0); }

		//! Determine if is identity transformation.
		bool IsIdentity () const
			{ return (m_bIsIdentity); }
			
		//! Reserve space for specified number of source elements
		ERRVALUE Reserve (
			UINT32 MaxItems					//!< Maximum number of source elements
			);

		//! Resize to new number of source elements
		ERRVALUE Resize (
			UINT32 NumItems,					//!< Number of source elements
			bool bKeepExisting				//!< Whether to keep existing items
			);

		//! Set translation to identity for specified number of items.
		void SetIdentity (
			UINT32 NumItems					//!< Number of item
			);

		//! Set single target element number for specified source element number.
		ERRVALUE SetTgtNum (
			ELEMNUM SrcNum,					//!< Source element number
			ELEMNUM TgtNum						//!< Target element number
			);
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ELEMNUM m_MaxTgtNum;
		ELEMNUMSET *m_TgtSets;
		UINT32 *m_TgtNums;
		UINT32 m_NumItems;
		UINT32 m_NumAlloc;
		bool m_bIsIdentity;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#endif	// INC_MI32_ELEMNUMTRANS_H
