/**
 * \file mi32/fft.h
 * \brief FFT (Fast Fourier Transform) class
 *
 * \if NODOC
 * $Id: fft.h_v 1.4 2015/09/16 21:33:43 mju Exp $
 *
 * $Log: fft.h_v $
 * Revision 1.4  2015/09/16 21:33:43  mju
 * Add CtoR mode for Real vs Magnitude.
 *
 * Revision 1.3  2015/09/09 14:19:20  mju
 * Add calcNext/PreviousFastSize.
 *
 * Revision 1.2  2015/09/08 19:22:43  mju
 * Add mode member.
 *
 * Revision 1.1  2015/09/03 16:36:22  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_FFT_H
#define	INC_MI32_FFT_H

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_COMPLEX
#define	INC_COMPLEX
	#include <complex>
#endif

typedef std::complex<double> DCOMPLEX;

//=====================================================================================================================

//! Fast Fourier Transform processor.
//! To use, first call the InitializeXd() method appropriate for number of dimensions.
//! Then call the corresponding Execute... method.
//!
//! Notes:
//! The half-complex modes require that the (last) dimension be even.
//! The MODE_RtoC_Full mode with an odd (last) dimension and the MODE_CtoR_Full mode will take
//! at least twice the time of the half-complex modes and will require a temporary "complex"
//! buffer with the same dimensions as the input.
//! The DC level will be at index 0 in all dimensions. When viewing in frequency domain it is common
//! to shift this to the center. There are various techniques for doing this, such as flipping the signs of the
//! input values in an alternating checkerboard pattern, or shifting the output result.
class FFT {
	public:

		//! Operation mode.
		enum MODE {
			MODE_None =				0,
			MODE_CtoC_Forward =	1,			//!< Complex to Complex Forward transformation
			MODE_CtoC_Inverse =	2,			//!< Complex to Complex Inverse transformation
			MODE_RtoC_Half =		3,			//!< Real to Half Complex (forward) transformation (requires last dimension to be even)
			MODE_RtoC_Full =		4,			//!< Real to Full Complex (forward) transformation
			MODE_CtoR_Half =		5,			//!< Half Complex to Real (inverse) transformation (requires last dimension to be even)
			MODE_CtoR_FullM =		6,			//!< Full Complex to Real (inverse) transformation (returns magnitude of result)
			MODE_CtoR_FullR =		7,			//!< Full Complex to Real (inverse) transformation (returns real component of result)
			};

		//! Operation flags.
		enum FLAGS {
			FLAG_None =				0,
			FLAG_Normalize =		0x10,		//!< Normalize values (divide by N) after transform
			};

		//! Calculate next size >= N that uses only fast factors.
		static UINT32 CalcNextFastSize (
			UINT32 n,							//!< Minimum size
			bool bNeedEven = false			//!< Whether must be divisible by 2
			);

		//! Calculate previous size <= N that uses only fast factors.
		static UINT32 CalcPreviousFastSize (
			UINT32 n,							//!< Maximum size
			bool bNeedEven = false			//!< Whether must be divisible by 2
			);

		//! Constructor.
		FFT ();

		//! Destructor.
		~FFT ();

		//! Execute Complex to Complex operation.
		void ExecuteCtoC (
			const DCOMPLEX *in,
			DCOMPLEX *out
			) const;

		//! Execute Complex to Real (inverse) operation.
		void ExecuteCtoR (
			const DCOMPLEX *in,
			double *out
			) const;

		//! Execute Real to Complex operation.
		void ExecuteRtoC (
			const double *in,
			DCOMPLEX *out
			) const;

		//! Free internal structures.
		void Free ();

		//! Initialize 1-dimensional operation.
		ERRVALUE Initialize1d (
			UINT32 n,					//!< Number of array elements
			MODE mode,					//!< Operation mode
			FLAGS flags	= FLAG_None	//!< Optional modifier flags
			);

		//! Initialize 2-dimensional operation
		ERRVALUE Initialize2d (
			UINT32 n0,					//!< Number of array "rows"
			UINT32 n1,					//!< Number of array "columns"
			MODE mode,					//!< Operation mode
			FLAGS flags	= FLAG_None	//!< Optional modifier flags
			);

		//! Initialize N-dimensional operation.
		ERRVALUE InitializeNd (
			const UINT32 *dims,		//!< Array dimensions, last dimension varies fastest (contiguous in memory)
			UINT32 ndims,				//!< Number of dimensions
			MODE mode,					//!< Operation mode
			FLAGS flags	= FLAG_None	//!< Optional modifier flags
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		mutable DOUBLE_ARRAY<DCOMPLEX> m_TempC;
		void *m_Cfg;
		int *m_dims;
		int m_NumDims;
		UINT32 m_NumElems;
		MODE m_Mode;
		FLAGS m_Flags;
		bool m_bFullComplexEven;
		bool m_bFullComplexForward;
		bool m_bFullComplexInverse;

		void Normalize (double *out, UINT32 NumValues) const;
		void Normalize (DCOMPLEX *out, UINT32 NumValues) const;

		// Unimplemented.
		FFT (const FFT&);
		FFT& operator= (const FFT&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FFT::FLAGS)
#endif

//=====================================================================================================================

#endif	INC_MI32_FFT_H
