/**
 * \file mi32/geofrmla.h
 * \brief GeoFormula class definitions
 *
 * \if NODOC
 * $Id: geofrmla.h_v 1.44 2014/02/28 16:48:09 mju Exp $
 *
 * $Log: geofrmla.h_v $
 * Revision 1.44  2014/02/28 16:48:09  mju
 * Add isStarted.
 *
 * Revision 1.43  2014/02/27 21:52:20  mju
 * Add dlgSelectObjectsAndCreate and evaluate using multiple buffers.
 *
 * Revision 1.42  2014/02/25 15:25:56  mju
 * Begin adding output class, remove old writeDefn fns.
 *
 * Revision 1.41  2014/02/24 17:03:07  mju
 * Add encoding for read/writeDefn.
 *
 * Revision 1.40  2014/02/21 22:12:21  mju
 * Add dlgGetObjects, setExtentsMode, extentsmode_AppDefined and other changes.
 *
 * Revision 1.39  2014/02/19 22:37:25  mju
 * Add paneToolbar for createPageScript.
 *
 * Revision 1.38  2014/02/19 18:31:09  mju
 * Update to use mgui and some restructuring.
 *
 * Revision 1.37  2014/01/03 21:30:11  mju
 * Exclude deprecated setOutputTrans if no_deprecated.
 *
 * Revision 1.21  2004/07/23 14:23:26  mju
 * Add getTransToExtCRS, remove getTransObjToMap.
 *
 * Revision 1.18  2004/06/29 14:31:44  mju
 * Add GetTransObjToMap.
 *
 * Revision 1.15  2004/04/28 15:28:04  mju
 * Add ability to disable scriptcontext updates in editor.
 *
 * Revision 1.10  2003/08/26 21:06:24  mju
 * When create formula from objects adjust duplicate names.
 *
 * Revision 1.1  2000/11/08 21:47:41  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_GEOFRMLA_H
#define	INC_MI32_GEOFRMLA_H

#ifndef	INC_RVC_TIN_H
#include <rvc/tin.h>
#endif

#ifndef	INC_RVC_VECTOR_H
#include <rvc/vector.h>
#endif

#ifndef	INC_RVC_RASTER_H
#include <rvc/raster.h>
#endif

#ifndef	INC_MI32_CONTRASTPARM_H
#include <mi32/contrastparm.h>
#endif

#ifndef	INC_MI32_SERIALIZERITEM_H
#include <mi32/serializeritem.h>
#endif



#ifndef GENERATING_DOXYGEN_OUTPUT
class RASTINTERP;
namespace SML {
	class SYMBOL;
	class EDITOR;
	class CONTEXT;
	}
namespace MGUI {
	class LAYOUT_PANE_BASE;
	class CTRL_STATUSBAR;
	class FORM_BUTTONTEXT;
	}
#endif


//! Geospatial Formula class.
class GEOFORMULA {
	public:

		//! Output type of formula.
		// Do not change the values as they are stored in serializations.
		enum OUTPUTTYPE {
			OUTPUTTYPE_Numeric =		0,		//!< Single numeric output
			OUTPUTTYPE_Color =		1,		//!< Single (separate RGB) color output
			OUTPUTTYPE_Multi =		2,		//!< Multiple outputs
			};

		//! Mode for determining extents.
		// Do not change the values as they are stored in serializations.
		enum EXTENTSMODE {
			EXTENTSMODE_UserDefined =		0,
			EXTENTSMODE_Intersection =		1,
			EXTENTSMODE_Union =				2,
			EXTENTSMODE_Region =				3,
			EXTENTSMODE_AppDefined =		4
			};

		enum SCRIPTFLAGS {
			SCRIPTFLAG_None =				0x00,
			SCRIPTFLAG_AllSymbols =		0x01,
			SCRIPTFLAG_NoParse =			0x02
			};

		//! Base for spatial object used in formula.
		class OBJECT {
			public:

				//! Copy constructor.
				OBJECT (
					const OBJECT& rhs
					);

				//! Destructor.
				virtual ~OBJECT () = 0;

				//! Assignment.
				OBJECT& operator= (
					const OBJECT& rhs
					);

				ERRVALUE AddSymbols (
					GEOFORMULA& formula,
					bool OnlyIfUsed = true
					);

				void ClearSymbols ();

				void Close ();

				const RVC::OBJITEM& GetObjItem () const
					{ return (m_objitem); }

				const FILEPATH& GetPath () const
					{ return (m_objitem.GetFilePath()); }

				const SPATREF::COORDREFSYS& GetCoordRefSys () const
					{ return (GetMapRegion().GetCoordRefSys()); }

				const REGION2D& GetMapRegion () const
					{ return (m_MapRegion); }

				const REGION2D& GetObjRegion () const
					{ return (m_ObjRegion); }

				const MISTRING& GetSymbolName () const
					{ return (m_SymbolName); }

				ERRVALUE Open ();

			protected:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				MISTRING m_SymbolName;					//!< Name used in formula script
				SML::SYMBOL *m_symobj;						//!< Symbol for object in context
				SIMPLE_ARRAY<DPOINT2D> m_pointbuf;	//!< Point buffer

				//! Construct from RVC object.
				OBJECT (
					const RVC::OBJITEM& objitem
					);

				//! Constructor used when reading formula definition.
				OBJECT (
					RVC::OBJTYPE objtype,
					INIHANDLE handle,
					int objnum
					);

				//! Constructor used in reading from serialization.
				OBJECT (
					RVC::OBJTYPE objtype,
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SetObjPoint (
					INT32 idx
					);
					
			private:

				SML::SYMBOL *m_symx;						// Symbol for object X coordinate
				SML::SYMBOL *m_symy;						// Symbol for object Y coordinate
				TRANS2D_AFFINE m_affine;
				bool m_IsAffine;							// Set if transformation is affine

				RVC::OBJITEM m_objitem;
				REGION2D m_ObjRegion;					// Region defining object extents
				REGION2D m_MapRegion;					// Region defining object extents
				TRANS2D_MAPGEN m_trans;					// Transformation from object to virtual raster

				ERRVALUE CalcObjPoints (
					const GEOFORMULA& formula,
					const DPOINT2D& start,
					const DPOINT2D& inc,
					INT32 numvals
					);

				ERRVALUE SetupTrans (
					GEOFORMULA& formula
					);

				virtual ERRVALUE v_AddSymbols (GEOFORMULA& formula, bool OnlyIfUsed);
				virtual void v_ClearSymbols ();
				virtual void v_Close () = 0;
				virtual ERRVALUE v_Open () = 0;
				virtual ERRVALUE v_SetObjPoint (INT32 idx) = 0;

				friend class GEOFORMULA;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::OBJECT

		//! Raster Object.
		class RASTER : public OBJECT {
			public:

				//! Copy constructor.
				RASTER (
					const RASTER& rhs
					);

				//! Destructor.
				virtual ~RASTER ();

				//! Assignment.
				RASTER& operator= (
					const RASTER& rhs
					);

				ERRVALUE ComputeCellSizeMeters (
					double& cellsize
					) const;

				CONTRASTPARM::METHOD GetLookupMethod () const
					{ return (m_LookupMethod); }

				const MISTRING& GetLookupTableName () const
					{ return (m_LookupTableName); }

				bool GetLookupUseTable () const
					{ return (m_bLookupUseTable); }

				void SetLookupMethod (
					CONTRASTPARM::METHOD LookupMethod
					) { m_LookupMethod = LookupMethod; m_bLookupUseTable = false; }

				void SetLookupTableName (
					const MISTRING& LookupTableName
					) { m_LookupTableName = LookupTableName; m_bLookupUseTable = true; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				enum DATATYPE {
					DATATYPE_RasterNumeric =		0,
					DATATYPE_RasterColor =			1,
					DATATYPE_RasterComplex =		2
					};

				RVC::RASTER m_ObjRaster;
				DATATYPE m_datatype;							//!< Allowed data type
				CONTRASTPARM::METHOD m_LookupMethod;
				MISTRING m_LookupTableName;
				bool m_bLookupUseTable;					//!< Use contrast table if available
				RASTINTERP *m_pRastInterp;
				SML::SYMBOL *m_symval;
				SML::SYMBOL *m_symred;
				SML::SYMBOL *m_symgreen;
				SML::SYMBOL *m_symblue;
				bool m_NeedValue;								//!< Set if need interpolated value

				//! Construct from RVC object.
				RASTER (const RVC::OBJITEM& ObjItem);

				//! Constructor used when reading formula definition.
				RASTER (
					INIHANDLE handle,
					int objnum
					);

				//! Constructor used in reading from serialization.
				RASTER (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);

				virtual ERRVALUE v_AddSymbols (GEOFORMULA& formula, bool OnlyIfUsed);
				virtual void v_ClearSymbols ();
				virtual void v_Close ();
				virtual ERRVALUE v_Open ();
				virtual ERRVALUE v_SetObjPoint (INT32 idx);

				friend class GEOFORMULA;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::RASTER

		//! Geoformula vector object.
		class VECTOR : public OBJECT {
			public:

				//! Default constructor.
				VECTOR ();

				//! Copy constructor.
				VECTOR (
					const VECTOR& rhs
					);

				//! Destructor.
				virtual ~VECTOR ();

				//! Assignment.
				VECTOR& operator= (
					const VECTOR& rhs
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::VECTOR m_ObjVector;
				
				//! Construct from RVC object.
				VECTOR (
					const RVC::OBJITEM& ObjItem
					);

				//! Constructor used when reading formula definition.
				VECTOR (
					INIHANDLE handle,
					int objnum
					):
					OBJECT(RVC::OBJTYPE_Vector,handle,objnum)
					{ }

				//! Constructor used in reading from serialization.
				VECTOR (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);

				virtual void v_Close ();
				virtual ERRVALUE v_Open ();
				virtual ERRVALUE v_SetObjPoint (INT32 idx);

				friend class GEOFORMULA;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::VECTOR

		//! TIN object.
		class TIN : public OBJECT {
			public:

				//! Default constructor.
				TIN ();

				//! Copy constructor.
				TIN (
					const TIN& rhs
					);

				//! Destructor.
				virtual ~TIN ();

				//! Assignment.
				TIN& operator= (
					const TIN& rhs
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				RVC::TIN m_ObjTIN;
				SML::SYMBOL *m_symval;

				//! Construct from RVC object.
				TIN (
					const RVC::OBJITEM& ObjItem
					);

				//! Constructor used when reading formula definition.
				TIN (
					INIHANDLE handle,
					int objnum
					):
					OBJECT(RVC::OBJTYPE_TIN,handle,objnum)
					{ }

				//! Constructor used in reading from serialization.
				TIN (
					const MISTRING& SymbolName,
					const RVC::OBJITEM& ObjItem
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);

				virtual ERRVALUE v_AddSymbols (GEOFORMULA& formula, bool OnlyIfUsed);
				virtual void v_ClearSymbols ();
				virtual void v_Close ();
				virtual ERRVALUE v_Open ();
				virtual ERRVALUE v_SetObjPoint (INT32 idx);

				friend class GEOFORMULA;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::TIN

		//! User-settable value.
		class VALUE {
			public:

				VALUE ();

				VALUE (
					const VALUE& rhs
					);

				~VALUE ();

				VALUE& operator= (
					const VALUE& rhs
					);

				void ClearSymbol (
					) { m_Symbol = 0; }

				void CreateSymbol (
					SML::CONTEXT *context
					);

				double GetDefaultValue () const
					{ return (m_DefaultValue); }

				const MISTRING& GetSymbolName () const
					{ return (m_SymbolName); }

				double GetUserValue () const
					{ return (m_UserValue); }

				void SetDefaultValue (
					double value
					) { m_DefaultValue = m_UserValue = value; }

				void SetSymbolName (
					const MISTRING& SymbolName
					) { m_SymbolName = SymbolName; }

				void SetUserValue (
					double value
					) { m_UserValue = value; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT

				MISTRING m_SymbolName;					//!< Name used in formula script
				double m_DefaultValue;					//!< Default value
				double m_UserValue;						//!< User-specified value
				SML::SYMBOL *m_Symbol;						//!< Symbol in context

				//! Constructor used to read definition.
				VALUE (
					INIHANDLE handle,
					int valnum
					);

				//! Constructor used to create new value
				VALUE (
					const MISTRING& name,
					double dftvalue
					);

				static const SERIALIZERITEM* SerialGetItemDef ();

				ERRVALUE SerialRead (
					SERIALIZER& serializer
					);

				ERRVALUE SerialWrite (
					SERIALIZER& serializer
					);

				friend class GEOFORMULA;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};	// End of GEOFORMULA::VALUE

		class OUTPUT {
			public:

				OUTPUT ();

				OUTPUT (
					const OUTPUT& rhs
					);

				// Construct using name and cell type.
				OUTPUT (
					const MISTRING& name,
					RVC::IMAGE::CELLTYPE CellType
					);

				~OUTPUT ();

				OUTPUT& operator= (
					const OUTPUT& rhs
					);

				void AddSymbols (
					GEOFORMULA& formula
					);

				void ClearSymbols ();

				const MISTRING& GetSymbolName () const
					{ return (m_SymbolName); }

				void SetSymbolName (
					const MISTRING& SymbolName
					) { m_SymbolName = SymbolName; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				MISTRING m_SymbolName;					// Name used in formula script
				RVC::IMAGE::CELLTYPE m_CellType;
				SML::SYMBOL *m_pSymValue;
				SML::SYMBOL *m_pSymRed;
				SML::SYMBOL *m_pSymGreen;
				SML::SYMBOL *m_pSymBlue;

				static const SERIALIZERITEM* SerialGetItemDef ();

				ERRVALUE SerialRead (SERIALIZER& serializer);
				ERRVALUE SerialWrite (SERIALIZER& serializer);

				friend class GEOFORMULA;
				#endif
			};

		//! Editor.
		class EDITOR {
			public:

				//! Construct editor for specific formula.
				explicit EDITOR (
					GEOFORMULA& formula
					);

				//! Destructor.
				~EDITOR ();

				MGUI::FORM_BUTTONTEXT& AccessFormExtentsRegion ();

				void AttachStatusBar (MGUI::CTRL_STATUSBAR& statusbar);

				bool CheckValid ();

				void CreateFormExtents (MGUI::LAYOUT_PANE_BASE& paneParent);

				void CreateFormExtentsRegion (MGUI::LAYOUT_PANE_BASE& pane, const MISTRING& label);

				void CreatePaneObjects (MGUI::LAYOUT_PANE_BASE& paneParent);

				void CreatePaneScript (MGUI::LAYOUT_PANE_BASE& paneParent, MGUI::LAYOUT_PANE_BASE *pPaneToolbar = 0);

				void CreatePaneValues (MGUI::LAYOUT_PANE_BASE& paneParent);

				void CreateValueListItem (VALUE& val);

				void DisableUpdateContext ();

				ERRVALUE DlgSelectExtentsRegion (MDLGPARENT dlgparent);

				void EnableUpdateContext ();

				const MISTRING& GetScript () const;

				bool IsChanged () const;

				ERRVALUE Reset ();

				void ResetChanged ();

				void SetDelegateOnAnyChange (
					DELEGATE_VOID_NOPARMS delegate
					);

				void SetDelegateOnObjectListChange (
					DELEGATE_VOID_NOPARMS delegate
					);

				void SetDelegateOnSelectObjects (
					DELEGATE_VOID_NOPARMS delegate
					);

				//! Set extents mode.
				void SetExtentsMode (
					EXTENTSMODE ExtentsMode,
					bool bNotify
					);

				void UpdateScriptContext (
					bool bInitialize
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				class PRIV;
				PRIV *m_pPriv;

				EDITOR (const EDITOR&);
				EDITOR& operator= (const EDITOR&);
				#endif // GENERATING_DOXYGEN_OUTPUT

			};	// End of GEOFORMULA::EDITOR

		//! Default constructor.
		GEOFORMULA ();

		//! Copy constructor.
		GEOFORMULA (
			const GEOFORMULA& rhs
			);

		//! Destructor.
		~GEOFORMULA ();

		//! Assignment.
		GEOFORMULA& operator= (
			const GEOFORMULA& rhs
			);

		//! Add new "value" symbol to formula.
		ERRVALUE AddValue (
			const MISTRING& SymbolName,			//!< Symbol name
			double value								//!< Initial value
			);

		//! Create new formula from specified list of objects.
		ERRVALUE CreateFromObjects (
			const RVC::OBJITEMLIST& objitems
			);

		//! Display dialog allowing user to select region for extents.
		ERRVALUE DlgSelectExtentsRegion (
			MDLGPARENT dlgparent,
			const REGION2D *pTestRegion = 0					//!< Region to test for overlap with, 0 to allow all regions
			);

		//! Display dialog allowing user to select specific objects to use.
		ERRVALUE DlgSelectObjects (
			MDLGPARENT dlgparent
			);

		//! Prompt user to select objects and create formula from list.
		ERRVALUE DlgSelectObjectsAndCreate (
			MDLGPARENT dlgparent
			);

		//! Evaluate formula for points along specified interval with output to single (double or RGB byte) buffer.
		ERRVALUE Evaluate (
			const DPOINT2D& start,
			const DPOINT2D& increment,
			INT32 numvals,
			void *buffer,
			UINT8 *omask
			);

		//! Evaluate formula for points along specified interval with output to multiple (double) buffers.
		ERRVALUE Evaluate (
			const DPOINT2D& start,
			const DPOINT2D& increment,
			INT32 numvals,
			double * const * buffers,
			UINT8 *omask
			);

		//! Get list of unique CRS's used.
		void GetCoordRefSysList (
			SPATREF::COORDREFSYSLIST& crslist
			) const;

		//! Get extents region.
		const REGION2D& GetExtMapRegion () const
			{ return (m_ExtMapRegion); }

		//! Get number of objects used in formula.
		UINT32 GetNumObjects () const
			{ return (m_Rasters.GetNumItems() + m_Vectors.GetNumItems() + m_TINs.GetNumItems()); }

		//! Get objitem list
		void GetObjItems(RVC::OBJITEMLIST& objitems);

		//! Get raster object list.
		const MILIST<RASTER>& GetObjectListRaster () const
			{ return (m_Rasters); }

		//! Get TIN object list.
		const MILIST<TIN>& GetObjectListTIN () const
			{ return (m_TINs); }

		//! Get vector object list.
		const MILIST<VECTOR>& GetObjectListVector () const
			{ return (m_Vectors); }

		//! Get formula output type.
		OUTPUTTYPE GetOutputType () const
			{ return (m_OutputType); }

		//! Get formula script.
		const MISTRING& GetScript () const
			{ return (m_Script); }

		//! Get transformation from virtual formula object to extents CRS.
		ERRVALUE GetTransToExtCRS (
			TRANS2D_MAPGEN& Trans,					//!< Transformation returned
			bool dooutput = false					//!< Place in output size of transformation
			) const;

		//! Determine if Start has been called.
		bool IsStarted () const
			{ return (m_bIsStarted); }

		//! Open objects to use in formula.
		ERRVALUE OpenObjects ();

		//! Read previously saved formula definition.
		ERRVALUE ReadDefn (
			const FILEPATH& filepath,
			CHAR_ENCODING textencoding = CHAR_ENCODING_UTF8
			);

		//! Read from serialization.
		ERRVALUE SerialRead (
			SERIALIZER& serializer
			);

		//! Write to serialization.
		ERRVALUE SerialWrite (
			SERIALIZER& serializer,
			const char *tagname = 0
			);

		//! Set coordinate reference system for defining extents.
		void SetExtMapCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			) { m_ExtMapRegion.SetCoordRefSys(CoordRefSys); m_Trans.SetOutputMapCoordRefSys(CoordRefSys); }

		//! Set extents map region based on specified object.
		ERRVALUE SetExtMapRegion (
			const RVC::OBJITEM& ObjItem
			);

		//! Set objitem list, must be created first
		ERRVALUE SetObjItems (const RVC::OBJITEMLIST& objitems);

		//! Set output coordinate reference system
		void SetOutputCoordRefSys (
			const SPATREF::COORDREFSYS& CoordRefSys
			);

		//! Set transformation from output object to map coordinates.
		void SetOutputTrans (
			const TRANS2D_AFFINE& trans
			);

		//! Set output transformation from "output side" of a TRANS2D_MAPGEN reference.
		void SetOutputTrans (
			const TRANS2D_MAPGEN& trans
			);

		//! Set transformation from output object to map coordinates.
		void SetOutputTransIdentity ();

		//! Set formula output type
		void SetOutputType (
			OUTPUTTYPE OutputType
			) { m_OutputType = OutputType; }

		//! Set script to use in formula.
		void SetScript (
			const MISTRING& script
			) { m_Script = script; }

		//! Start formula evaluation.
		ERRVALUE Start (
			const DRECT2D *extents,
			SCRIPTFLAGS flags = SCRIPTFLAG_None
			);

		//! Stop formula evaluation.
		void Stop ();

		//! Write formula definition to specified file.
		ERRVALUE WriteDefn (
			const FILEPATH& filepath,
			CHAR_ENCODING textencoding = CHAR_ENCODING_UTF8
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		OUTPUTTYPE m_OutputType;						// Formula type
		EXTENTSMODE m_ExtentsMode;						// Extents mode
		SML::CONTEXT *m_Context;						// Query context
		int m_Segment;										// Query segment
		SML::SYMBOL *m_symred;							// Query symbol for red output value
		SML::SYMBOL *m_symgreen;						// Query symbol for green output value
		SML::SYMBOL *m_symblue;							// Query symbol for blue output value
		bool m_AllAffine;									// All transformations are affine
		bool m_bIsStarted;
		EDITOR *m_editor;
		MILIST<RASTER> m_Rasters;						// Raster objects used in formula
		MILIST<VECTOR> m_Vectors;						// Vector objects used in formula
		MILIST<TIN> m_TINs;								// TIN objects used in formula
		MILIST<VALUE> m_Values;
		MISTRING m_Script;								// Formula script expression
		RVC::OBJITEM m_ObjItemRegion;
		REGION2D m_ExtMapRegion;						// Extents region
		TRANS2D_MAPGEN m_Trans;							// Transformation from output raster to map
		SIMPLE_ARRAY<DPOINT2D> m_ipointbuf;

		static const SERIALIZERITEM* SerialGetItemDef ();
		static ERRVALUE SerializerCB_Object (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);
		static ERRVALUE SerializerCB_Value (SERIALIZER&, const SERIALIZERITEM*, void*, SERIALIZERITEM::ACTION);

		ERRVALUE AddObjectRegion (
			const OBJECT& object
			);

		ERRVALUE AddObjects (
			const RVC::OBJITEMLIST& objitems
			);

		SML::SYMBOL* AddSymbol (
			const MISTRING& basename,
			const char* subname,
			bool OnlyIfUsed
			);

		ERRVALUE CalcObjPoints (const DPOINT2D& start, const DPOINT2D& inc, INT32 numvals);
		void Clear ();
		void Copy (const GEOFORMULA& rhs);

		OBJECT* FindObjectByName (
			const MISTRING& symname
			);

		const VALUE* FindValueByName (
			const MISTRING& symname
			) const;

		const OBJECT* GetFirstObject () const;

		int GetNumNameUses (const MISTRING& symname) const;

		ERRVALUE InitQuery (
			SCRIPTFLAGS flags = SCRIPTFLAG_None
			);

		//! @return true if successful, false if name used by another object or value.
		bool SetObjectSymbolName (
			OBJECT& obj,
			const MISTRING& symname
			);

		ERRVALUE SetObjPoints (INT32 idx);

		void SetupTrans ();

		//! @return true if successful, false if name used by another object or value.
		bool SetValueSymbolName (
			VALUE& val,
			const MISTRING& symname
			);

		void StopQuery ();
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GEOFORMULA::SCRIPTFLAGS)
#endif

#endif	// INC_MI32_GEOFRMLA_H
