/**
 * \file mi32/glcm.h
 * \brief Gray Level Cooccurrence Matrix (GLCM) class definitions
 *
 * \if NODOC
 * $Id: glcm.h_v 1.1 2014/09/02 19:31:54 mju Exp $
 *
 * $Log: glcm.h_v $
 * Revision 1.1  2014/09/02 19:31:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_GLCM_H
#define	INC_MI32_GLCM_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif


//=====================================================================================================================

//! Gray Level Coocurrence Matrix for computing various local image texture properties.
class GLCM {
	public:

		enum DIRECTION {
			DIRECTION_Horizontal = 0,
			DIRECTION_Vertical,
			DIRECTION_Invariant
			};

		//! Values computed from glcm.
		enum VALUES {
			VALUE_None =				0,
			VALUE_Contrast =			0x01,
			VALUE_Dissimilarity =	0x02,
			VALUE_Homogeneity =		0x04,
			VALUE_ASM =					0x08,		//!< Angular second moment
			VALUE_Entropy =			0x10,
			VALUE_Mean =				0x20,
			VALUE_Variance =			0x40,
			VALUE_Correlation =		0x80,
			VALUES_All =				0xFF
			};

		//! Container for accumulated GLCM counts.
		class ACCUMULATOR {
			public:

				ACCUMULATOR ();
				~ACCUMULATOR ();

				//! Add one accumulator to another.
				ACCUMULATOR& operator+= (
					const ACCUMULATOR& rhs
					);

				//! Matrix element accessor.
				UINT32& operator () (
					UINT32 i1,
					UINT32 i2
					) { return (m_count[GetIndex(i1,i2)]); }

				//! Matrix element accessor.
				UINT32 operator () (
					UINT32 i1,
					UINT32 i2
					) const { return (m_count[GetIndex(i1,i2)]); }

				UINT32 operator[] (
					UINT32 i
					) const { return (m_count[i]); }

				UINT32 CalcSum () const;

				//! Clear accumulator.
				void Clear ()
					{ m_count.ClearItems(); }

				UINT32 GetSize () const
					{ return (m_count.GetNumItems()); }

				//! Resize accumulator.
				ERRVALUE Resize (
					UINT32 numitems
					);

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				SIMPLE_ARRAY<UINT32> m_count;

				static UINT32 GetIndex (
					UINT32 i1,
					UINT32 i2
					) { return ((i1 < i2) ? (i2 * (i2 + 1) / 2 + i1) : (i1 * (i1 + 1) / 2 + i2)); }

				#endif
			};

		//! Constructor.
		GLCM ();

		//! Destructor.
		~GLCM ();

		//! Accumulate for pair of image rows.
		ERRVALUE Accumulate (
			UINT32 numcells,						//!< Number of cells
			const double *pData0,				//!< Row 0 cells
			const double *pData1,				//!< Row 1 cells
			const UINT8 *pValidity0 = 0,		//!< Validity for row 0 cells, 0 if all valid
			const UINT8 *pValidity1 = 0		//!< Validity for row 1 cells, 0 if all valid
			);

		//! Calculate values using current accumulator.
		ERRVALUE CalcValues ()
			{ return (CalcValues(m_accumulator)); }

		//! Calculate values using specified accumulator.
		ERRVALUE CalcValues (
			const ACCUMULATOR& accumulator
			);

		//! Get accumulator.
		const ACCUMULATOR& GetAccumulator () const
			{ return (m_accumulator); }

		//! Initialize for specific image parameters.
		ERRVALUE Initialize (
			UINT16 numbins,					//!< Number of bins to quantize into
			double minvalue,					//!< Minimum input value
			double interval					//!< Quantization interval
			);

		//! Get specified value.
		//! Must call CalcValues before using.
		double GetValue (
			VALUES value						//!< Value to get
			) const;

		//! Determine if has been initialized.
		bool IsInitialized () const
			{ return (m_numbins != 0); }

		//! Reset accumulator.
		void Reset ();

		//! Set cooccurrence direction.
		//! Will use invariant if not specified.
		void SetDirection (
			DIRECTION direction
			) { m_direction = direction; }

		//! Set values to be calculated.
		//! If not called, all values will be computed.
		void SetValues (
			VALUES values
			) { m_values = values; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		ACCUMULATOR m_accumulator;
		double m_minvalue;
		double m_interval;
		double m_Contrast;
		double m_Dissimilarity;
		double m_Homogeneity;
		double m_ASM;
		double m_Entropy;
		double m_Mean;
		double m_Variance;
		double m_Correlation;
		UINT16 m_numbins;
		DIRECTION m_direction;
		VALUES m_values;

		UINT32 CalcBin (
			double value
			) const { double bin = (value - m_minvalue) / m_interval; return ((bin < 0) ? 0 : (bin >= m_numbins) ? (m_numbins - 1) : static_cast<UINT32>(bin)); }

		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(GLCM::VALUES)
#endif

//=====================================================================================================================

#endif	// INC_MI32_GLCM_H
