/**
 * \file mi32/houghlinear.h
 * \brief HOUGH_LINEAR definitions
 *
 * \if NODOC
 * $Id: houghlinear.h_v 1.2 2014/02/05 17:58:44 mju Exp $
 *
 * $Log: houghlinear.h_v $
 * Revision 1.2  2014/02/05 17:58:44  mju
 * More initial implementation.
 *
 * Revision 1.1  2014/01/31 22:20:10  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_HOUGHLINEAR_H
#define	INC_MI32_HOUGHLINEAR_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
namespace SPATMOD {
	namespace IMAGE {
		class BUFFER;
		class STAGE_AUTOPTR;
		}
	}
#endif

//=====================================================================================================================

//! Hough Transform for linear features.
class HOUGH_LINEAR {
	public:

		//! Constructor.
		HOUGH_LINEAR ();

		//! Destructor.
		~HOUGH_LINEAR ();

		//! Accumulate for specified point
		ERRVALUE Accumulate (
			const DPOINT2D& ptFeature	//!< Point in feature coordinates to accumulate
			);

		//! Calculate curve in hough accumulator coordinates from point.
		ERRVALUE CalcCurveFromFeaturePoint (
			const DPOINT2D& ptFeature,	//!< Point in feature coordinates
			POLYLINE& polyline			//!< Curve in "hough" coordinates returned
			);

		//! Calculate line from hough point coordinates.
		bool CalcLineFromHoughPoint (
			const DPOINT2D& ptHough,	//!< Point in "hough" coordinates
			POLYLINE& line					//!< 2-point line in "feature" coordinates returned
			) const;

		//! Construct image stage for reading transform.
		ERRVALUE ConstructImageStage (
			SPATMOD::IMAGE::STAGE_AUTOPTR& StagePtr		//!< Container for stage pointer, will have ownership
			);

		//! Find maximum count within specified radius of point
		//! @return Maximum accumulator count
		UINT32 FindMaxCount (
			const DPOINT2D& ptCenter,	//!< Center point for search in hough coordinates
			double radius,					//!< Radius in hough space for search
			LPOINT2D& ptMaxCount			//!< Point in hough coordinates corresponding to maximum count
			) const;

		//! Finish accumulation.
		ERRVALUE Finish ();

		//! Get pointer to accumulator row.
		const UINT32* GetAccumulatorRowPtr (
			UINT32 row				//!< Accumulator row
			) const { return (&m_Accumulator[row*m_NumAngleSteps]); }

		//! Get accumulator count value.
		UINT32 GetAccumulatorValue (
			const LPOINT2D& ptHough
			) const { return (GetAccumulatorRowPtr(ptHough.y)[ptHough.x]); }

		//! Get maximum accumulator count.
		UINT32 GetMaxCount () const
			{ return (m_MaxCount); }

		//! Get number of accumuator columns.
		UINT32 GetNumAccumCols () const
			{ return (m_NumAngleSteps); }

		//! Get number of accumulator rows.
		UINT32 GetNumAccumRows () const
			{ return (m_NumAccumRows); }

		//! Initialize accumulator.
		//! The accumulator buffer will require NumAngleSteps * (NumDistanceSteps * 2 + 1) * 4 bytes.
		ERRVALUE Initialize (
			double MaxDistance,					//!< Maximum absolute horizontal/vertical distance (maximum absolute X or Y value of point to accumulate)
			UINT32 NumDistanceSteps = 128,	//!< Number of distance steps for scaling
			UINT32 NumAngleSteps = 180			//!< Number of angular steps corresponding to (0 inclusive to 180 exclusive)
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		SIMPLE_ARRAY<UINT32> m_Accumulator;
		DOUBLE_ARRAY<double> m_CosTable;
		DOUBLE_ARRAY<double> m_SinTable;
		SIMPLE_ARRAY<UINT8> m_Validity;
		double m_DistanceScale;
		INT32 m_DistanceOffsetScaled;		// Scaled distance offset (zero row in accumulator)
		UINT32 m_NumAngleSteps;				// Number of angle steps from 0 <= angle < 180
		UINT32 m_NumAccumRows;
		UINT32 m_MaxCount;
		SPATMOD::IMAGE::BUFFER *m_pImageBuffer;

		void CalcValidity ();

		HOUGH_LINEAR (const HOUGH_LINEAR&);
		HOUGH_LINEAR& operator= (const HOUGH_LINEAR&);
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_HOUGHLINEAR_H
