/**
 * \file mi32/longlist.h
 * \brief Definitions for LONGLIST
 *
 * \if NODOC
 * $Id: longlist.h_v 1.13 2014/10/13 20:20:50 mju Exp $
 *
 * $Log: longlist.h_v $
 * Revision 1.13  2014/10/13 20:20:50  mju
 * Add setFlags.
 *
 * Revision 1.12  2014/10/10 20:38:51  mju
 * Add iterator and exchange method.
 * Remove methods replaced by iterator and add documentation.
 *
 * Revision 1.10  2010/08/19 17:39:48  mju
 * Use unsignedset in setBits.
 *
 * Revision 1.8  2006/10/03 15:59:38  dwilliss
 * Added a method to get the list as simple array of int32
 *
 * Revision 1.1  1999/04/08  14:44:29  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_LONGLIST_H
#define	INC_MI32_LONGLIST_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class UNSIGNEDSET;
#endif


//! A LONGLIST serves two purposes. 
//! 	When possible, it keeps track of a list of numbers using a BITSET.
//!	It can also be made to keep track of a set of numbers like a SIMPLE_ARRAY
//!	but with an extra UINT8 flags for each entry.
class LONGLIST {
	public:

		//! Iterator to step forward through all items in list.
		class ITERATOR {
			public:

				//! Default constructor.
				ITERATOR ();

				//! Dereference operator, returns item value.
				INT32 operator* () const
					{ return (m_Value); }

				//! Cast operator to INT32 item value.
				operator INT32 () const
					{ return (m_Value); }

				//! Pre-increment operator.
				ITERATOR& operator++ ();

				//! Equality operator.
				bool operator== (
					const ITERATOR& rhs
					) const {
					return (m_pList == rhs.m_pList && m_NextIndex == rhs.m_NextIndex);
					}

				//! Inequality operator.
				bool operator!= (
					const ITERATOR& rhs
					) const {
					return (!(*this == rhs));
					}

				//! Get "flag" value if available.
				//! Will return 0 if no flags stored in list.
				UINT8 GetFlag () const
					{ return (m_Flag); }

				bool IsEnd () const
					{ return (m_pList == 0 || m_NextIndex > m_pList->m_NumAlloc); }

			private:
	   		#ifndef GENERATING_DOXYGEN_OUTPUT
				const LONGLIST *m_pList;
				UINT32 m_NextIndex;
				INT32 m_Value;
				UINT8 m_Flag;

				// Construct for "Begin".
				ITERATOR (
					const LONGLIST *pList
					):
					m_pList(pList),
					m_NextIndex(0),
					m_Value(-1),
					m_Flag(0)
					{ operator++(); }

				// Construct for "End".
				ITERATOR (
					const LONGLIST *pList,
					bool
					):
					m_pList(pList),
					m_NextIndex(pList->m_NumAlloc+1),
					m_Value(-1),
					m_Flag(0)
					{ }

				friend class LONGLIST;
	   	#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Constructor.		
		LONGLIST ();

		//! Destructor.
		~LONGLIST ();

		//! Add value to list without checking if unique.
		//! If list is using bitset then will still only add once.
		ERRVALUE Add (INT32 value);

		//! Add value to list if unique.
		ERRVALUE AddUnique (INT32 value);

		//! Add value and flag to list if value is unique.
		ERRVALUE AddUniqueWithFlag (INT32 value, UINT8 flag);

		//! Add value and flag to list without checking if unique.
		ERRVALUE AddWithFlag (INT32 value, UINT8 flag);

		//! Return iterator for first value in list.
		ITERATOR Begin () const
			{ return (ITERATOR(this)); }

		//! Determine if list contains specified value.
		bool Contains (INT32 value) const;

		//! Clear the list.
		void Clear ();

		//! Return iterator for end of list.
		ITERATOR End () const
			{ return (ITERATOR(this,true)); }

		//! Exchange (swap) contents with another LONGLIST.
		void Exchange (LONGLIST& other);

		//! Free allocated resources.
		void Free ();

		//! Get values as array.
		ERRVALUE GetArray (SIMPLE_ARRAY<INT32>& List) const;

		//! Get first value in list.
		//! Return value will be -1 if list is empty.
		INT32 GetFirst () const;

		//! Get flag using index.
		//! No validation check is made on index.
		UINT8 GetFlags (UINT32 Index) const
			{ return (m_flags[Index]); }

		//! Get last value in list.
		//! Return value will be -1 if list is empty.
		INT32 GetLast () const;

		//! Get number of items in list.
		UINT32 GetNumItems () const
			{ return (m_NumItems); }

		//! Get value using index.
		//! No validation check is made on index.
		INT32 GetValue (UINT32 Index) const
			{ return (m_List[Index]); }

		//! Determine if has flag values.
		bool HasFlags () const
			{ return (!m_bUseBits && m_flags != 0); }

		//! Determine if sorted.
		bool IsSorted () const
			{ return (m_bSorted); }

		//! Remove value from list.
		ERRVALUE Remove (INT32 value);

		//! Set using bits from UNSIGNEDSET.
		ERRVALUE SetBits (const UNSIGNEDSET& set);

		//! Set flags using index.
		//! No validation check is made on index.
		void SetFlags (UINT32 Index, UINT8 flags)
			{ if (m_flags != 0 && !m_bUseBits) m_flags[Index] = flags; }

		//! Set whether list is sorted.
		//! This is useful if you know you added the items in sorted order.
		//! You may also want to set it to false to force a re-sort if you change
		//! the sort criteria.
		void SetSorted (
			bool bSorted = true
			) { m_bSorted = bSorted; }

		//! Set whether items are kept track of by bits.
		void SetUseBits (
			bool val = true
			) { m_bUseBits = val; }

		//! Sort the list.
		ERRVALUE Sort (
			int (*cmpfunc)(INT32, INT32, void*) = 0,
			void *cmpdata = 0
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		INT32 *m_List;
		UINT8 *m_flags; 
		UINT32 m_NumAlloc;
		UINT32 m_NumItems;
		bool m_bSorted;
		bool m_bUseBits;

		static int HeapSortCompare(INT32, INT32, void*);
		static int HeapSortSwap(INT32, INT32, void*);

		INT32 GetList (INT32** listp) const;

		LONGLIST (const LONGLIST&);
		LONGLIST& operator= (const LONGLIST&);
		#endif
	};

#endif	// INC_MI32_LONGLIST_H
