/**
 * \file mi32/mitin.h
 * \brief MITIN definitions
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_MI32_MITIN_H
#define	INC_MI32_MITIN_H

#ifndef	INC_MI32_SIMPLEAR_H
#include <mi32/simplear.h>
#endif

//=====================================================================================================================

//! TIN base class.
class MITIN {
	public:

		//! Edge, containing indices of beginning/ending nodes and left/right faces.
		class EDGE {
			public:
				//! Default constructor.
				EDGE ()
					m_NodeBegin(UINT32_MAX), m_NodeEnd(UINT32_MAX), m_FaceLeft(UINT32_MAX), m_FaceRight(UINT32_MAX)
					{}

				//! Get index of left face.
				UINT32 GetFaceIdxLeft () const
					{ return (m_FaceLeft); }
				//! Get index of right face.
				UINT32 GetFaceIdxRight () const
					{ return (m_FaceRight); }
				//! Get index of beginning node.
				UINT32 GetNodeIdxBegin () const
					{ return (m_NodeBegin); }
				//! Get index of ending node.
				UINT32 GetNodeIdxEnd () const
					{ return (m_NodeEnd); }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				UINT32 m_NodeBegin;
				UINT32 m_NodeEnd;
				UINT32 m_FaceLeft;
				UINT32 m_FaceRight;
				UINT32 m_Flags;
				#endif
			};

		//! Face (triangle) containing indices for 3 bounding edges.
		class FACE {
			public:
				//! Default constructor.
				FACE ()
					{}
				//! Get edge index at specified position.
				UINT32 GetEdgeIdx (
					int pos							//! Position (0-2)
					) const { return (m_Edges[pos]); }
				//! Get edge indices.
				const UINT32* GetEdgeIdxs () const
					{ return (m_Edges); }
			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				UINT32 m_Edges[3];
				#endif
			};

		//!< Constructor.
		MITIN ();

		//!< Destructor.
		virtual ~MITIN ();

		const SIMPLE_ARRAY<FACE>& GetEdges () const
			{ return (m_Edges); }

		const SIMPLE_ARRAY<FACE>& GetFaces () const
			{ return (m_Faces); }

	protected:

		UINT32 EdgeAdd (UINT32 NodeIdx1, UINT32 NodeIdx2);

		ERRVALUE FaceAdd (UINT32 EdgeIdx01, UINT32 EdgeIdx12, UINT32 EdgeIdx20);

		#ifndef GENERATING_DOXYGEN_OUTPUT
		#if defined(WIN32) && defined(DEBUG)
		void FaceAssertValid (UINT32 FaceIdx) const;
		#endif
		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SIMPLE_ARRAY<EDGE> m_Edges;
		SIMPLE_ARRAY<FACE> m_Faces;

		MITIN (const MITIN&);
		MITIN& operator= (const MITIN&);

		#endif
	};

//=====================================================================================================================

//! TIN with INT32 node coordinates.
class MITIN_INT32 : public MITIN {
	public:
		class NODE {
			INT32 m_row;
			INT32 m_col;
//			INT32 m_RefNext;		// Number of references or next free node
			};

	private:
		SIMPLE_ARRAY<UINT32> m_EdgeIdxTop;
		SIMPLE_ARRAY<UINT32> m_EdgeIdxBottom;
		SIMPLE_ARRAY<UINT32> m_EdgeIdxWork;
		SIMPLE_ARRAY<UINT32> m_EdgeIdxTest;
		SIMPLE_ARRAY<UINT32> m_FaceIdxTest;

		void EdgeTestAdd (UINT32 EdgeIdx, UINT32 FaceIdx);

		int EdgeTestCell (const EDGE& Edge, INT32 row, INT32 col) const
			{ return (NodesTestCell(Edge.m_NodeStart,Edge.m_NodeEnd,row,col)); }

		int EdgeTestCell (UINT32 EdgeIdx, INT32 row, INT32 col) const
			{ return (EdgeTestCell(m_Edges[EdgeIdx],row,col)); }

		void EdgeTestResplit (UINT32 EdgeIdx, UINT32 FaceIdx);

		void EdgeTestSliver (UINT32 EdgeIdx, SIMPLE_ARRAY<UINT32>& EdgeIdxTest);

		UINT32 NodeAdd (INT32 row, INT32 col);

		UINT32 NodeCalcDistance2 (UINT32 NodeIdx1, UINT32 NodeIdx2) const
			{
			INT32 drow = m_Nodes[NodeIdx2].m_row - m_Nodes[NodeIdx1].m_row;
			INT32 dcol = m_Nodes[NodeIdx2].m_col - m_Nodes[NodeIdx1].m_col;
			return (drow*drow + dcol*dcol);
			}

		int NodesTestCell (UINT32 NodeIdxStart, UINT32 NodeIdxEnd, INT32 row, INT32 col) const
			{ return (NodesTestCell(m_Nodes[NodeIdxStart],m_Nodes[NodeIdxEnd],row,col)); }

		int NodesTestCell (const NODE& NodeStart, const NODE& NodeEnd, INT32 row, INT32 col) const
			{
			INT32 drows = NodeEnd.m_row - NodeStart.m_row;
			INT32 dcols = NodeEnd.m_col - NodeStart.m_col;
			INT32 trows = row - NodeStart.m_row;
			INT32 tcols = col - NodeStart.m_col;
			return (dcols * trows - drows * tcols);	// < 0 if left, > 0 if right
			}

		void PutValues (INT32 row, INT32 col, UINT32 num) const
			{
			double *pZ = m_Values;
			*pZ = m_Zrow * row + m_Zcol * col + m_Z0;
			for (UINT32 i = 1; (i < num); ++i, ++pZ) {
				pZ[1] = pZ[0] + m_Zcol;
				}
			m_DelegatePutValues(row,col,num,m_Values);
			}

		void FaceFill (const FACE& Face) const;
		bool FaceTestNodeInCircle (const FACE& Face, const NODE& Node) const;
	};

//=====================================================================================================================


// Typical TIN with N nodes has 2N faces and 3N edges.

#endif
