/**
 *	\file mitree.h <mi32/mitree.h>
 *	\brief Definition for MITREE template
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef INC_MI32_MITREE_H
#define INC_MI32_MITREE_H

#ifndef	INC_MI32_STDDEFNS_H
	#include <mi32/stddefns.h>
#endif

#include <functional>

//! Template to contain hierarchical "tree".
//! Most operations are performed by obtaining a suitable ITERATOR or CONST_ITERATOR that refers to
//! a particular subtree (could be root).
template <typename _ITEMTYPE> class MITREE {

	private:

		#ifndef GENERATING_DOXYGEN_OUTPUT

		// Internal item holder.
		class NODE {
			public:
				NODE *m_pParent;			// Pointer to parent, self if root
				NODE *m_pPrev;				// Pointer to previous sibling or head
				NODE *m_pNext;				// Pointer to next sibling or head
				NODE *m_pHead;				// Pointer to head node of child list, NULL if itself head node or has no children
				_ITEMTYPE *m_pItem;		// Pointer to item, NULL if head node

				// Construct "head" node.
				NODE (
					NODE *pParent
					): m_pParent(pParent), m_pNext(this), m_pPrev(this), m_pHead(0), m_pItem(0) { }

				// Construct "item" node.
				NODE (
					NODE *pNext,
					const _ITEMTYPE& item
					) : m_pParent(pNext->m_pParent), m_pPrev(pNext->m_pPrev), m_pNext(pNext), m_pHead(0), m_pItem(new _ITEMTYPE(item))
					{
					m_pPrev->m_pNext = this;
					m_pNext->m_pPrev = this;
					}

				// Destructor - Does not delete children!
				~NODE (
					) { delete m_pHead; delete m_pItem; }

				// Return pointer to first child node, NULL if none.
				NODE* FirstChild (
					) { return ((m_pHead == 0) ? 0 : m_Head->m_pNext); }

				// Return pointer to first child item, NULL if none.
				_ITEMTYPE* FirstChildItem (
					) { return ((m_pHead == 0) ? 0 : m_Head->m_pNext->m_pItem); }

				// Determine if this is empty (leaf) node.
				bool IsEmpty (
					) const { return (m_pHead == 0); }

				// Determine if this is head node.
				bool IsHead (
					) const { return (m_pItem == 0); }

				// Return pointer to last child node, NULL if none.
				NODE* LastChild (
					) { return ((m_pHead == 0) ? 0 : m_Head->m_pPrev); }

				// Return pointer to last child item, NULL if none.
				_ITEMTYPE* LastChildItem (
					) { return ((m_pHead == 0) ? 0 : m_Head->m_pPrev->m_pItem); }

			private:
				NODE (const NODE&);
				NODE& operator= (const NODE&);
			};

		#endif

	public:

		#ifndef GENERATING_DOXYGEN_OUTPUT
		// Forward declarations.
//		class ITERATOR;					
//		class CONST_ITERATOR;
//		friend class CONST_ITERATOR;
//		friend class ITERATOR;
		#endif

		//! Const iterator class
		class CONST_ITERATOR {
			public:

				//! Default constructor
				CONST_ITERATOR (
					) {}

				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! Internal constructor
				CONST_ITERATOR (
					NODE *pNode
					) : m_pNode(pNode) {}
				#endif

				//! Copy constructor
				CONST_ITERATOR (
					const CONST_ITERATOR& iterator
					) : m_pNode(iterator.m_pNode) {}

				//! Dereference operator, returns reference to templated item
				const _ITEMTYPE& operator* (
					) const { return (*m_pNode->m_pItem); }

				//! Dereference operator, returns pointer to templated item
				const _ITEMTYPE* operator-> (
					) const { return (&**this); }

				//! Pre-increment operator.
				CONST_ITERATOR& operator++ (
					) {
					if (m_pNode != 0) m_pNode = m_pNode->m_pNext;
					return (*this);
					}

				//! Post-increment operator.
				CONST_ITERATOR operator++(int
					) {
					CONST_ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				CONST_ITERATOR& operator-- (
					) {
					if (m_pNode != 0) m_pNode = m_pNode->m_pPrev;
					return (*this);
					}

				//! Post-decrement operator.
				CONST_ITERATOR operator--(int
					) {
					CONST_ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				//! Equality operator
				bool operator== (
					const CONST_ITERATOR& rhs
					) const { return (CONST_ITERATOR::m_pNode == rhs.m_pNode); }

				//! Inequality operator
				bool operator!= (
					const CONST_ITERATOR& rhs
					) const { return (!(*this == rhs)); }

				//! Return CONST_ITERATOR that refers to first child.
				CONST_ITERATOR Begin (
					) const { return (CONST_ITERATOR(m_pNode->GetFirstChild())); }

				//! Determine if specified item is immediate child.
				//! This does a linear search of the child list, so consider efficiency implications when using.
				//! Cost is identical to FindChild, so if intend to use child if found, use FindChild() and check
				//! IsEnd() instead.
				//! Item must support equality comparison operator.
				bool ContainsChild (
					const _ITEMTYPE& item
					) const {
					return (!FindChild(item).IsEnd());
					}

				//! Determine if specified item is immediate child.
				//! This does a linear search of the subtree, so consider efficiency implications when using.
				//! Cost is identical to FindDescendent, so if intend to use child if found, use FindDescendent()
				//! and check IsEnd() instead.
				//! Item must support equality comparison operator.
				bool ContainsDescendent (
					const _ITEMTYPE& item
					) const {
					return (!FindDescendent(item).IsEnd());
					}

				//! Return CONST_ITERATOR that refers to the end of the child list, not a valid tree item.
				CONST_ITERATOR End (
					) const { return (CONST_ITERATOR(m_pNode->m_pHead)); }

				//! Find item in immediate child list.
				//! @return CONST_ITERATOR for item, will reference End() if not found.
				CONST_ITERATOR FindChild (
					const _ITEMTYPE& item
					) const {
					CONST_ITERATOR it = Begin();
					for (; (it != End()); ++it) {
						if (*it == item) break;
						}
					return (it);
					}

				//! Find descendent item.
				//! @return CONST_ITERATOR for item, will reference End() if not found.
				CONST_ITERATOR FindDescendent (
					const _ITEMTYPE& item
					) const {
					for (CONST_ITERATOR itChild = Begin(); (itChild != End()); ++itChild) {
						if (*itChild == item) return (itChild);
						}
					CONST_ITERATOR itChild = Begin();
					for (; (itChild != End()); ++itChild) {
						CONST_ITERATOR itFound = itChild.FindDescendent(item);
						if (!itFound.IsEnd()) return (itFound);
						}
					return (itChild);
					}

				//! Return reference to first child item.
				const _ITEMTYPE& GetFirst (
					) const { return (m_pNode->FirstChildItem()); }

				//! Return reference to last child item.
				const _ITEMTYPE& GetLast (
					) const { return (m_pNode->LastChildItem()); }

				//! Determine if iterator refers to empty (leaf) node.
				bool IsEmpty (
					) const { return (m_pNode == 0 || m_pNode->IsEmpty()); }

				//! Determine if iterator refers to end (non-valid) item.
				bool IsEnd (
					) const { return (m_pNode == 0 || m_pNode->m_pItem == 0); }

				//! Determine if iterator is valid.
				bool IsValid (
					) const { return (m_pNode != 0); }

				//! Return CONST_ITERATOR that refers to the parent.
				CONST_ITERATOR Parent (
					) const { return (CONST_ITERATOR(m_pNode->m_pParent)); }

				//! Return CONST_ITERATOR that refers to the last child.
				CONST_ITERATOR ReverseBegin (
					) const { return (CONST_ITERATOR(m_pNode->LastChild())); }

				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! Internal, get a NODE*
				NODE *GetNode (
					) const { return (m_pNode); }
				#endif

			protected:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				NODE *m_pNode;
				#endif
			};

		//! Iterator.
		class ITERATOR : public CONST_ITERATOR {
			public:

				//! Default constructor
				ITERATOR (
					) {}

				#ifndef GENERATING_DOXYGEN_OUTPUT
				//! Internal constructor
				ITERATOR (
					NODE *pNode
					) : CONST_ITERATOR(pNode) {}
				#endif
					
				//! Dereference operator, returns reference to templated item
				_ITEMTYPE& operator* (				
					) const { return (*m_pNode->m_pItem); }

				//! Dereference operator, returns pointer to templated item
				_ITEMTYPE* operator-> (
					) const { return (&**this); }

				//! Pre-increment operator.
				ITERATOR& operator++ (
					) {
					if (m_pNode != 0) m_pNode = m_pNode->m_pNext;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator++(int
					) {
					ITERATOR temp = *this;
					++*this;
					return (temp);
					}

				//! Pre-decrement operator.
				ITERATOR& operator-- (
					) {
					if (m_pNode != 0) m_pNode = m_pNode->m_pPrev;
					return (*this);
					}

				//! Post-increment operator.
				ITERATOR operator--(int
					) {
					ITERATOR temp = *this;
					--*this;
					return (temp);
					}

				//! Equality operator
				bool operator== (
					const ITERATOR& rhs
					) const { return (CONST_ITERATOR::m_pNode == rhs.m_pNode); }

				//! Inequality operator
				bool operator!= (
					const ITERATOR& rhs
					) const { return (!(*this == rhs)); }

				//! Adopt children of other subtree, adding to end.
				//! The child list of subtree copied from is cleared, does not sort.
				void AdoptChildren (
					ITERATOR& other
					) {
					if (other.IsEmpty()) return;
					NODE *pOtherHead = other.m_pNode->m_pHead;
					for (NODE *pAdopted = pOtherHead->m_pNext; (pAdopted != pOtherHead); pAdopted = pAdopted->m_pNext) {
						pAdopted->m_pParent = m_pNode;
						}
					if (IsEmpty()) {
						delete m_pNode->m_pHead;
						m_pNode->m_pHead = pOtherHead
						other.m_pNode->m_pHead = 0;
						m_pNode->m_pHead->m_pParent = m_pNode;
						}
					else {
						pOtherHead->m_pPrev->m_pNext = m_pHead;
						pOtherHead->m_pNext->m_pPrev = m_pHead->m_pPrev;
						m_pHead->m_pPrev->m_pNext = pOtherHead->m_pNext;
						m_pHead->m_pPrev = pOtherHead->m_pPrev;

						pOtherHead->m_pPrev = pOtherHead->m_pNext = pOtherHead;
						}
					return;
					}

				//! Return ITERATOR that refers to first child.
				ITERATOR Begin (
					) const { return (ITERATOR(m_pNode->m_pHead->m_pNext)); }

				//! Return ITERATOR that refers to the end of the tree, not a valid tree item.
				ITERATOR End (
					) const { return (ITERATOR(m_pNode->m_pHead)); }

				//! Find item in immediate child list.
				//! @return ITERATOR for item, will reference End() if not found.
				ITERATOR FindChild (
					const _ITEMTYPE& item
					) const {
					ITERATOR it = Begin();
					for (; (it != End()); ++it) {
						if (*it == item) break;
						}
					return (it);
					}

				//! Find descendent item.
				//! @return ITERATOR for item, will reference End() if not found.
				ITERATOR FindDescendent (
					const _ITEMTYPE& item
					) const {
					for (ITERATOR itChild = Begin(); (itChild != End()); ++itChild) {
						if (*itChild == item) return (itChild);
						}
					ITERATOR itChild = Begin();
					for (; (itChild != End()); ++itChild) {
						ITERATOR itFound = itChild.FindDescendent(item);
						if (!itFound.IsEnd()) return (itFound);
						}
					return (itChild);
					}

				//! Return pointer to first child item.
				//! Will return NULL if no children.
				_ITEMTYPE* GetFirst (
					) { return (m_pNode->FirstChildItem()); }

				//! Return pointer to last child item.
				//! Will return NULL if no children.
				_ITEMTYPE* GetLast (
					) { return (m_pNode->LastChildItem()); }

				//! Insert item into the place BEFORE this iterator.
				//! @return Reference to new item in tree (not same as 'item' parameter as will be copied).
				_ITEMTYPE& InsertBefore (
					const _ITEMTYPE& item
					) {
					NODE *newnode = new NODE(m_pNode,item);
					return (newnode->m_pItem);
					}

				//! Moves item referenced by this iterator to the back of the tree
				void MoveToBack (
					) {
					if (IsEnd() || m_pNode->m_pNext->IsHead()) return;
					ITERATOR end = End();
					RemoveFromList();
					InsertIntoList(end);
					}

				//! Moves item referenced by this iterator to the front of the tree
				void MoveToFront (
					) {
					if (IsEnd() || m_pNode->m_pPrev->IsHead()) return;
					ITERATOR front = Begin();
					RemoveFromList();
					InsertIntoList(front);
					}

				//! Moves item referenced by this iterator toward the end of the tree.
				void MoveTowardBack (
					) {
					if (IsEnd() || m_pNode->m_pNext->IsHead()) return;
					ITERATOR next = *this;
					++next;
					++next;
					RemoveFromList();
					InsertIntoList(next);
					}

				//! Moves item referenced by this iterator toward the beginning of the tree.
				void MoveTowardFront (
					) {
					if (IsEnd() || m_pNode->m_pPrev->IsHead()) return;
					ITERATOR next = *this;
					--next;
					RemoveFromList();
					InsertIntoList(next);
					}

				//! Return ITERATOR that refers to the parent.
				ITERATOR Parent (
					) const { return (ITERATOR(m_pNode->m_pParent)); }

				//! Insert child item on the end of the tree.
				//! @return Reference to new item in tree (not same as 'item' parameter as will be copied).
				_ITEMTYPE& PushBack (					
					const _ITEMTYPE& item
					) {
					if (m_pNode->m_pHead == 0) m_pNode->m_pHead = new NODE(m_pNode);
					return (End().InsertBefore(item));
					}

				//! Insert child item at the beginning of the tree.
				//! @return Reference to new item in tree (not same as 'item' parameter as will be copied).
				_ITEMTYPE& PushFront (
					const _ITEMTYPE& item
					) {
					if (m_pNode->m_pHead == 0) m_pNode->m_pHead = new NODE(m_pNode);
					return (Begin().InsertBefore(item));
					}

				//! Remove item specified by this iterator from the tree.
				//! Do not use the iterator after using this method as it will be invalid.
				void Remove (
					) {
					if (IsEnd()) return;
					RemoveFromList();
					m_pNode->RemoveAllChildren();
					delete m_pNode;
					}

				void RemoveAllChildren (
					) {
					if (m_pNode->m_pHead == 0) return;
					for (ITERATOR itChild = Begin(); (itChild != End()); ++itChild) {
						itChild.RemoveAllChildren();
						delete itChild.m_pNode;
						}
					delete m_pNode->m_pHead;
					m_pNode->m_pHead = 0;
					}

				//! Remove first child item.
				void RemoveFirstChild (
					) { Remove(Begin()); }

				//! Remove last child item.
				void RemoveLastChild (
					) { Remove(--End()); }
					
				//! Reverse order of child items.
				void Reverse (
					) {
					NODE *pItem = m_pItem->m_pHead;
					do {
						NODE *pNext = pItem->m_pNext;
						pItem->m_pNext = pItem->m_pPrev;
						pItem->m_pPrev = pNext;
						pItem = pNext;
						} while (pItem != m_pItem->m_pHead);
					return;
					}

				//! Return ITERATOR that refers to the last child.
				ITERATOR ReverseBegin (
					) const { return (ITERATOR(m_pNode->LastChild())); }

			private:

				#ifndef GENERATING_DOXYGEN_OUTPUT

				// Insert node referenced by this iterator into position before specified iterator.
				// This method assumes the specified iterator has the same parent as this iterator.
				void InsertIntoList (
					ITERATOR place
					) {
					NODE *pNodeAfter = place.GetNode();
					m_pNode->m_pNext = pNodeAfter;
					m_pNode->m_pPrev = pNodeAfter->m_pPrev;
					pNodeAfter->m_pPrev->m_pNext = m_pNode;
					pNodeAfter->m_pPrev = m_pNode;
					}

				void RemoveFromList (
					) {
					m_pNode->m_pNext->m_pPrev = m_pNode->m_pPrev;
					m_pNode->m_pPrev->m_pNext = m_pNode->m_pNext;
					}

				#endif	// GENERATING_DOXYGEN_OUTPUT
					
			};

		//! Constructor for an empty tree.
		MITREE (								
			) : m_pHead(new NODE(0)) { }

		//! Destructor.
		~MITREE (							
			) {
			Clear();
			delete m_pHead;
			}

		//! Remove all entries from the tree.
		void Clear (						
			) { Root().RemoveAllChildren(); }

		//! Determine if any tree entries exist.
		bool IsEmpty (						
			) const { return (m_pHead->IsEmpty()); }

		//! Return CONST_ITERATOR for root, does not itself refer to valid item.
		CONST_ITERATOR Root (
			) const { return (CONST_ITERATOR(m_pHead)); }

		//! Return ITERATOR for root, does not itself refer to valid item.
		ITERATOR Root (
			) { return (ITERATOR(m_pHead)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		NODE *m_pHead;

		// Unimplemented
		MITREE (const MITREE<_ITEMTYPE>& rhs);
		MITREE<_ITEMTYPE>& operator= (const MITREE<_ITEMTYPE>& rhs);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#endif		// INC_MI32_MITREE_H
