/**
 *	\file mi32/planeconic.h
 * \brief PLANECONIC class definition
 *
 * \if NODOC
 * $Id: planeconic.h_v 1.14 2017/02/08 23:39:29 mju Exp $
 *
 * $Log: planeconic.h_v $
 * Revision 1.14  2017/02/08 23:39:29  mju
 * Add setCircle, setEllipse.
 *
 * Revision 1.13  2017/02/08 23:26:12  mju
 * Rename computeCircle methods to make explicit whether 2D or 3D.
 * Add computeEllipseFrom5Points.
 * Change terminology to "semi-major/minor" instead of x/y axes.
 *
 * Revision 1.12  2013/12/27 18:59:56  mju
 * Add setCenterPoint from dpoint2d.
 *
 * Revision 1.9  2006/06/08 20:08:26  scowan
 * Added rect to ellipse and back methods.
 *
 * Revision 1.7  2005/06/02 19:59:46  scowan
 * Added generating arc using two points and a bulge factor.
 *
 * Revision 1.6  2005/06/01 20:15:59  scowan
 * Added method to generate three points defining the arc or circle.
 *
 * Revision 1.3  2004/05/06 20:57:00  scowan
 * Added coordinate conversion methods.
 *
 * Revision 1.1  2004/04/15 16:24:51  scowan
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_PLANECONIC_H
#define	INC_MI32_PLANECONIC_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifdef GEOMDLL
	#define GEOMLIBEXPORT MI_DLLEXPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define GEOMLIBEXPORT MI_DLLIMPORT
	#define GEOMLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations
struct DPOINT3D;
class POLYLINE;
class TRANS2D_AFFINE;
class TRANS2D_MAPGEN;
class TRANS3D;
#endif

//=====================================================================================================================

//! Container for conic section.
//! Currently supports circle and ellipse forms, including rotation and angle range for arc.
//! Arc can be open, closed with chord or as wedge.
class GEOMLIBCLASSEXPORT PLANECONIC {
	public:

		//! Closure for connecting arc endpoints.
		enum CLOSURE {
			CLOSURE_Open,				//!< Not closed
			CLOSURE_Chord,				//!< Connect segment from start to end, assumes reasonable angle range
			CLOSURE_Wedge,				//!< Circular or Elliptical arc with closure through center
			CLOSURE_Complete			//!< Complete circle or ellipse
			};
			
		//! Default ctor
		PLANECONIC ();
			
		//! Copy ctor
		PLANECONIC (
			const PLANECONIC &rhs
			);
			
		//! Assigment
		PLANECONIC& operator= (
			const PLANECONIC &rhs
			);

		//! Clear to initial state (no conic defined)
		void Clear ();

		//! Compute circular arc based on three 2D points.
		//! @return true if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeArc (
			const DPOINT2D &p1, 
			const DPOINT2D &p2, 
			const DPOINT2D &p3
			);
			
		//! Compute circular arc based on three 3D points.
		//! Extrusion vertex calculated based on the plane defined by the three points
		//! @return true if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeArc (
			const DPOINT3D &p1, 
			const DPOINT3D &p2, 
			const DPOINT3D &p3
			);
			
		//! Compute circular arc based on endpoints and bulge distance.
		//! @return true if successful, false if not (bulge == 0 or 2 points are the same)
		bool ComputeArcByBulge (
			const DPOINT2D &p1, 
			const DPOINT2D &p2, 
			double bulge					//!< If positive, generates the arc on the left side of the line, if negative, on the right side
			);
			
		//! Compute circular arc based on 3D endpoints and bulge distance.
		//! @return true if successful, false if not (bulge == 0 or 2 points are the same)
		bool ComputeArcByBulge (
			const DPOINT3D &p1, 
			const DPOINT3D &p2, 
			double bulge,
			const DPOINT3D &Normal 
			);
			
		//! Compute circle based on two points on diameter.
		//! @return true if successful, false if not (Both points are the same).
		bool ComputeCircleFrom2Points2D (
			const DPOINT2D &p1, 
			const DPOINT2D &p2
			);

		//! Compute circle based on three 2D points.
		//! @return true if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeCircleFrom3Points2D (
			const DPOINT2D &p1,
			const DPOINT2D &p2,
			const DPOINT2D &p3
			);
			
		//! Compute circle based on three 3D points.
		//! Extrusion vertex calculated based on the plane defined by the three points
		//! @return true if successful, false if not (All 3 points lie on a line or 2 points are the same)
		bool ComputeCircleFrom3Points3D (
			const DPOINT3D &p1, 
			const DPOINT3D &p2, 
			const DPOINT3D &p3
			);

		//! Compute distance from point to edge
		//! @return Distance from point to edge.
		double ComputeDistanceFrom (
			const DPOINT2D &pt
			) const;
			
		//! Compute distance from point to edge
		//! @return Distance from point to edge
		double ComputeDistanceFrom (
			const DPOINT3D &pt
			) const;

		//! Compute 2D ellipse based on inscribed rectangle given
		//! @return true if successful, false if not (More than 5 points or area of zero)
		bool ComputeEllipseFromRect (
			const POLYLINE &PolyLine
			);

		//! Compute 2D ellipse from 5 points on boundary.
		//! @return true if successful, false if points do not define an ellipse.
		bool ComputeEllipseFrom5Points (
			const DPOINT2D *pts
			);
			
		//! Compute extents
		void ComputeExtents (
			DRECT3D &Extents
			) const;
			
		//! Compute circumscribed rectangle from 2D ellipse
		//! @return true if successful, false if not (Not an ellipse)
		bool ComputeRectFromEllipse (
			POLYLINE &PolyLine				//!< Place to put the computed rectangle to
			) const;
			
		//! Convert conic using affine transformation
		//! @return true if conversion valid, 'false' if approximated
		bool ConvertForward (
			const TRANS2D_AFFINE &taf
			);
		
		//! Convert conic using generic transformation		
		//! @return true if conversion valid, 'false' if approximated
		bool ConvertForward (
			const TRANS2D_MAPGEN &tmg
			);

		//! Convert conic using affine transformation
		//! @return true if conversion valid, 'false' if approximated
		bool ConvertInverse (
			const TRANS2D_AFFINE &taf
			);
		
		//! Convert conic using generic transformation		
		//! @return true if conversion valid, 'false' if approximated
		bool ConvertInverse (
			const TRANS2D_MAPGEN &tmg
			);

		//! Given a circular conic (arc or circle), generate a set of three points that will define the arc or circle.
		bool GenerateArc3Points (
			DPOINT2D &p1, 
			DPOINT2D &p2, 
			DPOINT2D &p3
			) const;

		//! Generate a POLYLINE from 'this' using the 'MinChordDist' to determine the number of points used to create 
		//! the line or polygon.
		ERRVALUE GenerateConicPoints (
			POLYLINE &PolyLine,
			double MinChordDist = 0.0					//!< Default is to use %5 of the Radius length
			) const;
			
		//! Get center in absolute coordinates.
		//! @return Center point.
		const DPOINT3D& GetCenterPoint () const
			{ return (m_Center); }
			
		//! Return closure mode.
		CLOSURE GetClosure () const
			{ return (m_Closure); }
			
		//! Get the end angle.
		//! @return End angle in radians starting from positive X axis.
		double GetEndAngle () const
			{ return (m_EndAngle); }
			
		//! Get extrusion point in absolute coordinates.
		//! This point, along with the center point, define the normal vertex to the plane that this conic resides.
		//! @return Extrusion point of the conic
		const DPOINT3D& GetExtrusionPoint () const
			{ return (m_Extrusion); }
			
 		//! Get rotation with respect to the X axis.
		//! @return Rotation in radians.
		double GetRotation () const
			{ return (m_Rotation); }

		//! Get semi-major axis length for ellipse or circle.
		double GetSemiMajorAxis () const
			{ return (m_SemiMajor); }

		//! Get semi-minor axis length for ellipse or circle.
		double GetSemiMinorAxis () const
			{ return (m_SemiMinor); }

		//! Get the start angle.
		//! @return Start angle in radians starting from positive X axis.
		double GetStartAngle () const
			{ return (m_StartAngle); }

		//! Get the X radius.			
		//! @return X radius.			
		DEPRECATED double GetXRadius () const
			{ return (GetSemiMajorAxis()); }

		//! Get the Y radius.			
		//! @return Y radius.			
		DEPRECATED double GetYRadius () const
			{ return (GetSemiMinorAxis()); }

		//! Is extrusion vertex valid?
		//! @return true if extrusion vertex is valid, 'false' if not
		bool HasExtrusion () const
			{ return (m_bHasExtrusion); }
			
		//! Is the conic element circular?
		//! @return true if circular, 'false' if elliptical
		bool IsCircular () const
			{ return (m_SemiMajor == m_SemiMinor); }
		
		//! Is the conic element defined?
		bool IsDefined () const
			{ return (m_SemiMajor > 0); }
			
		//! Is the point inside the conic
		//! If the conic is extruded, it will determine if the point lies inside the projected conic to the XY plane
		bool IsPointInside (
			const DPOINT2D &pt
			) const;
			
		//! Does the point lie inside the conic defined on the plane
		bool IsPointInside (
			const DPOINT3D &pt
			) const;
			
		//! Get center in absolute coordinates.
		void SetCenterPoint (
			const DPOINT2D &point
			) { m_Center.x = point.x; m_Center.y = point.y; m_Center.z = 0; }
			
		//! Get center in absolute coordinates.
		void SetCenterPoint (
			const DPOINT3D &point
			) { m_Center = point; }

		//! Set circle.
		void SetCircle (
			const DPOINT2D &center,
			double radius
			);
			
		//! Set the closure of the conic
		void SetClosure (
			CLOSURE Closure
			) { m_Closure = Closure; }

		//! Set ellipse center and axes.
		//! Will automatically set type as circle if axes are equal.
		void SetEllipse (
			const DPOINT2D &center,
			double SemiMajor,
			double SemiMinor
			);

		//! Set ellipse axes.
		//! Will automatically set type as circle if axes are equal.
		void SetEllipseAxes (
			double SemiMajor,
			double SemiMinor
			) { m_SemiMajor = SemiMajor; m_SemiMinor = (SemiMinor > 0) ? SemiMinor : SemiMajor; }
			
		//! Set the end angle of the arc or elliptical arc in radians.
		void SetEndAngle (
			double angle
			) { m_EndAngle = angle; }
			
		//! Set extrusion point in absolute coordinates.
		//! This point, along with the center point, define the normal vertex to the plane that this conic resides.
		void SetExtrusionPoint (
			const DPOINT3D &Extrusion
			);
			
		//! Set the start angle of the arc or elliptical arc in radians.
		void SetStartAngle (
			double angle
			) { m_StartAngle = angle; }

		//! Set the X radius.			
		DEPRECATED void SetXRadius (
			double SemiMajor
			) { SetEllipseAxes(SemiMajor,m_SemiMinor); }

		//! Set the Y radius.			
		DEPRECATED void SetYRadius (
			double SemiMinor
			) { SetEllipseAxes(m_SemiMajor,SemiMinor); }
			
		//! Set rotation angle between X axis and semi-major axis / line of symmetry.
		void SetRotation (
			double rotation
			) { m_Rotation = rotation; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DPOINT3D m_Center;					// Center point
		double m_StartAngle;						// Start angle of arc in radians
		double m_EndAngle;						// End angle of arc in radians
		double m_SemiMajor;						// Semi-major axis for ellipse
		double m_SemiMinor;						// Semi-minor axis for ellipse
		double m_Rotation;						// Rotation with respect to semi-major axis / line of symmetry
		DPOINT3D m_Extrusion;					// This point, along with the center point, define the normal vertex to the plane that this conic resides.
		bool m_bHasExtrusion;					// Extrusion vertex is valid
		CLOSURE m_Closure;

		double ComputeAngleRadius (double angle) const;
		void ComputeArcEndPoints (DPOINT2D &StartPt, DPOINT2D &EndPt) const;
		void ComputeStartEndAngle (const DPOINT2D &p1, const DPOINT2D &p2, const DPOINT2D &p3);
		void GenerateTrans3D (TRANS3D &t3d) const;
		#endif		// GENERATING_DOXYGEN_OUTPUT
			
	};

//=====================================================================================================================

#undef GEOMLIBEXPORT
#undef GEOMLIBCLASSEXPORT

#endif	// INC_MI32_PLANECONIC_H 
