/**
 * \file mi32/printmedia.h
 * \brief PRINTMEDIA definitions
 *
 * \if NODOC
 * $Id: printmedia.h_v 1.12 2013/09/05 22:06:34 mju Exp $
 *
 * $Log: printmedia.h_v $
 * Revision 1.12  2013/09/05 22:06:34  mju
 * Make some methods non-inline.
 *
 * Revision 1.11  2013/08/30 15:06:24  mju
 * Add setTotalWidth/Height.
 *
 * Revision 1.10  2013/08/28 15:56:48  mju
 * Add SetDimensions using printmedia.
 *
 * Revision 1.9  2013/08/26 20:16:03  mju
 * Add getResolution using screenunit.
 *
 * Revision 1.8  2013/08/26 19:06:35  mju
 * Add getMargin/Interior/Total methods using screenunit.
 *
 * Revision 1.7  2013/08/22 21:46:27  mju
 * Rename class.
 *
 * Revision 1.6  2013/08/21 21:36:41  mju
 * Add setHeight/Width.
 *
 * Revision 1.5  2013/08/21 20:58:05  mju
 * Add methods to set individual values.
 *
 * Revision 1.4  2013/08/20 16:53:28  mju
 * Add inlines for getInteriorHeight/Width.
 *
 * Revision 1.3  2013/08/20 16:27:33  mju
 * Add changeOrient and simplify.
 *
 * Revision 1.2  2013/08/15 16:50:26  mju
 * Add iniRead/Write, hasSameDimensions, isEqual.
 *
 * Revision 1.1  2013/05/15 15:51:26  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_PRINTMEDIA_H
#define	INC_MI32_PRINTMEDIA_H

#ifndef	INC_MI32_PRINTSIZE_H
#include <mi32/printsize.h>
#endif

#ifndef	INC_MI32_ORIENT8_H
#include <mi32/orient8.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

//=====================================================================================================================

//! Container for print media size, margins and resolution.
class PRINTMEDIA {
	public:

		//! Default constructor.
		PRINTMEDIA ();

		//! Compare for equality.
		bool operator== (
			const PRINTMEDIA& rhs
			) const { return (IsEqual(rhs)); }

		//! Compare for inequality.
		bool operator!= (
			const PRINTMEDIA& rhs
			) const { return (!IsEqual(rhs)); }

		//! Calculate pixel size in specified or stored units.
		DPOINT2D CalcPixelSize ( 
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			) const;

		//! Change orientation.
		void ChangeOrient (
			ORIENT8 orient									//!< Orientation change relative to current
			);

		//! Get bottom margin in stored units.
		double GetMarginBottom () const
			{ return (m_MarginBottom); }

		//! Get bottom margin in specified units.
		double GetMarginBottom (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get left margin in stored units.
		double GetMarginLeft () const
			{ return (m_MarginLeft); }

		//! Get left margin in specified units.
		double GetMarginLeft (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get right margin in stored units.
		double GetMarginRight () const
			{ return (m_MarginRight); }

		//! Get right margin in specified units.
		double GetMarginRight (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get top margin in stored units.
		double GetMarginTop () const
			{ return (m_MarginTop); }

		//! Get top margin in specified units.
		double GetMarginTop (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get height of interior area between margins in stored units.
		double GetInteriorHeight () const
			{ return (m_Size.GetHeight() - m_MarginTop - m_MarginBottom); }

		//! Get height of interior area between margins in specified units.
		double GetInteriorHeight (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get width of interior area between margins in stored units.
		double GetInteriorWidth () const
			{ return (m_Size.GetWidth() - m_MarginLeft - m_MarginRight); }

		//! Get width of interior area between margins in specified units.
		double GetInteriorWidth (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get resolution in pixels per stored unit.
		const DPOINT2D& GetResolution () const
			{ return (m_Resolution); }

		//! Get resolution in specified units.
		double GetResolution (
			SCREENUNIT units						//!< Units (of denominator)
			) const;

		//! Get total media size.
		const PRINTSIZE& GetSize () const
			{ return (m_Size); }

		//! Get height of total area including margins in specified units.
		double GetTotalHeight (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get width of total area including margins in specified units.
		double GetTotalWidth (
			SCREENUNIT units						//!< Units, if SCREENUNIT_Pixels will base on resolution setting
			) const;

		//! Get stored units.
		SCREENUNIT GetUnits () const
			{ return (m_Size.GetUnits()); }

		//! Determine if has same dimensions as other PRINTMEDIA instance.
		bool HasSameDimensions (
			const PRINTMEDIA& other
			) const;

		//! Read entry from .ini file.
		//! @return true if entry in file, false if not
		bool IniRead (
			INIHANDLE handle,								//!< Ini Handle created by IniOpen()
			const char *group,							//!< Group to look for ini entry, App Name if =0
			const char *field								//!< Field to read from
			);

		//! Write entry to .ini file.
		void IniWrite (
			INIHANDLE handle,								//!< Ini Handle created by IniOpen()
			const char *group,							//!< Group to write ini entry, App Name if =0
			const char *field								//!< Field to write entry to
			) const;

		//! Determine if equal.
		bool IsEqual (
			const PRINTMEDIA& rhs
			) const;

		//! Set all dimension values from PRINTMEDIA.
		void SetDimensions (
			const PRINTMEDIA& media						//!< Values to set
			);

		//! Set all dimension values.
		ERRVALUE SetDimensions (
			const PRINTSIZE& Size,						//!< Media size
			double MarginTop,								//!< Top margin in media size units
			double MarginBottom,							//!< Bottom margin in media size units
			double MarginLeft,							//!< Left margin in media size units
			double MarginRight							//!< Right margin in media size units
			);

		//! Set media height.
		void SetHeight (
			double height
			) { m_Size.SetHeight(height); }

		//! Set media width.
		void SetWidth (
			double width
			) { m_Size.SetWidth(width); }

		//! Set bottom margin.
		bool SetMarginBottom (
			double MarginBottom							//!< Bottom margin in media size units
			);

		//! Set left margin.
		bool SetMarginLeft (
			double MarginLeft								//!< Left margin in media size units
			);

		//! Set right margin.
		bool SetMarginRight (
			double MarginRight							//!< Right margin in media size units
			);

		//! Set top margin.
		bool SetMarginTop (
			double MarginTop								//!< Top margin in media size units
			);

		//! Set resolution.
		void SetResolution (
			double resolution,							//!< Resolution in pixels/unit
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			);

		//! Set resolution.
		void SetResolution (
			const DPOINT2D& resolution,				//!< Resolution in pixels/unit
			SCREENUNIT units = SCREENUNIT_COUNT		//!< Units, default to use same as stored
			);

		//! Set media size.
		void SetSize (
			const PRINTSIZE& size
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		PRINTSIZE m_Size;
		double m_MarginTop;
		double m_MarginBottom;
		double m_MarginLeft;
		double m_MarginRight;
		DPOINT2D m_Resolution;
		#endif
	};

//=====================================================================================================================

#endif	// INC_MI32_PRINTMEDIA_H
