/**
 * \file mi32/rationalpoly.h
 * \brief RATIONALPOLY class definitions
 *
 * \if NODOC
 * $Id: rationalpoly.h_v 1.6 2014/05/28 17:19:34 mju Exp $
 *
 * $Log: rationalpoly.h_v $
 * Revision 1.6  2014/05/28 17:19:34  mju
 * Add import.
 *
 * Revision 1.5  2014/05/23 19:00:17  mju
 * Return const references for scale/offset.
 *
 * Revision 1.4  2014/05/22 19:32:50  mju
 * Add enhance method.
 *
 * Revision 1.3  2014/05/21 16:43:07  mju
 * Add clear method.
 *
 * Revision 1.2  2014/01/16 20:21:42  mju
 * Fix dox.
 *
 * Revision 1.1  2013/05/01 21:09:00  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_MI32_RATIONALPOLY_H
#define	INC_MI32_RATIONALPOLY_H

#ifndef	INC_MI32_CTRLPT_H
#include <mi32/ctrlpt.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT

#ifdef GEOMDLL
	#define CLASSLIBEXPORT MI_DLLCLASSEXPORT
#else
	#define CLASSLIBEXPORT MI_DLLCLASSIMPORT
#endif

class MISTRING;
class FILEPATH;

#endif

//=====================================================================================================================

//! Rational polynomial coordinate conversion.
class CLASSLIBEXPORT RATIONALPOLY {
	public:

		//! Default constructor.
		RATIONALPOLY ();

		//! Copy constructor.
		RATIONALPOLY (
			const RATIONALPOLY& rhs
			);

		//! Assignment operator.
		RATIONALPOLY& operator= (
			const RATIONALPOLY& rhs
			);

		//! Destructor.
		~RATIONALPOLY ();

		//! Clear, resetting to undefined.
		void Clear ();

		//! Compute coefficients from control points.
		ERRVALUE Compute (
			const CTRLPT3D *pts,								//!< Control points, source is image, dest is geographic/height
			UINT32 NumPoints,									//!< Number of control points
			const DRECT2D& ImageRect,						//!< Image rectangle
			double MinHeight = 0,							//!< Minimum height (z) value for normalization
			double MaxHeight = 0								//!< Maximum height (z) value for normalization
			);

		//! Convert single (longitude, latitude, height) point to image (column, row).
		//! @return true if successful, false if not.
		bool Convert (
			const DPOINT3D& ptGeogH,						//!< longitude, latitude, height
			DPOINT2D& ptImage									//!< Image coordinates returned
			) const;

		//! Enhance predefined transformation using terrain-dependent control points.
		//! Control point dest.z values must include any geoid height adjustment for accurate match.
		ERRVALUE Enhance (
			const CTRLPT3D *pts,								//!< Control points, source is image, dest is geographic/height
			UINT32 NumPoints									//!< Number of control points
			);

		//! Get coefficients.
		void GetCoefficients (
			double NumX[20],									//!< X (column) numerator coefficients
			double DenX[20],									//!< X (column) denominator coefficients
			double NumY[20],									//!< Y (row) numerator coefficients
			double DenY[20]									//!< Y (row) denominator coefficients
			) const;

		//! Get formula display string.
		void GetFormulaDisplay (
			MISTRING& FormualStr								//!< String containing formula returned
			) const;

		//! Get normalization offset for geographic/height coordinates.
		const DPOINT3D& GetOffsetGeogH () const;

		//! Get normalization offset for image coordinates.
		const DPOINT2D& GetOffsetImage () const;

		//! Get normalization scale for geographic/height coordinates.
		const DPOINT3D& GetScaleGeogH () const;

		//! Get normalization scale for image coordinates.
		const DPOINT2D& GetScaleImage () const;

		//! Get values as string.
		void GetStringFormatted (
			MISTRING& str										//!< String to contain result
			) const;

		//! Import definition from RPC file.
		ERRVALUE Import (
			const FILEPATH& filepath						//!< Path to file
			);

		//! Determine if transformation is defined.
		bool IsDefined () const
			{ return (m_pPriv != 0); }

		//! Parse string for values.
		//! @return true if successful, false if not.
		bool Parse (
			const MISTRING& str								//!< String to parse
			);

		//! Set values.
		void SetValues (
			const double NumX[20],							//!< X (column) numerator coefficients
			const double DenX[20],							//!< X (column) denominator coefficients
			const double NumY[20],							//!< Y (row) numerator coefficients
			const double DenY[20],							//!< Y (row) denominator coefficients
			const DPOINT3D& OffsetGeogH,					//!< Normalization offset for geographic/height coordinates, applied before scaling
			const DPOINT3D& ScaleGeogH,					//!< Normalization scale for geographic/height coordinates
			const DPOINT2D& OffsetImage,					//!< Normalization offset for image coordinates
			const DPOINT2D& ScaleImage						//!< Normalization scale for image coordinates
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		void GetExclusive ();
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#undef CLASSLIBEXPORT

#endif	// INC_MI32_RATIONALPOLY_H 
