/**
 * \file shapeproperties.h
 * \brief SHAPEPROPERTIES definitions
 *
 * \if NODOC
 * $Id: shapeproperties.h_v 1.3 2014/09/10 21:41:58 mju Exp $
 *
 * $Log: shapeproperties.h_v $
 * Revision 1.3  2014/09/10 21:41:58  mju
 * Add members for Reock and ConvexHull compactness values.
 *
 * Revision 1.2  2014/09/09 15:09:37  mju
 * Rename value and add reock/convexhull.
 *
 * Revision 1.1  2014/08/14 20:00:48  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_SHAPEPROPERTIES_H
#define	INC_MI32_SHAPEPROPERTIES_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class POLYLINE;
class SUBREGION2D;
#endif

//=====================================================================================================================

//! Container and calculator for various shape properties.
class SHAPEPROPERTIES {
	public:

		enum VALUES {
			VALUE_None =								0,
			VALUE_Compactness_MI =					0x0001,	//!< Wenwen, Goodchild & Church (2013) (A2 / (2*pi*I)) where I is the moment of inertia about centroid
			VALUE_Compactness_IPQ =					0x0002,	//!< Osserman (1978) Iso-Perimetric Quotient (4*pi*A/P2)
			VALUE_Compactness_Richardson =		0x0004,	//!< Richardson (1961) Compactness Ratio (2*sqrt(pi*A)/P)
			VALUE_Compactness_Gibbs =				0x0008,	//!< Gibbs (1961) Compactness (4*A/L2) where L is length of longest line between two points on perimeter
			VALUE_Compactness_Reock =				0x0010,	//!< Reock (1961) Compactness (A / area of minimum spanning circle)
			VALUE_Compactness_ConvexHull =		0x0020,	//!< (A / area of convex hull)
			VALUE_GrainShapeIndex =					0x0100,	//!< P / Lmax
			VALUE_Circularity_MaxDist =			0x0200,	//!< sqrt(A / pi) / maxDtoCent)
			VALUE_Circularity_MinMaxDistRatio =	0x0400,	//!< sqrt(|minDtoCent| / maxDtoCent) * sign(minDtoCent)
			VALUE_Complexity_Schwartzberg =		0x0800,	//!< Reciprocal of Richardson Compactness Ratio
			VALUE_OrientationMax =					0x1000,	//!< Orientation based on maximum distance between pairs of points on perimeter (degrees from positive X axis toward positive Y axis)
			VALUE_ElongationMax =					0x2000,	//!< L / W based on maximum distance, typically not meaningful if has holes or significant concavity
			VALUE_AspectRatioMax =					0x4000,	//!< W / L based on maximum distance, typically not meaningful if has holes or significant concavity
			VALUES_All =								0x7FFF
			// Old "fuzzy" names:
			//		Form Ratio = Compactness_Gibbs / 4
			// 	Compactness = Compactness_Richardson
			// 	Thinness = Compactness_IPQ
			//		Simplicity (MeanSegmentLength2 / Area) [probably not correct or meaningful]
			//		Normalized Dispersion (PI * (mean distance from centroid to boundary) / area) [does not appear to have been computed correctly]
			};

		//! Constructor.
		explicit SHAPEPROPERTIES (
			VALUES values = VALUES_All
			);

		//! Destructor.
		~SHAPEPROPERTIES ();

		//! Get specified value.
		//! If value was not specified to be computed then the returned value will be 0.
		double GetValue (
			VALUES value
			) const;

		//! Process subregion.
		ERRVALUE Process (
			const SUBREGION2D& subregion		//!< Shape to process
			);

		//! Process single polygon.
		ERRVALUE Process (
			const POLYLINE& polygon				//!< Polygon to process
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		DPOINT2D m_centroid;
		double m_area;
		double m_perimeter;
		double m_I;								// Moment of inertia
		double m_DistanceCentroidMin;		// Minimum distance from centroid to point on perimeter, negative if centroid outside
		double m_DistanceCentroidMax;		// Maximum distance from centroid to point on perimeter
		double m_LengthMax;					// Length (greatest distance between pair of points on perimeter)
		double m_WidthMax;					// Width (range orthogonal to direction based on Length)
		double m_AngleMax;					// Orientation angle between farthest pair of points on perimeter, radians (0 - pi)
		double m_AreaConvexHull;			// Area of convex hull
		double m_AreaMinCircle;				// Area of minimum spanning circle
		VALUES m_values;

		ERRVALUE ProcessOuter (const POLYLINE& polygon);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SHAPEPROPERTIES::VALUES)
#endif

//=====================================================================================================================

#endif	// INC_MI32_SHAPEPROPERTIES_H
