/**
 * \file mi32/wrect2d.h
 * \brief WRECT2D definition
 *
 * \if NODOC
 * $Id: wrect2d.h_v 1.2 2016/02/03 22:45:45 mju Exp $
 *
 * $Log: wrect2d.h_v $
 * Revision 1.2  2016/02/03 22:45:45  mju
 * Remove checksize.
 *
 * Revision 1.1  2013/12/30 14:33:54  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_MI32_WRECT2D_H
#define	INC_MI32_WRECT2D_H

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

//-------------------------------------------------------------------------------------------------------------------

//! 2D rectangle using 16-bit integer coordinates.
struct WRECT2D {
	INT16 xinit;							//!< Minimum X coordinate (inclusive)
	INT16 yinit;							//!< Minimum Y coordinate (inclusive)
	INT16 xlast;							//!< Maximum X coordinate (inclusive)
	INT16 ylast;							//!< Maximum Y coordinate (inclusive)

	//! Default constructor, initializes to invalid (empty) rectangle.
	WRECT2D (
		): xinit(INT16_MAX), yinit(INT16_MAX), xlast(INT16_MIN), ylast(INT16_MIN) { }

	//! Construction from X/Y ranges.
	WRECT2D (
		INT16 xi,
		INT16 yi,
		INT16 xl,
		INT16 yl
		) : xinit(xi), yinit(yi), xlast(xl), ylast(yl) { }

	//! Check if contains specified point.
	bool Contains (
		INT16 x,
		INT16 y
		) const { return (x >= xinit && x <= xlast && y >= yinit && y <= ylast); }

	//! Check if contains specified point.
	bool Contains (
		const WPOINT2D& pt
		) const { return (pt.x >= xinit && pt.x <= xlast && pt.y >= yinit && pt.y <= ylast); }

	//! Check if contains specified rectangle.
	bool Contains (
		const WRECT2D& rhs
		) const { return (rhs.xinit >= xinit && rhs.xlast <= xlast && rhs.yinit >= yinit && rhs.ylast <= ylast); }

	//! Expand rectangle by specified amount.
	void Expand (
		const INT16 value
		) { xinit -= value; yinit -= value; xlast += value; ylast += value; }
	
	//! Expand rectangle by specified amounts.
	void Expand (
		const INT16 dx,
		const INT16 dy
		) { xinit -= dx; yinit -= dy; xlast += dx; ylast += dy; }

	//! Extend rectangle to include specified X,Y point.
	void Extend (
		const INT16 dx,
		const INT16 dy
		) {
		if (dx < xinit) xinit = dx;
		if (dy < yinit) yinit = dy;
		if (dx > xlast) xlast = dx;
		if (dy > ylast) ylast = dy;
		}

	//! Return height (Y size) of rectangle.
	UINT16 GetHeight () const
		{ return (ylast - yinit + 1); }

	//! Return width (X size) of rectangle.
	UINT16 GetWidth () const
		{ return (xlast - xinit + 1); }

	//! Check if rectangle is non-empty (has positive area).
	bool IsNonEmpty () const
		{ return (xinit < xlast && yinit < ylast); }

	//! Check if rectangle is valid.
	bool IsValid () const
		{ return (xinit <= xlast && yinit <= ylast); }

	//! Check if rectangle overlaps.
	bool Overlaps (
		const WRECT2D& rhs
		) const { return (!(rhs.xlast < xinit || rhs.xinit > xlast || rhs.ylast < yinit || rhs.yinit > ylast)); }
	
	//! Set given (x1,y1), (x2,y2)
	void Set (
		INT16 x1,
		INT16 y1,
		INT16 x2,
		INT16 y2
		) { xinit = x1; yinit = y1; xlast = x2; ylast = y2; }

	//! Sets the initial values to the maximum possible and the final values to the minimum possible, thus making the rectangle "invalid".
	//! Setting a rectangle to invalid means that no point will be in it.  It also makes it easy 
	//! to compute the min/max of a bunch of points.  You just iterate through all the points 
	//! and Extend() the rectangle.  Normally, you'd have to handle the first point specially.
	void SetInvalid ()
		{ xinit = yinit = INT16_MAX; xlast = ylast = INT16_MIN; }

	//! Perform byte-order swapping.
	void SwapBytes ()
		{ ::SwapBytes(xinit); ::SwapBytes(xlast); ::SwapBytes(yinit); ::SwapBytes(ylast); }

	};

//-------------------------------------------------------------------------------------------------------------------

#endif	// INC_MI32_WRECT2D_H
