/*****************************************************************************
 * \file mi32/xmldata.h
 * \brief XMLDATA definition
 *	
 * \if NODOC
 * $Id: xmldata.h_v 1.2 2013/09/19 20:54:54 mju Exp $
 *
 * $Log: xmldata.h_v $
 * Revision 1.2  2013/09/19 20:54:54  mju
 * Add iniRead/Write.
 *
 * Revision 1.1  2013/09/12 22:20:36  mju
 * Initial revision
 *
 * \endif
 ****************************************************************************/

#ifndef INC_MGUI_XMLDATA_H
#define INC_MGUI_XMLDATA_H

#ifndef INC_MI32_XML_H
#include <mi32/xml.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
struct COLOR;
#endif

//=====================================================================================================================

//! Simple data record stored as XMLDOC.
//!
//! This class is used to pass values to MGUI::LAYOUT_PANE_XMLFORM and MGUI::DLG_XML.
//! It can also be used as a general-purpose container where the structure can vary.
//! Contents of the document looks something like this...
//!
//! @code
//!	<?xml version="1.0" encoding="utf-8"?>
//! 	<record>
//! 		<member id="FirstName">John</member>
//! 		<member id="LastName">Doe</member>
//! 		<member id="Age">45</member>/>
//! 	</record>
//! @endcode
//!
class XMLDATA : public XMLDOC {
	public:

		enum PROPERTY {
			PROPERTY_Value = 0,				//!< The "value" attribute
			PROPERTY_Min,
			PROPERTY_Max,
			};

		//! Default constructor
		XMLDATA ();

		//! Copy constructor.
		XMLDATA (
			const XMLDATA&
			);

		//! Construct from XMLNODE.
		//! Copies initial values from the given node
		XMLDATA (
			const XMLNODE *InitialValues
			);

		//! Assignment.
		XMLDATA& operator= (
			const XMLDATA& rhs
			);

		//! Retrieve MISTRING value for specified item id.
		//! Always stores strings as the \<member\> tag's contents
		//! @return false if the id wasn't found, true if it was
		bool GetValue (
			const char *id,			//!< ID of the item to get value of
			MISTRING& value			//!< Value returned, unchanged if not found
			) const;

		//! Retrieve (double) value for specified item id.
		//! Some controls have more than one value.  For example,
		//! a range control would have a PROPERTY_Min and a PROPERTY_Max.
		//! Currently, only numeric controls can have more than one property
		//! @return false if the id wasn't found, true if it was
		bool GetValue (
			const char *id,			//!< ID of the item to get value of
			double& value,				//!< Value returned, unchanged if not found
			XMLDATA::PROPERTY property = PROPERTY_Value
			) const;

		//! Retrieve INT32 value for specified item id.
		//! Some controls have more than one value.  For example,
		//! a range control would have a PROPERTY_Min and a PROPERTY_Max.
		//! Currently, only numeric controls can have more than one property
		//! @return false if the id wasn't found, true if it was
		bool GetValue (
			const char *id,			//!< ID of the item to get value of
			INT32& value,				//!< Value returned, unchanged if not found
			XMLDATA::PROPERTY property = PROPERTY_Value
			) const;

		//! Retrieve "bool" value for specified item id.
		//! @return false if the id wasn't found, true if it was
		bool GetValue (
			const char *id,			//!< ID of the item to get value of
			bool& value					//!< Value returned, unchanged if not found
			) const;

		//! Retrieve COLOR value for specified item id.
		//! @return false if the id wasn't found, true if it was
		bool GetValue (
			const char *id,			//!< ID of the item to get value of
			COLOR& value				//!< Value returned, unchanged if not found
			) const;

		//! Determine if contains any values.
		bool HasAnyValues () const;

		//! Read values from INI file.
		//! @return true if entry exists, false if not.
		bool IniRead (
			INIHANDLE inihandle,
			const char *IniGroup,
			const char *IniName
			);

		//! Write values to INI file.
		//! If no values exist, entry will be deleted.
		void IniWrite (
			INIHANDLE inihandle,
			const char *IniGroup,
			const char *IniName
			) const;

		//! Set MISTRING value for specified item id.
		void SetValue (
			const char *id,			//!< ID of the item to get value of
			const MISTRING& value	//!< Value to set
			);

		//! Set numeric value for specified item id.
		void SetValue (
			const char *id,			//!< ID of the item to get value of
			double value,				//!< Value to set
			XMLDATA::PROPERTY property = PROPERTY_Value
			);

		//! Set "bool" value for specified item id.
		void SetValue (
			const char *id,			//!< ID of the item to get value of
			bool value					//!< Value to set
			);

		//! Set COLOR value for specified item id.
		void SetValue (
			const char *id,			//!< ID of the item to get value of
			const COLOR& value		//!< value to set
			);

		//! Validate using reference.
		//! This will remove any members that do not exist in the reference.
		//! Where members do exist, they will be checked for allowed values where possible.
		//! @return true if changed, false if not.
		bool Validate (
			const XMLDATA& reference	//!< Reference for validation
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		enum TYPE {
			TYPE_Int = 0,
			TYPE_Double,
			TYPE_String,
			TYPE_Bool,
			TYPE_Color,
			TYPE_NUMTYPES
			};

		XMLNODE *m_root;	// The root node of the record

		static const char* GetPropertyName (PROPERTY property);
		static TYPE TypeFromName (const char *name);
		static const char* TypeToName (TYPE type);

		XMLNODE* FindNode (const char *id, bool create = false) const;

		void InternalSetValue (
			const char *id,					//!< ID of the item to get value of
			const MISTRING& value,			//!< Value to set
			TYPE type,
			PROPERTY property = PROPERTY_Value
			);

		virtual void OnDocReplaced ();
		
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

#endif
