/**
 * \file rvc/cad.h
 * \brief interface for the RVCCAD class.
 *
 * \if NODOC
 * $Id: cad.h_v 1.95 2015/06/08 15:13:44 mju Exp $
 *
 * $Log: cad.h_v $
 * Revision 1.95  2015/06/08 15:13:44  mju
 * Remove obsolete deprecated typedef.
 *
 * Revision 1.94  2015/04/30 14:06:52  mju
 * Use cadelement and deprecate legacy typedef.
 *
 * Revision 1.93  2015/04/29 22:10:33  mju
 * Change virtuals for reading line/point, use elemidx.
 *
 * Revision 1.92  2015/04/27 22:06:58  mju
 * Add v_componentToElemType.
 *
 * Revision 1.91  2014/05/05 19:12:38  bcolombe
 * Use LinkString instead of filepath in LINKPARMS
 *
 * Revision 1.90  2014/04/07 17:58:49  mju
 * Use rvc::element for getExtents and readElement.
 *
 * Revision 1.89  2013/10/24 21:20:59  mju
 * Change v_readRegion to use unsignedset.
 *
 * Revision 1.88  2013/03/22 20:33:21  bcolombe
 * Add GDAL linktype
 *
 * Revision 1.87  2012/10/09 15:00:19  mju
 * Add "separate" to setupElemTransList.
 *
 * Revision 1.86  2011/12/06 22:26:52  mju
 * Remove redundant hdrs.
 *
 * Revision 1.85  2008/09/09 23:00:51  scowan
 * Added conversion to shape object.
 *
 * Revision 1.84  2007/05/09 17:01:28  mju
 * Don't inline elemiterator methods that read element, as won't be fast anyway.
 * That allows to not include errcodes and errhandler hdrs.
 *
 * Revision 1.83  2007/04/19 18:01:26  mju
 * Use errcodes hdr.
 *
 * Revision 1.82  2007/04/13 15:38:15  scowan
 * Added virtual readRegion method.
 *
 * Revision 1.80  2006/12/29 21:15:54  scowan
 * Added readElement virtual methods.
 *
 * Revision 1.77  2006/09/15 19:56:54  scowan
 * MOved cad element out of include file.
 *
 * Revision 1.76  2006/06/27 15:04:06  scowan
 * Make method now fills in objitem with created object info.
 *
 * Revision 1.74  2005/12/21 23:08:45  scowan
 * Added support for the readNext virtual methods.
 *
 * Revision 1.73  2005/12/19 18:43:41  dwilliss
 * adjusted mandatory credentials to account for prior optional parameters
 *
 * Revision 1.72  2005/12/19 17:02:23  linux32build!dwilliss
 * Credentials can now be made mandatory by setting a #define
 *
 * Revision 1.71  2005/12/09 17:29:13  scowan
 * Updated to use credentials.
 *
 * Revision 1.69  2005/10/20 17:00:13  scowan
 * Quadtree and geometric interface changes supported.
 *
 * Revision 1.66  2005/09/07 19:54:49  scowan
 * Added methods to get and set the block descriptor.
 *
 * Revision 1.65  2005/07/08 20:02:12  scowan
 * Support objMakeParms for cad.
 *
 * Revision 1.64  2005/06/27 17:01:54  scowan
 * Added dimension parm to draw methods.
 *
 * Revision 1.63  2005/06/02 21:11:11  scowan
 * Added methods to parse GML element to CAD element.
 *
 * Revision 1.62  2005/06/01 23:02:41  scowan
 * Added get gml element method.
 *
 * Revision 1.61  2005/04/11 17:05:06  dfriberg
 * Added LINKTYPE_DWG
 *
 * Revision 1.60  2005/01/19 21:12:17  scowan
 * Added support for descriptor.
 *
 * Revision 1.59  2004/12/18 00:07:58  scowan
 *  Added component field to findClosestElements.
 *
 * Revision 1.58  2004/12/01 18:39:31  dfriberg
 * LINKTYPE_DGN = 5 (as defined in RVC::CADHANDLE::LINKTYPE)
 *
 * Revision 1.55  2004/11/09 00:14:32  dfriberg
 * Add DGN to LINKTYPE.
 *
 * Revision 1.51  2004/07/28 16:12:41  scowan
 * Moved polyline generation from a box to the box class.
 *
 * Revision 1.50  2004/07/15 17:00:09  scowan
 * Added new private element method.
 *
 * Revision 1.47  2004/06/14 20:09:40  scowan
 * Added method to get the text frame parms.
 *
 * Revision 1.46  2004/06/14 16:20:55  scowan
 * Added methods to set and get baseline flags.
 *
 * Revision 1.45  2004/06/11 16:17:32  scowan
 * Addow increment of cad element type enums.
 *
 * Revision 1.44  2004/06/10 16:53:54  scowan
 * Added delete last element method.
 *
 * Revision 1.43  2004/06/07 15:32:06  scowan
 * Added method to update element.
 *
 * Revision 1.42  2004/06/04 19:25:27  scowan
 * Added methods to help in cad merge.
 *
 * Revision 1.41  2004/05/28 20:35:28  scowan
 * Added new mehtods and virtual method overrides.
 *
 * Revision 1.40  2004/05/28 15:56:57  scowan
 * Changed to have element iterator use new element class.
 *
 * Revision 1.39  2004/05/27 19:59:00  scowan
 * Added make parm ctor.
 *
 * Revision 1.38  2004/05/27 16:11:39  scowan
 * Added methods to access certain fields in cad element.
 *
 * Revision 1.37  2004/05/20 22:05:30  scowan
 * Added database index methods.
 *
 * Revision 1.36  2004/05/20 19:40:03  scowan
 * Added cad element class and other sub-classes.
 *
 * Revision 1.35  2003/11/21 22:17:02  scowan
 * Support new Capabilities system.
 *
 * Revision 1.34  2003/10/13 16:46:37  scowan
 * Added cad type to text id method.
 *
 * Revision 1.27  2003/06/04 23:35:06  scowan
 * Added find element draw set.
 *
 * Revision 1.24  2002/09/30 23:22:18  scowan
 * Made element search code const.
 *
 * Revision 1.21  2002/08/29 22:13:22  scowan
 * Virtual method adjustment.
 *
 * Revision 1.18  2002/06/18 22:15:22  scowan
 * Added extract interface.
 *
 * Revision 1.17  2002/04/30 17:33:50  scowan
 * Change base make parm class name.
 *
 * Revision 1.12  2002/03/01 15:34:35  scowan
 * Added set element ID method.
 *
 * Revision 1.11  2002/02/27 23:24:46  scowan
 * Added get element extents methods.
 *
 * Revision 1.10  2001/10/18 15:29:32  scowan
 * Added update method.
 *
 * Revision 1.8  2001/10/08 20:50:00  scowan
 * Added element region method.
 *
 * Revision 1.7  2001/09/28 15:29:11  scowan
 * Added extents to the make parms.
 *
 * Revision 1.5  2001/07/12 20:14:55  scowan
 * Added encoding for linked objects.
 *
 * Revision 1.1  2001/05/29 21:12:27  scowan
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_CAD_H
#define INC_RVC_CAD_H

#ifndef INC_RVC_GEOMETRC_H
	#include <rvc/geometrc.h>
#endif

#ifndef INC_RVC_CADELEMENT_H
	#include <rvc/cadelement.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward Declarations
class BITSET_UNOWNED;
struct CADBLOCK;
class POLYLINE;
class POLYLINELIST;
namespace RVC {
	class CADHANDLE;
	class DESCRIPTOR;
	class QSEARCHTREE;
	}
#endif


namespace RVC {

//! Manages RVC::OBJTYPE_CAD RVC objects.
//!
//!	RVC Object Types that this class supports: RVC::OBJTYPE_CAD.
//!
//!	Default RVC Object Type for this class: RVC::OBJTYPE_CAD.
//!
//!	Valid parents for this class:
//!		- RVC::OBJTYPE_File
//!		- RVC::OBJTYPE_Folder
class CAD : public GEOMETRIC {
	public:

		typedef INT32 CADBLOCKNUM;				//!< Typedef for CAD block number type

		enum LINKTYPE {
			LINKTYPE_Internal = 0,
			LINKTYPE_MapInfo = 1,		//!<  Mapinfo .MIF format 
			LINKTYPE_Shapefile = 3,		//!<  Arc/Info Shapefile format 
			LINKTYPE_DGN = 5,				//!<  MicroStation DGN format
			LINKTYPE_DWG = 6,				//!<  AutoCAD DWG format
			LINKTYPE_GDAL = 7,			//!< GDAL OGR format
			LINKTYPE_Edit = 1000
			};

		#ifndef GENERATING_DOXYGEN_OUTPUT	
		class CONST_ELEMITERATOR;
		friend class CONST_ELEMITERATOR;
		class DRAW;
		class LINKPARMS;
		class MAKEPARMS;
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Default constructor
		CAD (
			);

		//! Copy constructor
		CAD (
			const RVC::CAD &obj
			);

		//! Destructor
		virtual ~CAD (
			);

		//! Assignment, will close and re-open object if possible
		RVC::CAD& operator= (
			const RVC::CAD &rhs
			);

		//! Add an element to the CAD object	
		INT32 AddElement (
			const CADELEMENT &Element
			);

		//! Copy a block inside a CAD object
		ERRVALUE CopyBlock (
			CADBLOCKNUM DestBlockNum,
			CADBLOCKNUM SrcBlockNum
			);

		//! Delete the element specified.  Only works if the GEOMETRIC::CAPABILITY_CanDeleteAnyElement is available
		ERRVALUE DeleteElement (
			INT32 ElemNum
			);
			
		ERRVALUE DeleteLastElement (
			);
			
		//! Draws the block by calling the virtual draw routines.
		//! @return The number of elements drawn. 
		INT32 DrawBlock (
			DRAW &Draw,
			INT32 BlockNum = 0,
			double MinChordDist = 0.0,					//!< Default is to use %5 of the Radius length
			DIMENSION dim = DIMENSION_2D
			) const;

		//! Draws an element.
		//! Returns > 0 if drawn, 0 if not, > 0 error. 
		int DrawElement (							
			DRAW &Draw,
			INT32 BlockNum,
			INT32 ElemNum,									//!< Element to draw
			double MinChordDist = 0.0,					//!< Default is to use %5 of the Radius length
			DIMENSION dim = DIMENSION_2D
			) const;

		//! Obtain a constant iterator to the first CAD element in the current block
		//! @return Constant iterator to first CAD element in the current block
		CONST_ELEMITERATOR ElementBegin (
			) const;

		//! Obtain an iterator to the last+1 CAD element in the current block
		//! @return Constant iterator to last+1 CAD element in the current block
		CONST_ELEMITERATOR ElementEnd (
			) const;

		//! Erase all elements from a block
		ERRVALUE EraseBlock (
			CADBLOCKNUM BlockNum
			);

		//! Get the DESCRIPTOR for a block
		ERRVALUE GetBlockDescriptor (
			INT32 BlockNum,
			DESCRIPTOR &Descriptor
			);
			
		//! Get the current block number
		//! @return Current block number
		CADBLOCKNUM GetCurrentBlock (
			) const;

		//! Obtain the link type
		//! @return RVCCAD::LINKTYPE
		LINKTYPE GetLinkType (
			) const;

		//! Get MAKEPARM values from current CAD object instance			
		ERRVALUE GetMakeParms (
			RVC::CAD::MAKEPARMS &MakeParms
			) const;
			
		//! Obtain next available element ID for element ID table
		//! @return Next available Element ID
		INT32 GetNextElemID (
			) const;

		//! Obtain the number of blocks in this CAD object
		//! @return Number of blocks
		INT32 GetNumBlocks (
			) const;

		//! Link CAD object to external file
		ERRVALUE Link (								//! Create a raster object
			const RVC::OBJECT &parent,
			const RVC::DESCRIPTOR &Descriptor,
			const LINKPARMS &LinkParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new CAD object
		ERRVALUE Make (								//! Create a raster object
			const RVC::OBJECT &parent,
			const RVC::DESCRIPTOR &Descriptor,
			const MAKEPARMS &MakeParms,
			const CREDENTIALS* Credentials RVC_DEFAULT_CREDENTIALS				//!< User credentials passed in, NULL for default behavior
			);

		//! Create a new CAD object
		ERRVALUE Make (
			RVC::OBJITEM &ObjItem,								//!< Parent object and DESCRIPTOR of object to create, new location RETURNED
			const MAKEPARMS &MakeParms,
#ifdef RVC_CREDENTIALS_MANDATORY
			const char* Source,
			const CREDENTIALS* Credentials
#else
			const char* Source = 0,
			const CREDENTIALS* Credentials = 0				//!< User credentials passed in, NULL for default behavior
#endif
			);

		//! Read the block information
		ERRVALUE ReadBlock (
			CADBLOCKNUM BlockNum,
			CADBLOCK &block,
			bool MakeCurrent = true					//!< Make current block
			) const;

		//! Read next CAD element in the sequence in the current block
		ERRVALUE ReadElement (
			CADELEMENT &Element,
			DIMENSION dim = DIMENSION_2D
			) const;

		//! Read specific CAD element in the current block
		ERRVALUE ReadElement (
			INT32 ElemNum,
			CADELEMENT &Element,
			DIMENSION dim = DIMENSION_2D
			) const;

		//! Rewind CAD object block reading to the begining
		ERRVALUE Rewind (
			);

		//! Set the DESCRIPTOR for a block			
		ERRVALUE SetBlockDescriptor (
			INT32 BlockNum,
			const DESCRIPTOR &Descriptor
			);
			
		ERRVALUE SetEditBlock (
			INT32 BlockNum
			);

		//! Set next available element ID for element ID table
		ERRVALUE SetNextElemID (
			INT32 NextElemID
			);

		//! Set CAD object point coordinate type
		ERRVALUE SetPointType (
			POINTTYPE PointType
			);

		//! Change the drawing order of an element in the CAD object			
		ERRVALUE SortElement (
			INT32 ElemNum,
			bool SortToFront
			);
			
		//! Change the drawing order of the set of elements in the CAD object			
		ERRVALUE SortElements (
			const BITSET_UNOWNED &SortSet,
			bool SortToFront
			);
			
		//! Write the block information
		ERRVALUE WriteBlock (
			CADBLOCKNUM BlockNum,
			CADBLOCK &block,
			bool MakeCurrent = true					//!< Make current block
			);

		//! Write element information, only valid if linktype == LINKTYPE_Edit
		ERRVALUE WriteElement (
			INT32 ElemNum,
			const CADELEMENT &Element
			);
			
		//! Update element information (in CADELEMOPT section)
		ERRVALUE UpdateElement (
			INT32 ElemNum,
			const CADELEMENT &Element
			);
			
		ERRVALUE UpdateExtentsFromInsertElements (
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		INT32 DrawBlock (DRAW &Draw, INT32 BlockNum, const TRANS2D_AFFINE &BlockToCAD, INT32 InsertLevel, double MinChordDist, DIMENSION dim) const;
		int DrawElement (DRAW &Draw, INT32 BlockNum, INT32 ElemNum, const TRANS2D_AFFINE &BlockToCAD, INT32 InsertLevel, double MinChordDist, DIMENSION dim) const;
		ERRVALUE DrawInsert (DRAW &Draw, INT32 BlockNum, const CADELEMENT &Element, const TRANS2D_AFFINE &BlockToCAD, INT32 InsertLevel, double MinChordDist, DIMENSION dim) const;
		
		ERRVALUE ExtractToCAD (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags, const SIMPLE_ARRAY<INT32> &FontTrans, const SIMPLE_ARRAY<INT32> PattTrans[RVC::PATTERNTYPE_NumTypes]) const;
		ERRVALUE ExtractToRegion (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToShape (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		ERRVALUE ExtractToVector (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		
		INT64 GetCurrentPosition () const;
		void SetCurrentPosition (INT64 Position) const;
		
		int SetupBlockTransList(ELEMTRANSLIST &TransList) const;
		int SetupDBRecordTransList(ELEMTRANSLIST &TransList) const;
		int SetupElementTransList(const ELEMSELECTPARMS &SelectParms, ELEMTRANSLIST &TransList, RVC::GEOMETRIC::ELEMSELECTFILTER* SelectFilter) const;

		//! Implementation of RVC::OBJECT virtual methods
		virtual ERRVALUE v_CloseObject ();					//! Close the open object
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS* Credentials);

		//! Implementation of RVC::GEOSPATIAL virtual methods
		virtual void v_GetExtents (DRECT3D &extents) const;
		virtual void v_GetScale (double &xscale, double &yscale) const;
		virtual double v_GetZScale () const;
		virtual double v_GetZOffset () const;
		virtual bool v_HasCapability (int Capability) const;
		virtual void v_SetScale (double xscale, double yscale);
		virtual void v_SetZOffset (double zoffset);
		virtual void v_SetZScale (double zscale);
		
		//! Implementation of RVC::GEOMETRIC pure virtual methods
		virtual ELEMTYPE v_ComponentToElemType (GEOMETRIC::COMPONENT component) const;
		virtual ERRVALUE v_ExtractTo (RVC::GEOMETRIC &DestObj, RVC::GEOMETRIC::ELEMTRANSLIST &TransList, const TRANS2D_MAPGEN &tmg, RVC::GEOMETRIC::EXTRACTFLAGS ExtractFlags) const;
		virtual INT32 v_FindClosestElementLow (ELEMTYPE ElemType, const DPOINT2D &SearchPt, SEARCHFILTER* SearchFilter, double SearchDist, double* DistFromElem) const;
		virtual ERRVALUE v_GetElementExtents (const RVC::ELEMIDX &elemidx, DRECT3D &Extents) const;
		virtual INT32 v_GetNextElementID (ELEMTYPE ElemType) const;
		virtual INT32 v_GetNumElements (ELEMTYPE ElemType) const;
		virtual POINTTYPE v_GetPointType () const;
		virtual INT32 v_GetPointSize () const;
		virtual QSEARCHTREE* v_GetSearchTree (ELEMTYPE ElemType) const;
		virtual double v_GetTotalLineLength () const;
		virtual bool v_IsElementDeleted (const RVC::ELEMIDX &elemidx) const;
		virtual bool v_IsEmpty () const;
		virtual ERRVALUE v_SetExtents (const DRECT3D &extents);
		virtual ERRVALUE v_SetNextElementID (ELEMTYPE ElemType, INT32 NextElementID);
		virtual void v_SetSearchTree (ELEMTYPE ElemType, QSEARCHTREE* SearchTree) const;
		virtual ERRVALUE v_SetTotalLineLength (double length);
		virtual int v_SetupElemTransList (const GEOMETRIC::ELEMSELECTPARMS &SelectParms, GEOMETRIC::ELEMTRANSLIST &TransList, GEOMETRIC::ELEMSELECTFILTER *SelectFilter, bool separate) const;
		virtual bool v_TestElementAgainstRegion (const RVC::ELEMIDX &elemidx, const REGION2D &region, TESTCRITERIA TestMode) const;
		virtual ERRVALUE v_UpdateExtents (const DRECT3D &extents);
			
		//! Implementation of RVC::GEOMETRIC virtual methods
		virtual ERRVALUE v_FindClosestElementsLow (const DPOINT2D &SearchPt, double SearchDist, SEARCHFILTER* SearchFilter, SIMPLE_ARRAY<RVC::ELEMIDX> &elementList, COMPONENT Component) const;
		virtual ELEMNUM v_GetAttachmentIndex (const RVC::ELEMIDX &elemidx) const;
		virtual ERRVALUE v_ReadElement (const RVC::ELEMIDX &elemidx, REGION2D &Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadLabel (const RVC::ELEMIDX &elemidx, LABELLIST &labels, DIMENSION dim, STYLE *pObjStyle) const;
		virtual ERRVALUE v_ReadLine (const RVC::ELEMIDX &elemidx, POLYLINELIST &LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextLabel (RVC::ELEMIDX &elemidx, MILIST<LABEL> &Label, DIMENSION dim, STYLE* StyleObj) const;
		virtual ERRVALUE v_ReadNextLine (RVC::ELEMIDX &elemidx, POLYLINELIST &LineList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextPoint (RVC::ELEMIDX &elemidx, POLYLINE &PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadNextRegion (RVC::ELEMIDX &elemidx, REGION2D &Region, DIMENSION dim) const;
		virtual ERRVALUE v_ReadPoint (const RVC::ELEMIDX &elemidx, POLYLINE &PointList, DIMENSION dim) const;
		virtual ERRVALUE v_ReadRegion (ELEMTYPE ElemType, const UNSIGNEDSET &ElemSet, REGION2D &region, DIMENSION dim, bool bExcludeIslands) const;
		
		CADHANDLE* m_CadHandle;
		LINKTYPE m_LinkType;			//!< 2	Type of link (DXF, MIF, ...) 
		mutable QSEARCHTREE *m_CADQTree;
		class OPTIMIZE;
		mutable OPTIMIZE* m_Optimize;
		#endif // GENERATING_DOXYGEN_OUTPUT

	};


//! CAD Interface class to filter CAD elements before drawing
//! Inherits from CADELEMENT::DRAW
class CAD::DRAW : public CADELEMENT::DRAW {
	public:
		DRAW () {}
		virtual ~DRAW () {}
	
		//! Filter CAD elements before they are rendered.  Specific place to modify certain attributes of the element and update element status
		//! @return >0 if the element is to be rendered, ==0 if not, <0 error
		int FilterElement (
			CADELEMENT &Element,					//!< Original CADELEMENT, can change specific items in the element in the filter call
			const TRANS2D_AFFINE &BtoC,			//!< Block to CAD transformation
			INT32 InsertLevel,						//!< How deep the block insertion stack is, 0 for main block
			INT32 ElemNum,								//!< Element number in the current drawing block
			const DRECT3D &Extents					//!< Element extents, maybe invalid
			) { return (v_FilterElement(Element, BtoC, InsertLevel, ElemNum, Extents)); }
			
	private:
	#ifdef WIN32
		#pragma warning (push)
		#pragma warning (disable:4100)	// Unreferenced formal parameter
	#endif
		virtual int v_FilterElement (CADELEMENT &Element, const TRANS2D_AFFINE &BtoC, INT32 InsertLevel, INT32 ElemNum, const DRECT3D &Extents) { return (1); }
	#ifdef WIN32
		#pragma warning (pop)
	#endif
	};
	

//! Constant element iterator class
//!
//! This class is used to iterate through a CAD object's elements.  See methods ElementBegin() and ElementEnd() for obtaining a 
//! CAD object element iterator
class CAD::CONST_ELEMITERATOR {
	public:
		//! Default constructor
		CONST_ELEMITERATOR (
			) : 
			m_CADObj(0),
			m_Element(0),
			m_ReachedEnd(false)
			{}

		//! Copy constructor
		CONST_ELEMITERATOR (
			const CONST_ELEMITERATOR &rhs
			) : 
			m_Element(0),
			m_CADObj(rhs.m_CADObj),
			m_ReachedEnd(rhs.m_ReachedEnd)
			{
			if (rhs.m_Element != 0) m_Element = new CADELEMENT(*rhs.m_Element);
			}

		#ifndef GENERATING_DOXYGEN_OUTPUT
		//! Internal constructor used by ElementEnd() methods
		CONST_ELEMITERATOR (
			const RVC::CAD* CADObj,
			INT32 // LastElemNum (not used, just here to differentiate from ctor for ElementBegin.
			) : 
			m_Element(0),
			m_CADObj(CADObj),
			m_ReachedEnd(true)
			{}

		//! Internal constructor used by ElementBegin() methods
		CONST_ELEMITERATOR (
			const RVC::CAD* CADObj
			);
				
		#endif // GENERATING_DOXYGEN_OUTPUT

		//! Destructor
		virtual ~CONST_ELEMITERATOR (
			) {
			delete m_Element;
			}

		//! Assignment
		CONST_ELEMITERATOR& operator= (
			const CONST_ELEMITERATOR &rhs
			) {
			if (this != &rhs) {
				delete m_Element;
				m_Element = 0;
				if (rhs.m_Element != 0) m_Element = new CADELEMENT(*rhs.m_Element);
				m_CADObj = rhs.m_CADObj;
				m_ReachedEnd = rhs.m_ReachedEnd;
				}
			return (*this);
			}

		//! Dereference
		const CADELEMENT& operator* (
			) const { return (*m_Element); }

		//! Dereference arrow
		const CADELEMENT* operator-> (
			) const { return (&**this); }

		//! Pre-increment
		CONST_ELEMITERATOR& operator++ (
			);

		//! Comparison for equality
		bool operator== (
			const CONST_ELEMITERATOR &rhs
			) const { return (m_CADObj == rhs.m_CADObj && m_ReachedEnd == rhs.m_ReachedEnd); }

		//! Comparison for inequality
		bool operator!= (
			const CONST_ELEMITERATOR &rhs
			) const { return (!(*this == rhs)); }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		CADELEMENT* m_Element;
		const CAD* m_CADObj;
		bool m_ReachedEnd;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class CAD::LINKPARMS {
	public:

		//! Default constructor, initialize to 2D Polygonal topology by default
		LINKPARMS (
			const MISTRING &LinkString,
			LINKTYPE LinkType,
			CHAR_ENCODING encoding = CHAR_ENCODING_ASCII
			) :
			m_LinkString(LinkString),
			m_LinkType(LinkType),
			m_encoding(encoding)
			{}

		//! Obtain CAD object link character encoding
		//! @return CHAR_ENCODING of linked object
		CHAR_ENCODING GetEncoding (
			) const {return (m_encoding);}

		//! Obtain CAD object link string
		//! @return MISTRING of linked object
		const MISTRING& GetLinkString (
			) const {return (m_LinkString);}

		//! Obtain CAD object link type
		//! @return RVCCAD::LINKTYPE
		LINKTYPE GetLinkType (
			) const {return (m_LinkType);}
			
		//! Set CAD object link character encoding
		void SetEncoding (
			CHAR_ENCODING encoding
			) {m_encoding = encoding;}

		//! Set CAD object link file string
		void SetLinkString (
			const MISTRING &LinkString
			) {m_LinkString = LinkString;}

		//! Set CAD object link type
		void SetLinkType (
			LINKTYPE LinkType
			) {m_LinkType = LinkType;}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_LinkString;
		LINKTYPE m_LinkType;
		CHAR_ENCODING m_encoding;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};


class CAD::MAKEPARMS : public RVC::GEOMETRIC::MAKEPARMSBASE {
	public:

		//! Default constructor
		MAKEPARMS (
			POINTTYPE PointType = POINTTYPE_2DXY
			) :
		  	GEOMETRIC::MAKEPARMSBASE(PointType),
			m_NextElemID(1),
			m_EditBlockNum(-1)
			{}

		//! Template constructor
		MAKEPARMS (
			const RVC::CAD &Template
			);

		//! Combination constructor
		MAKEPARMS (
			const RVC::OBJITEMLIST &GeoObjItemList		//!< Currently 'vector' and 'shape' are the only valid objects
			);

		//! Get which block to edit, causes the CAD object to create an editable block in the project file
		//! Only valid to get when creating CAD objects in temporary files.			
		INT32 GetEditBlockNum (
			) const { return (m_EditBlockNum); }
			
		//! Obtain next CAD element ID value
		//! @return Next CAD element ID value
		INT32 GetNextElemID (
			) const {return (m_NextElemID);}

		//! Set which block to edit, causes the CAD object to create an editable block in the project file
		//! Only valid to set when creating CAD objects in temporary files.			
		void SetEditBlockNum (
			INT32 BlockNum
			) { m_EditBlockNum = BlockNum; }
			
		//! Set CAD element next element ID value			
		void SetNextElemID (
			INT32 NextElemID
			) {m_NextElemID = NextElemID;}

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		INT32 m_NextElemID;
		INT32 m_EditBlockNum;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

}		// End of RVC namespace

#endif 	// INC_RVC_CAD_H
