/**
 * \file rvc/descriptor.h
 * \brief RVC::DESCRIPTOR class definitions
 *
 * $Id: descriptor.h_v 1.29 2014/01/03 21:32:27 mju Exp $
 *
 * $Log: descriptor.h_v $
 * Revision 1.29  2014/01/03 21:32:27  mju
 * Remove legacy ctor from objectname.
 *
 * Revision 1.27  2011/05/27 20:50:22  mju
 * Add constructor with name/desc.
 *
 * Revision 1.25  2007/09/25 21:24:22  scowan
 * Added isLessThan and inline operator< methods.
 *
 * Revision 1.24  2006/04/18 15:18:37  mju
 * Add setName variant using delegate to ensure generated short name unique.
 *
 * Revision 1.1  2005/01/12 22:47:31  scowan
 * Initial revision
**/

#ifndef	INC_RVC_DESCRIPTOR_H
#define	INC_RVC_DESCRIPTOR_H

#ifndef INC_MI32_FIXEDSTR_H
	#include <mi32/fixedstr.h>
#endif

#ifdef RVCSYSDLL
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSEXPORT
#else
	#define RVCSYSLIBCLASSEXPORT MI_DLLCLASSIMPORT
#endif

namespace RVC {

//! "Short" object name, must be unique by object type for specific parent.
typedef FIXEDSTRING<16>	OBJECTNAME;

//! Container for specifying name, description, and source for an object
class RVCSYSLIBCLASSEXPORT DESCRIPTOR {
	public:

		//! Default constructor
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR ();

		//! Constructor taking a name as a MISTRING.
		//! Will fill in the name field and short name fi with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR (
			const MISTRING& name
			);

		//! Constructor taking a name as a TEXTID.
		//! Will fill in the name field and short name field with validating the name (does not guarentee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR (
			TEXTID TextID
			);

		//! Construct with both name and description.
		DESCRIPTOR (
			const MISTRING& name,
			const MISTRING& description
			);

		//! Copy constructor
		DESCRIPTOR (
			const DESCRIPTOR& rhs
			) :
			m_Name(rhs.m_Name),
			m_ShortName(rhs.m_ShortName),
			m_Description(rhs.m_Description),
			m_Source(rhs.m_Source)
			{ }

		//! Destructor
		~DESCRIPTOR ();

		//! Assignment operator
		DESCRIPTOR& operator= (
			const DESCRIPTOR& rhs
			) {
			if (this != &rhs) {
				m_Name = rhs.m_Name;
				m_ShortName = rhs.m_ShortName;
				m_Description = rhs.m_Description;
				m_Source = rhs.m_Source;
				}
			return (*this);
			}
			
		//! Assignment from MISTRING, will clear out previous settings.
		//! Will fill in the name field and short name field with validating the name (does not guarantee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR& operator= (
			const MISTRING& name
			);
			
		//! Assignment from TEXTID, will clear out previous settings.
		//! Will fill in the name field and short name field with validating the name (does not guarantee uniqueness).
		//! Will set the description field if the resulting short name is different than 'name'
		//! Presets the 'source' field from MsysGetAppName().
		DESCRIPTOR& operator= (
			TEXTID textid
			);
			
		//! Clear the instance of all strings
		void Clear (
			) {
			m_Name.Clear();
			m_ShortName.Clear();
			m_Description.Clear();
			m_Source.Clear();
			}

		//! Get the description of the object
		//! @return Constant reference to the description
		const MISTRING& GetDescription () const
			{ return (m_Description); }
			
		//! Get the name of the object
		//! @return Constant reference to the name
		const MISTRING& GetFullName () const
			{ return (m_Name); }

		//! Get the short name of the object
		//! @return Constant reference to the short name
		const OBJECTNAME& GetShortName () const
			{ return (m_ShortName); }

		//! Get the source of the object
		//! @return Constant reference to the source
		const OBJECTNAME& GetSource () const;

		//! Is the descriptor empty?
		bool IsEmpty () const
			{ return (m_Name.IsEmpty() && m_Description.IsEmpty()); }
			
		//! Are the descriptor's equal?
		bool IsEqual (
			const DESCRIPTOR& rhs
			) const;

		//! Are the descriptor's equal?
		bool IsLessThan (
			const DESCRIPTOR& rhs
			) const { return (m_Name < rhs.m_Name || (m_Name == rhs.m_Name && m_Description < rhs.m_Description)); }

		//! Set the object description
		void SetDescription (
			const MISTRING& Description			//!< The MISTRING containing the long description
			) { m_Description = Description; }
			
		//! Set the object name.
		//! This will set the short name value as well.
		void SetName (
			const MISTRING& Name						//!< The MISTRING containing the long name
			);

		//! Set the object name with check for uniquely generated short name.
		//! This will set the short name value as well.
		//! Delegate will only be called if the generated short name has changed and is not identical to the full name.
		void SetName (
			const MISTRING& Name,					//!< The MISTRING containing the long name
			FastDelegate<int(const RVC::OBJECTNAME&)> DelegateTestUniqueShortName
			);

		//! Set the short name (object ID tag)
		void SetShortName (
			const OBJECTNAME& name
			);
			
		//! Set the object source
		void SetSource (
			const OBJECTNAME& Source				//!< The OBJECTNAME containing the source string
			) { m_Source = Source; }

		//! Set the object source
		//! Pass NULL for the 'Source' parameter to set to MsysGetAppName().
		void SetSource (
			const char* Source						//!< The ASCII string containing the source string
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		MISTRING m_Name;
		MISTRING m_Description;
		OBJECTNAME m_ShortName;
		mutable OBJECTNAME m_Source;
		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
}		// End of RVC namespace

#undef RVCSYSLIBCLASSEXPORT

inline bool operator== (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (lhs.IsEqual(rhs)); }

inline bool operator!= (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (!lhs.IsEqual(rhs)); }

//! Used for STL containers that sort their items
inline bool operator< (
	const RVC::DESCRIPTOR& lhs,
	const RVC::DESCRIPTOR& rhs
	) { return (lhs.IsLessThan(rhs)); }

#endif		// INC_RVC_DESCRIPTOR_H
