/**
 * \file rvc/elemidx.h
 * \brief RVC::ELEMIDX class definition.
 *
 * \if NODOC
 * $Id: elemidx.h_v 1.19 2015/04/30 13:36:25 mju Exp $
 *
 * $Log: elemidx.h_v $
 * Revision 1.19  2015/04/30 13:36:25  mju
 * Rename hdr.
 *
 * Revision 1.18  2015/04/29 22:09:12  mju
 * Rename class as elemidx so less conflict with other element classes.
 * Redesign so fits in 8 bytes.
 *
 * Revision 1.17  2014/03/28 21:26:53  mju
 * Add increment operator.
 *
 * Revision 1.16  2013/01/10 16:22:02  mju
 * Add set method.
 *
 * Revision 1.14  2010/08/19 14:47:58  mju
 * Add ctor using type only.
 * Use elementnum_None.
 *
 * Revision 1.9  2004/07/22 15:27:26  dwilliss
 * Added an operator< so I could use RVC::ELEMIDX as a key in std::map
 *
 * Revision 1.1  2002/04/11 17:46:11  scowan
 * Initial revision
 * \endif
**/
 
#ifndef INC_RVC_ELEMIDX_H
#define INC_RVC_ELEMIDX_H

#ifndef INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif


namespace RVC {

//=====================================================================================================================

//! Container for element index specifying number and ELEMTYPE.
//! This class is designed to fit in 8 bytes and allows element numbers between -15 and (2^56 - 16).
class ELEMIDX {
	public:

		//! Default constructor.	
		ELEMIDX ()
			{ }

		//! Construct with type only.
		explicit ELEMIDX (
			ELEMTYPE ElemType
			) { SetType(ElemType); }

		//! Construct from element number and optional type.
		explicit ELEMIDX (
			ELEMNUM ElemNum,
			ELEMTYPE ElemType = ELEMTYPE_Empty
			) { Set(ElemType,ElemNum); }

		//! Copy construct from ELEMIDX and fallback element type.
		ELEMIDX (
			const ELEMIDX &rhs,						//!< Element to copy
			ELEMTYPE ElemType							//!< Fallback element type, used if not set in 'rhs'
			) : m_value(rhs.m_value) { if (GetType() == ELEMTYPE_Empty) SetType(ElemType); }

		//! Pre-increment operator.
		ELEMIDX& operator++ ()
			{ if ((m_value.m_Num & Num_Mask) == Num_None) m_value.m_Num &= ~Num_Mask; else ++m_value.m_Num; return (*this); }

		//! Comparison for ordering.
		//! Elements are sorted by type then by element number.
		//! This comparison allows RVC::ELEMIDX to be used as a Key in std::map.
		bool operator< (
			const ELEMIDX &rhs
			) const { return (m_value.m_Num < rhs.m_value.m_Num); }

		//! Clear to element info back to no element.
		void Clear ()
			{ m_value.m_Num = Num_None; }

		//! Get element number.
		ELEMNUM GetNumber () const
			{ ELEMNUM num = (m_value.m_Num & Num_Mask); if (num > Num_Max) num |= ~Num_Mask; return (num); }
			
		//! Get element type.
		ELEMTYPE GetType () const
			{ return (static_cast<ELEMTYPE>(m_value.m_Byte[TypeIdx])); }

		//! Test for equality.
		bool IsEqual (
			const ELEMIDX &rhs
			) const { return (m_value.m_Num == rhs.m_value.m_Num); }

		//! Set type and number
		void Set (
			ELEMTYPE ElemType,
			ELEMNUM ElemNum
			) { if (ElemNum < -15 || ElemNum > Num_Max) ElemNum = ELEMENTNUM_None; m_value.m_Num = ElemNum; SetType(ElemType); }

		//! Set element number.
		void SetNumber (
			ELEMNUM ElemNum
			) { ELEMTYPE type = GetType(); m_value.m_Num = ElemNum; SetType(type); }

		//! Set element type (ELEMTYPE).
		void SetType (
			ELEMTYPE ElemType
			) { m_value.m_Byte[TypeIdx] = static_cast<UINT8>(ElemType); }
			
	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		static const UINT64 Num_Mask = 0x00FFFFFFFFFFFFFF;
		static const UINT64 Num_None = 0x00FFFFFFFFFFFFFF;
		static const ELEMNUM Num_Max = 0x00FFFFFFFFFFFFF0;
		#ifdef BYTEORDER_LoHi
			static const int TypeIdx = 7;
		#else
			static const int TypeIdx = 0;
		#endif
		union VALUE {
			UINT64 m_Num;
			UINT8 m_Byte[8];
			VALUE () : m_Num(Num_None) { }
			};
		VALUE m_value;
		#endif // GENERATING_DOXYGEN_OUTPUT
		
	};

//! Compare ELEMIDXs for equality.
inline bool operator== (
	const ELEMIDX &lhs,
	const ELEMIDX &rhs
	) { return (lhs.IsEqual(rhs)); }
	
//! Compare ELEMIDXs for inequality.
inline bool operator!= (
	const ELEMIDX &lhs,
	const ELEMIDX &rhs
	) { return (!lhs.IsEqual(rhs)); }
	
//=====================================================================================================================

}		// End of RVC namespace

#endif 	// INC_RVC_ELEMIDX_H

