/**
 * \file rvc/featurelocator.h
 * \brief interface for the RVC::FEATURELOCATOR class.
 *
 * \if NODOC
 * $Id: featurelocator.h_v 1.12 2016/02/08 21:56:18 mju Exp $
 *
 * $Log: featurelocator.h_v $
 * Revision 1.12  2016/02/08 21:56:18  mju
 * No dft detector type in makeOrOpen.
 *
 * Revision 1.11  2015/12/01 13:58:06  mju
 * Add detectortype_BRISK and parm to specify in makeOrOpen.
 *
 * Revision 1.10  2015/11/24 18:47:12  mju
 * Add readBlocks.
 *
 * Revision 1.9  2015/11/19 16:42:42  mju
 * Add tierLevel parm to makeOrOpen.
 * Add getNumItems.
 *
 * Revision 1.8  2015/11/18 18:31:11  mju
 * Add makeOrOpen, change simple make method to private.
 *
 * Revision 1.7  2015/11/13 20:25:08  mju
 * Add readAll.
 *
 * Revision 1.6  2015/11/13 17:57:23  mju
 * Make raster parm non-const.
 *
 * Revision 1.5  2015/11/12 22:50:12  mju
 * Add calcBlockRange/Rect.
 *
 * Revision 1.4  2015/11/11 15:20:00  mju
 * *** empty log message ***
 *
 * Revision 1.3  2015/11/10 20:58:06  mju
 * Add calc private method.
 *
 * Revision 1.2  2015/11/10 18:29:40  mju
 * Add filter for matching detector and/or raster component.
 *
 * Revision 1.1  2015/11/09 18:07:20  mju
 * Initial revision
 * \endif
**/

#ifndef INC_RVC_FEATURELOCATOR_H
#define INC_RVC_FEATURELOCATOR_H

#ifndef INC_RVC_OBJECT_H
	#include <rvc/object.h>
#endif

#ifndef INC_MI32_RECT_H
	#include <mi32/rect.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
namespace cv {
	class Mat;
	class Feature2D;
	}
namespace RVC {
	class FEATURELOCATORHANDLE;
	struct FEATURELOCATORINFO;
	class RASTER;
	}
namespace SPATMOD {
	namespace IMAGE {
		class KEYPOINTS;
		}
	}
#endif


namespace RVC {

//=====================================================================================================================

//! Manage RVC Feature Locator objects.
//!
//!	Object Types that this class supports: RVC::OBJTYPE_FeatureLocator
//!	Valid parents for this class: Any derived from RVC::IMAGE
class FEATURELOCATOR : public RVC::OBJECT {  
	public:

		//! Type of keypoint/descriptor detector.
		enum DETECTORTYPE {
			DETECTORTYPE_Undefined,
			DETECTORTYPE_AKAZE,
			DETECTORTYPE_BRISK,
			};

		class MAKEPARMS {
			public:

				MAKEPARMS ();

				const DPOINT2D& GetBlockSize () const
					{ return (m_BlockSize); }

				int GetDescMatCols () const
					{ return (m_DescMatCols); }

				int GetDescMatType () const
					{ return (m_DescMatType); }

				int GetDescSizeBytes () const
					{ return (m_DescSizeBytes); }

				int GetDetectorSubType () const
					{ return (m_DetectorSubType); }

				DETECTORTYPE GetDetectorType () const
					{ return (m_DetectorType); }

				const DRECT2D& GetExtents () const
					{ return (m_Extents); }

				const LPOINT2D& GetNumBlocks () const
					{ return (m_NumBlocks); }

				UINT8 GetPointDimension () const
					{ return (m_PointDimension); }

				int GetRasterComponent () const
					{ return (m_RasterComponent); }

				void SetDetectorValues (
					cv::Feature2D &detector,
					DETECTORTYPE DetectorType,
					int subtype = 0
					);

				void SetPointValues (
					const DRECT2D &extents,
					const DPOINT2D &BlockSize,
					const LPOINT2D &NumBlocks,
					int PointDimension = 2
					);

				void SetRasterComponent (
					int RasterComponent
					) { m_RasterComponent = RasterComponent; }

				#ifndef GENERATING_DOXYGEN_OUTPUT
				ERRVALUE Get (FEATURELOCATORINFO &objinfo) const;
				void Set (const FEATURELOCATORINFO &objinfo);
				#endif

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				DRECT2D m_Extents;
				DPOINT2D m_BlockSize;
				LPOINT2D m_NumBlocks;
				int m_DescMatType;
				int m_DescMatCols;
				int m_DescSizeBytes;
				DETECTORTYPE m_DetectorType;
				int m_DetectorSubType;
				int m_RasterComponent;
				UINT8 m_PointDimension;
				#endif
			};

		class FILTER : public OBJECT::FILTER {
			public:

				//! Constructor.
				FILTER ();

				//! Destructor.
				virtual ~FILTER ();

				int GetDetectorSubType () const
					{ return (m_DetectorSubType); }

				DETECTORTYPE GetDetectorType () const
					{ return (m_DetectorType); }

				int GetRasterComponent () const
					{ return (m_RasterComponent); }

				//! Set to match detector type and subtype.
				void SetDetectorType (
					DETECTORTYPE DetectorType,
					int subtype = 0
					) { m_DetectorType = DetectorType; m_DetectorSubType = subtype; m_bMatchDetector = true; }

				//! Set to match detector and descriptor values.
				void SetDetectorValues (
					cv::Feature2D &detector,
					DETECTORTYPE DetectorType,
					int subtype = 0
					);

				//! Set to match detector and descriptor values based on MAKEPARMS.
				void SetDetectorValues (
					const MAKEPARMS &makeparms
					);

				//! Set to match raster component.
				void SetRasterComponent (
					int RasterComponent				//!< Component to match
					) { m_RasterComponent = RasterComponent; m_bMatchComponent = true; }

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				int m_DescMatType;
				int m_DescMatCols;
				int m_DescSizeBytes;
				DETECTORTYPE m_DetectorType;
				int m_DetectorSubType;
				int m_RasterComponent;
				bool m_bMatchDetector;
				bool m_bMatchDescriptor;
				bool m_bMatchComponent;

				virtual bool Filter (const RVC::OBJECT &object);
				#endif
			};

		//! Default Constructor
		FEATURELOCATOR ();

		//! Copy Constructor
		FEATURELOCATOR (
			const RVC::FEATURELOCATOR &rhs					
			);										

		//! Destructor
		virtual ~FEATURELOCATOR ();										

		//! Assignment
		RVC::FEATURELOCATOR& operator= (
			const RVC::FEATURELOCATOR &rhs
			);

		//! Calc block index range corresponding to specified rectangle.
		LRECT2D CalcBlockRange (
			const DRECT2D &rect,
			bool bLimitToValid = true
			) const;

		//! Calc rectangle for block.
		DRECT2D CalcBlockRect (
			INT32 blockrow,
			INT32 blockcol
			) const;

		//! Get creation parameters.
		const MAKEPARMS& GetMakeParms () const
			{ return (m_MakeParms); }

		//! Get total number of items.
		INT32 GetNumItems () const;

		//! Make or open for raster/component with default detector.
		ERRVALUE MakeOrOpen (
			RASTER &ObjRaster,							//!< Parent raster object, may be pyramid tier
			DETECTORTYPE DetectorType,					//!< Detector type
			int component = 0,							//!< RASTER::COMPONENT
			int TierLevel = 0,							//!< Pyramid tier level (log2(sampling), 0 = full res)
			bool bCalcAllBlocks = false,				//!< Calculate all blocks
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS
			);
			
		//! Make or open using filter.
		ERRVALUE MakeOrOpen (
			RASTER &ObjRaster,							//!< Parent raster object, may be pyramid tier
			FILTER &filter,								//!< Filter specifying component and detector to match or use
			int TierLevel = 0,							//!< Pyramid tier level (log2(sampling), 0 = full res)
			bool bCalcAllBlocks = false,				//!< Calculate all blocks
			const CREDENTIALS *pCredentials RVC_DEFAULT_CREDENTIALS
			);
			
		//! Read all keypoints.
		ERRVALUE ReadAll (
			SPATMOD::IMAGE::KEYPOINTS &keypoints	//!< Keypoints returned, will be cleared first
			) const;

		//! Read keypoints for single block.
		ERRVALUE ReadBlock (
			UINT32 blockrow,								//!< Block row
			UINT32 blockcol,								//!< Block column
			SPATMOD::IMAGE::KEYPOINTS &keypoints,	//!< Keypoints returned
			bool bClear = true							//!< Whether to clear existing keypoints
			) const;

		//! Read keypoints for range of blocks.
		ERRVALUE ReadBlocks (
			const LRECT2D &blockrange,					//!< Block range, xlast/ylast are "exclusive"
			SPATMOD::IMAGE::KEYPOINTS &keypoints,	//!< Keypoints returned
			bool bClear = true							//!< Whether to clear existing keypoints
			) const;

		//! Read keypoints overlapping specified range.
		//! Note that this actually reads all the blocks overlapping the range, so
		//! points outside the range will usually be included.
		ERRVALUE ReadRange (
			const DRECT2D &range,						//!< Coordinate range
			SPATMOD::IMAGE::KEYPOINTS &keypoints	//!< Keypoints returned, will be cleared first
			) const;

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FEATURELOCATORHANDLE *m_pHandle;
		MAKEPARMS m_MakeParms;

		ERRVALUE CalcBlocks (const LRECT2D &blockrange) const;
		ERRVALUE CalcBlocksLow (RASTER &ObjRaster, cv::Feature2D &detector, const LRECT2D &blockrange) const;
		ERRVALUE Make (const RVC::OBJECT &parent, const RVC::DESCRIPTOR &descriptor, const MAKEPARMS &makeparms, const CREDENTIALS *pCredentials);

		// Implementation of RVC::OBJECT virtual methods.
		virtual ERRVALUE v_CloseObject ();
		virtual RVC::OBJTYPE v_GetDftObjectType () const;
		virtual bool v_IsObjectOpen () const;					
		virtual ERRVALUE v_OpenObject (OPENMODE OpenMode, MDLGPARENT parent, const CREDENTIALS *pCredentials);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};
	
//=====================================================================================================================

}		// End of RVC namespace

#endif	// INC_RVC_FEATURELOCATOR_H
