/**
 * \file rvc/opattributetransfer.h
 * \brief RVC::OP::ATTRIBUTE_TRANSFER definitions
 *
 * \if NODOC
 * $Id: opattributetransfer.h_v 1.3 2015/06/03 19:54:24 mju Exp $
 *
 * $Log: opattributetransfer.h_v $
 * Revision 1.3  2015/06/03 19:54:24  mju
 * Add setSrc/TgtElemIdTableField.
 *
 * Revision 1.2  2015/05/13 16:20:52  mju
 * *** empty log message ***
 *
 * Revision 1.1  2015/05/12 21:44:06  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPATTRIBUTETRANSFER_H
#define	INC_RVC_OPATTRIBUTETRANSFER_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
	#include <mi32/unsignedset.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif

// \if NODOC
// Forward declarations.
class MIJOB;
class XMLNODE;
// \endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Transfer attributes from source object to target object.
class ATTRIBUTE_TRANSFER {
	public:

		//! Element matching criteria.
		enum CRITERIA {
			CRITERIA_Undefined = 0,
			CRITERIA_SourceContainsTarget,
			CRITERIA_TargetContainsSource,
			CRITERIA_Overlapping,
			CRITERIA_Equivalent,
			CRITERIA_NearestSourceForTarget,
			CRITERIA_NearestTargetForSource,
			CRITERIA_NearestBoth,
			CRITERIA_SplitLinesByPolygon,
			CRITERIA_AllWithinDistance
			};

		//! Table handling flags.
		enum TABLEFLAGS {
			TABLEFLAG_None =					0,
			TABLEFLAG_CopyAll =				0x01,		//!< Copy all records, default is to copy only 
			TABLEFLAG_BuiltIn_LabelText =	0x02,		//!< Create table from label text
			TABLEFLAG_MASK_BuiltIn =		TABLEFLAG_BuiltIn_LabelText
			};

		//! Get label for matching criteria.
		static MISTRING GetCriteriaLabel (
			CRITERIA criteria,
			ELEMTYPE ElemTypeSrc,
			ELEMTYPE ElemTypeTgt
			);

		//! Get valid criteria for specified source/target element types.
		//! @return Array of valid CRITERIA values, terminated by CRITERIA_Undefined.
		static const CRITERIA* GetValidCriteria (
			ELEMTYPE ElemTypeSrc,
			ELEMTYPE ElemTypeTgt
			);

		//! Determine if distance value is used for specified criteria and element types.
		static bool IsDistanceUsed (
			CRITERIA criteria,
			ELEMTYPE ElemTypeSrc,
			ELEMTYPE ElemTypeTgt
			);

		//! Constructor.
		ATTRIBUTE_TRANSFER ();

		//! Destructor.
		~ATTRIBUTE_TRANSFER ();

		//! Add table to transfer.
		void AddTable (
			const MISTRING &TableNameSrc,			//!< Source table name
			const DESCRIPTOR &TableDescTgt,		//!< Target table descriptor, if existing will join (must be compatible)
			TABLEFLAGS TableFlags					//!< Table handling flags
			);

		//! Get label for matching criteria.
		MISTRING GetCriteriaLabel () const
			{ return (GetCriteriaLabel(m_MatchCriteria,m_ElemTypeSrc,m_ElemTypeTgt)); }

		//! Get source element type.
		ELEMTYPE GetElemTypeSrc () const
			{ return (m_ElemTypeSrc); }

		//! Get target element type.
		ELEMTYPE GetElemTypeTgt () const
			{ return (m_ElemTypeTgt); }

		//! Get source object location.
		const OBJITEM& GetObjItemSrc () const
			{ return (m_ObjItemSrc); }

		//! Get target object location.
		const OBJITEM& GetObjItemTgt () const
			{ return (m_ObjItemTgt); }

		//! Determine if distance value is used.
		bool IsDistanceUsed () const
			{ return (IsDistanceUsed(m_MatchCriteria,m_ElemTypeSrc,m_ElemTypeTgt)); }

		//! Read all parameters from job file.
		ERRVALUE JobRead (
			MIJOB &job
			);

		//! Write parameters to job file.
		ERRVALUE JobWrite (
			MIJOB &job
			);

		//! Run operation.
		ERRVALUE Run ();

		//! Set location for report file.
		void SetFilePathReport (
			const FILEPATH &filepath
			) { m_FilePathReport = filepath; }

		//! Set criteria for matching elements.
		void SetMatchCriteria (
			CRITERIA MatchCriteria,					//!< Match criteria
			double MatchDistance						//!< Match distance
			) { m_MatchCriteria = MatchCriteria; m_MatchDistance = MatchDistance; }

		void SetObjItemCopyTo (
			const OBJITEM &ObjItemCopyTo
			) { m_ObjItemCopyTo = ObjItemCopyTo; }

		//! Set source object and element type.
		ERRVALUE SetSource (
			const OBJITEM &ObjItem,					//!< Source object
			ELEMTYPE ElemType,						//!< Source element type (Point, Line, Polygon, Label, RasterCellValue)
			const UNSIGNEDSET *pElemSet = 0		//!< Optional subset of elements to use
			);

		//! Set source element ID table/field.
		void SetSrcElemIdTableField (
			const MISTRING &TableField				//!< Table.Field string
			) { m_SrcElemIdTableField = TableField; }

		//! Set target object and element type.
		ERRVALUE SetTarget (
			const OBJITEM &ObjItem,					//!< Target object
			ELEMTYPE ElemType,						//!< Target element type (Point, Line, Polygon, Label)
			const UNSIGNEDSET *pElemSet = 0		//!< Optional subset of elements to use
			);

		//! Set target element ID table/field.
		void SetTgtElemIdTableField (
			const MISTRING &TableField				//!< Table.Field string
			) { m_TgtElemIdTableField = TableField; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		class TABLEITEM {
			public:
				TABLEITEM ();
				TABLEITEM (const MISTRING &NameSrc, const DESCRIPTOR &DescTgt, TABLEFLAGS flags);
				~TABLEITEM ();

				TABLEFLAGS GetFlags () const
					{ return (m_Flags); }

				const MISTRING& GetSrcName () const
					{ return (m_NameSrc); }

				const DESCRIPTOR& GetTgtDesc () const
					{ return (m_DescTgt); }

				void GetXml (XMLNODE *node) const;

				ERRVALUE SetXml (const XMLNODE *node);

			private:
				MISTRING m_NameSrc;
				DESCRIPTOR m_DescTgt;
				TABLEFLAGS m_Flags;
			};
		typedef MILIST<TABLEITEM> TABLELIST;

		OBJITEM m_ObjItemSrc;
		OBJITEM m_ObjItemTgt;
		mutable OBJITEM m_ObjItemCopyTo;
		FILEPATH m_FilePathReport;
		UNSIGNEDSET m_ElemSetSrc;
		UNSIGNEDSET m_ElemSetTgt;
		TABLELIST m_TableList;
		MISTRING m_SrcElemIdTableField;
		MISTRING m_TgtElemIdTableField;
		double m_MatchDistance;
		CRITERIA m_MatchCriteria;
		ELEMTYPE m_ElemTypeSrc;
		ELEMTYPE m_ElemTypeTgt;

		ERRVALUE CheckValid () const;
		bool IsSrcElemSelected (const ELEMIDX &elem) const;
		bool IsTgtElemSelected (const ELEMIDX &elem) const;

		ATTRIBUTE_TRANSFER (ATTRIBUTE_TRANSFER &);
		ATTRIBUTE_TRANSFER& operator= (ATTRIBUTE_TRANSFER &);
		#endif
	};

//	\if NODOC
DEFINE_ENUM_OP_BITWISE(ATTRIBUTE_TRANSFER::TABLEFLAGS)
// \endif

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPATTRIBUTETRANSFER_H
