/**
 * \file rvc/opimagerasterstatistics.h
 * \brief RVC::OP::IMAGE_RASTER_STATISTICS definition
 *
 * \if NODOC
 * $Id: opimagerasterstatistics.h_v 1.2 2014/08/21 16:10:09 mju Exp $
 *
 * $Log: opimagerasterstatistics.h_v $
 * Revision 1.2  2014/08/21 16:10:09  mju
 * Use relative not normalized mean.
 *
 * Revision 1.1  2014/08/18 19:35:49  mju
 * Initial revision
 *
 *	\endif
**/

#ifndef	INC_RVC_IMAGERASTERSTATISTICS_H
#define	INC_RVC_IMAGERASTERSTATISTICS_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_MILIST_H
	#include <mi32/milist.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MIJOB;
class XMLNODE;
#endif // GENERATING_DOXYGEN_OUTPUT

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Compute image statistics by category raster cell values.
class IMAGE_RASTER_STATISTICS {
	public:

		//! Statistics to compute.
		enum STATS {
			STAT_None =					0,
			STAT_Count =				0x0001,	//!< Cell count
			STAT_Minimum =				0x0002,
			STAT_Maximum =				0x0004,
			STAT_Mean =					0x0008,
			STAT_Median =				0x0010,
			STAT_Mode =					0x0020,
			STAT_StdDev =				0x0040,
			STAT_RelStdDevPct =		0x0080,
			STAT_RelativeMean =		0x0100,
			STAT_Histogram =			0x1000,
			STATS_AllNoHistogram =	0x01FF
			};

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		//! Constructor.
		IMAGE_RASTER_STATISTICS ();

		//! Destructor.
		~IMAGE_RASTER_STATISTICS ();

		//! Add image to process.
		//! If image is multiband or hyperspectral, the band name/number will be appended to corresponding field names.
		ERRVALUE AddObjImage (
			const OBJITEM& ObjItem,
			const OBJITEM *pObjItemTableStat = 0,		//!< Descriptor or filepath of output table for statistics by attribute
			const OBJITEM *pObjItemTableHistogram = 0	//!< Descriptor or filepath of output table for histogram by attribute
			);

		//! Read all parameters from job file.
		ERRVALUE JobRead (
			MIJOB& job
			);

		//! Write parameters to job file.
		ERRVALUE JobWrite (
			MIJOB& job
			);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set whether to allow spaces in field names.
		void SetAllowSpaceInFieldName (
			bool bAllowSpaceInFieldName
			) { m_bAllowSpaceInFieldName = bAllowSpaceInFieldName; }

		//! Set field for category name or id.
		void SetCategoryIdField (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_IdTableName = TableName; m_IdFieldName = FieldName; m_bIdUseCellValue = false; }

		//! Set category id to use cell value.
		void SetCategoryIdUseCellValue ()
			{ m_bIdUseCellValue = true; }

		//! Set raster object for categories.
		ERRVALUE SetObjItemCategory (
			const RVC::OBJITEM& ObjItem			//!< Category Raster object location
			);

		//! Set optional mask raster to use.
		ERRVALUE SetObjItemMask (
			const RVC::OBJITEM& ObjItem			//!< Mask raster location
			);

		//! Set output table name / location when have all images in same table.
		void SetOutputTableStatAll (
			const RVC::OBJITEM& ObjItemTable		//!< Table location (descriptor or filepath)
			) { m_ObjItemTableStatAll = ObjItemTable; }

		//! Set which statistics to compute.
		void SetStats (
			STATS stats									//!< Statistics to compute
			) { m_Stats = stats; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		enum TARGET {
			TARGET_Existing,
			TARGET_Text
			};

		class IMAGEITEM {
			public:
				IMAGEITEM ();
				IMAGEITEM (
					const OBJITEM& ObjItemImage,
					const OBJITEM *pObjItemTableStat,
					const OBJITEM *pObjItemTableHistogram
					);
				~IMAGEITEM ();

				const OBJITEM& GetObjItemImage () const
					{ return (m_ObjItemImage); }

				const OBJITEM& GetObjItemTableHistogram () const
					{ return (m_ObjItemTableHistogram); }

				const OBJITEM& GetObjItemTableStat () const
					{ return (m_ObjItemTableStat); }

				void GetXml (XMLNODE *node) const;

				ERRVALUE SetXml (const XMLNODE *node);

			private:
				OBJITEM m_ObjItemImage;
				OBJITEM m_ObjItemTableStat;
				OBJITEM m_ObjItemTableHistogram;
			};
		typedef MILIST<IMAGEITEM> IMAGEITEMLIST;

		RVC::OBJITEM m_ObjItemCategory;
		RVC::OBJITEM m_ObjItemMask;
		RVC::OBJITEM m_ObjItemTableStatAll;
		MISTRING m_IdTableName;
		MISTRING m_IdFieldName;
		IMAGEITEMLIST m_ImageItemList;
		STATS m_Stats;
		bool m_bAllowSpaceInFieldName;
		bool m_bIdUseCellValue;
		bool m_bMultiImageFields;
		bool m_bOutputText;
	
		static void SML_Compute (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);

		ERRVALUE CheckSettings ();
		ERRVALUE RunSML (SML::CONTEXT *context, SML::ARG *args, int numargs);

		IMAGE_RASTER_STATISTICS (const IMAGE_RASTER_STATISTICS&);
		IMAGE_RASTER_STATISTICS& operator= (const IMAGE_RASTER_STATISTICS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(IMAGE_RASTER_STATISTICS::STATS)
#endif

//=====================================================================================================================

} // End OP namespace
} // End RVC namespace

#endif	// INC_RVC_IMAGERASTERSTATISTICS_H
