/**
 * \file rvc/opimageregionstatistics.h
 * \brief RVC::OP::IMAGE_REGION_STATISTICS definitions
 *
 * \if NODOC
 * $Id: opimageregionstatistics.h_v 1.4 2016/11/14 21:55:45 mju Exp $
 *
 * $Log: opimageregionstatistics.h_v $
 * Revision 1.4  2016/11/14 21:55:45  mju
 * Move mask extents to operation.
 *
 * Revision 1.3  2015/02/18 16:01:47  mju
 * Add member for keeping track of whether glcm bin interval is integer.
 *
 * Revision 1.2  2014/09/02 19:32:44  mju
 * Add glcm support.
 *
 * Revision 1.1  2014/08/07 17:10:44  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPIMAGEREGIONSTATISTICS_H
#define	INC_RVC_OPIMAGEREGIONSTATISTICS_H

#ifndef	INC_MI32_STATUNIVAR_H
#include <mi32/statunivar.h>
#endif

#ifndef	INC_MI32_RECT_H
#include <mi32/rect.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class REGION2D;
class GLCM;
namespace RVC {
	class OBJITEM;
	class IMAGE;
	class NULLMASK;
	class RASTER;
	}
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Calculate (univariate) image statistics within region.
class IMAGE_REGION_STATISTICS {
	public:

		//! Boundary cell handling.
		enum BOUNDARYCELLS {
			BOUNDARYCELLS_IncludeAll =		1,	//!< Include if partially inside
			BOUNDARYCELLS_IncludeCenter =	2,	//!< Include if center inside
			BOUNDARYCELLS_Proportional =	3,	//!< Count proportional to area inside
			BOUNDARYCELLS_ExcludeAll =		4	//!< Exclude if partially outside
			};

		enum FLAGS {
			FLAG_None =						0,
			FLAG_MatchHistogramBins =	0x01,	//!< Match bin range/interval used with histogram
			FLAG_GLCM_Calc =				0x02,	//!< Compute Gray Level Cooccurrence Matrix (GLCM)
			};

		//! Constructor.
		IMAGE_REGION_STATISTICS ();

		//! Destructor.
		~IMAGE_REGION_STATISTICS ();

		//! Compute statistics for region.
		ERRVALUE ComputeStatistics (
			const REGION2D& region					//!< Region in image coordinates
			);

		//! Get GLCM for specified band.
		const GLCM& GetGLCM (
			UINT32 BandNum = 0
			) const;

		//! Get statistics for specified band.
		const STAT_UNIVARIATE& GetStatistics (
			UINT32 BandNum = 0
			) const { return (m_StatArray[(BandNum < m_NumBands) ? BandNum : 0]); }

		//! Initialize for specified image.
		ERRVALUE Initialize (
			const RVC::OBJITEM& ObjItemImage,	//!< Image location
			BOUNDARYCELLS BoundaryCells,			//!< Boundary cell handling
			FLAGS flags = FLAG_None
			);

		//! Set number of bins for GLCM.
		//! Will use 16 bins if not specified. Must be called before Initialize.
		void SetNumBinsGLCM (
			UINT16 NumBins								//!< Number of bins (2-64) in each direction
			) { m_NumBinsGLCM = NumBins; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		RVC::IMAGE *m_pObjImage;
		RVC::NULLMASK *m_pObjNullMask;
		RVC::RASTER *m_pObjTempMask;
		STAT_UNIVARIATE *m_StatArray;
		GLCM *m_glcmArray;
		bool *m_bIntervalIntGLCM;
		RANGE<double> *m_BinRanges;
		double *m_BinIntervals;
		LRECT2D m_LastMaskExtents;
		UINT32 m_NumBands;
		UINT16 m_NumBinsGLCM;
		BOUNDARYCELLS m_BoundaryCells;
		bool m_bNeedRange;

		void Free ();

		IMAGE_REGION_STATISTICS (const IMAGE_REGION_STATISTICS&);
		IMAGE_REGION_STATISTICS& operator= (const IMAGE_REGION_STATISTICS&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(IMAGE_REGION_STATISTICS::FLAGS)
#endif

//=====================================================================================================================

} // End OP namespace
} // End RVC namespace

#endif	// INC_RVC_OPIMAGEREGIONSTATISTICS_H
