/**
 * \file rvc/oppolygonstatistics.h
 * \brief RVC::OP::POLYGON_STATISTICS definitions
 *
 * \if NODOC
 * $Id: oppolygonstatistics.h_v 1.6 2014/07/22 21:11:15 mju Exp $
 *
 * $Log: oppolygonstatistics.h_v $
 * Revision 1.6  2014/07/22 21:11:15  mju
 * Add operation_mask_ByElement.
 *
 * Revision 1.5  2014/07/15 15:02:42  mju
 * Exclude private section from dox.
 *
 * Revision 1.4  2014/07/11 21:08:56  mju
 * Add settings to summarize by attribute, specify id field.
 *
 * Revision 1.3  2014/07/07 20:19:36  mju
 * Add output enum, rename method for separation attribute.
 *
 * Revision 1.2  2014/07/07 14:13:43  mju
 * Add setting for including zero count totals.
 *
 * Revision 1.1  2014/07/03 21:48:47  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPPOLYGONSTATISTICS_H
#define	INC_RVC_OPPOLYGONSTATISTICS_H

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
#include <mi32/unsignedset.h>
#endif

#ifndef	INC_MI32_MILIST_H
#include <mi32/milist.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
class MIJOB;
class UNSIGNEDSET;
class XMLNODE;
namespace RVC {
	class GEOTABLE;
	}
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Compute statistics by polygon using elements from another object.
class POLYGON_STATISTICS {
	public:

		//! Operations to perform.
		enum OPERATION {
			OPERATION_None =									0,
			OPERATION_ByElement_Total =					0x01,	//!< Totals for target element/attribute
			OPERATION_ByElement_FieldsByAttribute =	0x02,	//!< By source element using separate fields by attribute
			OPERATION_ByElement_RecordsByAttribute =	0x04,	//!< By source element using separate records by attribute
			OPERATION_ByField_Total =						0x10,	//!< For source fields, totals by target element
			OPERATION_ByField_RecordsByAttribute =		0x40,	//!< Separate records by attribute
			OPERATION_MASK_Total =							OPERATION_ByElement_Total | OPERATION_ByField_Total,
			OPERATION_MASK_ByAttribute =					OPERATION_ByElement_FieldsByAttribute | OPERATION_ByElement_RecordsByAttribute | OPERATION_ByField_RecordsByAttribute,
			OPERATION_MASK_ByElement =						OPERATION_ByElement_Total | OPERATION_ByElement_FieldsByAttribute | OPERATION_ByElement_RecordsByAttribute,
			OPERATION_MASK_ByField =						OPERATION_ByField_Total | OPERATION_ByField_RecordsByAttribute,
			};

		//! Element statistics to compute.
		enum ELEMSTATS {
			ELEMSTAT_None =				0,
			ELEMSTAT_Count =				0x01,
			ELEMSTAT_Length =				0x02,
			ELEMSTAT_Area =				0x04,
			ELEMSTAT_CoveragePct =		0x08,
			ELEMSTAT_All =					0xFF,
			};

		//! Field statistics to compute.
		enum FIELDSTATS {
			FIELDSTAT_None =				0,
			FIELDSTAT_Minimum =			0x01,
			FIELDSTAT_Maximum =			0x02,
			FIELDSTAT_Mean =				0x04,
			FIELDSTAT_Variance =			0x08,
			FIELDSTAT_StdDev =			0x10,
			FIELDSTAT_RelStdDevPct =	0x20,
			FIELDSTATS_All =				0xFF
			};

		enum OUTPUT {
			OUTPUT_None =				0,
			OUTPUT_ByPolygon =		0x01,
			OUTPUT_Summary =			0x02,
			};

		//! Constructor.
		POLYGON_STATISTICS ();

		//! Destructor.
		~POLYGON_STATISTICS ();

		//! Add field for statistics.
		void AddStatField (
			UINT16 FieldNum,															//!< Field number in table specified via SetFieldStats()
			const MISTRING *pTableNameOrFilePathByPolygonTotal = 0,		//!< Table name or encoded file path for total if separate
			const MISTRING *pTableNameOrFilePathByPolygonByAttrib = 0,	//!< Table name or encoded file path for by attribute if separate
			const MISTRING *pTableNameOrFilePathSummaryTotal = 0,			//!< Table name or encoded file path for total if separate
			const MISTRING *pTableNameOrFilePathSummaryByAttrib = 0		//!< Table name or encoded file path for by attribute if separate
			);

		ERRVALUE JobRead (
			MIJOB& job
			);

		ERRVALUE JobWrite (
			MIJOB& job
			);

		ERRVALUE Run ();

		void SetAllowSpaceInFieldName (
			bool bAllowSpaceInFieldName
			) { m_bAllowSpaceInFieldName = bAllowSpaceInFieldName; }

		void SetElemStats (
			ELEMSTATS ElemStat
			) { m_ElemStat = ElemStat; }

		void SetFieldStats (
			FIELDSTATS FieldStat,
			const MISTRING& TableNameSource
			) { m_FieldStat = FieldStat; m_FieldStatTableName = TableNameSource; }

		void SetIncludeZeroCountTotal (
			bool bIncludeZeroCountTotal
			) { m_bIncludeZeroCountTotal = bIncludeZeroCountTotal; }

		ERRVALUE SetObjPolygons (
			const RVC::OBJITEM& ObjItem,
			const UNSIGNEDSET *pElemSet = 0		//!< Set of elements to process, 0 for all
			);

		ERRVALUE SetObjSource (
			const RVC::OBJITEM& ObjItem,
			ELEMTYPE ElemType,
			const UNSIGNEDSET *pElemSet = 0
			);

		ERRVALUE SetObjSource (
			const RVC::OBJITEM& ObjItemTable,
			const RVC::GEOTABLE& GeoTable,
			const UNSIGNEDSET *pElemSet = 0
			);

		void SetOperation (OPERATION operation)
			{ m_operation = operation; }

		void SetOutput (
			OUTPUT Output
			) { m_Output = Output; }

		void SetOutputCopyTo (
			const RVC::OBJITEM& ObjItemCopyTo
			) { m_ObjItemOutput = ObjItemCopyTo; m_OutputMode = OUTPUTMODE_CopyPolygons; }

		void SetOutputTableByPolygonByAttribute (
			const MISTRING& TableNameOrFilePath		//!< Table name or encoded file path
			) { m_OutputTableNameByPolygonByAttribute = TableNameOrFilePath; }

		void SetOutputTableByPolygonTotal (
			const MISTRING& TableNameOrFilePath		//!< Table name or encoded file path
			) { m_OutputTableNameByPolygonTotal = TableNameOrFilePath; }

		void SetOutputTableSummaryByAttribute (
			const MISTRING& TableNameOrFilePath		//!< Table name or encoded file path
			) { m_OutputTableNameSummaryByAttribute = TableNameOrFilePath; }

		void SetOutputTableSummaryTotal (
			const MISTRING& TableNameOrFilePath		//!< Table name or encoded file path
			) { m_OutputTableNameSummaryTotal = TableNameOrFilePath; }

		void SetSourceSeparationAttribute (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_SeparationTableName = TableName; m_SeparationFieldName = FieldName; }

		void SetTargetPolygonIdField (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_PolygonIdTableName = TableName; m_PolygonIdFieldName = FieldName; }

		void SetTargetPolygonIdUseElemNum ();

		void SetTargetSummaryAttribute (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_SummaryTableName = TableName; m_SummaryFieldName = FieldName; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		//! Mode for determining where the output goes.
		enum OUTPUTMODE {
			OUTPUTMODE_ExistingPolygons = 1,
			OUTPUTMODE_CopyPolygons,
			OUTPUTMODE_Text
			};

		class ENGINE;

		class STATFIELD {
			public:

				STATFIELD ();
				STATFIELD (const STATFIELD& rhs);

				STATFIELD (
					UINT16 FieldNum,
					const MISTRING *pTableNameByPolygonTotal,
					const MISTRING *pTableNameByPolygonByAttribute,
					const MISTRING *pTableNameSummaryTotal,
					const MISTRING *pTableNameSummaryByAttribute
					);

				~STATFIELD ();
				STATFIELD& operator= (const STATFIELD& rhs);

				UINT16 GetFieldNum () const
					{ return (m_FieldNum); }

				const MISTRING& GetTableNameByPolygonByAttribute () const
					{ return (m_TableNameByPolygonByAttribute); }

				const MISTRING& GetTableNameByPolygonTotal () const
					{ return (m_TableNameByPolygonTotal); }

				const MISTRING& GetTableNameSummaryByAttribute () const
					{ return (m_TableNameSummaryByAttribute); }

				const MISTRING& GetTableNameSummaryTotal () const
					{ return (m_TableNameSummaryTotal); }

				void GetXml (XMLNODE *node) const;

				ERRVALUE SetXml (const XMLNODE *node);

			private:
				MISTRING m_TableNameByPolygonTotal;
				MISTRING m_TableNameByPolygonByAttribute;
				MISTRING m_TableNameSummaryTotal;
				MISTRING m_TableNameSummaryByAttribute;
				UINT16 m_FieldNum;
			};
		typedef MILIST<STATFIELD> STATFIELDLIST;

		MISTRING m_PolygonIdTableName;
		MISTRING m_PolygonIdFieldName;
		MISTRING m_SummaryTableName;
		MISTRING m_SummaryFieldName;
		MISTRING m_FieldStatTableName;
		MISTRING m_SeparationTableName;
		MISTRING m_SeparationFieldName;
		MISTRING m_OutputTableNameByPolygonTotal;
		MISTRING m_OutputTableNameByPolygonByAttribute;
		MISTRING m_OutputTableNameSummaryTotal;
		MISTRING m_OutputTableNameSummaryByAttribute;
		RVC::OBJITEM m_ObjItemPolygons;
		RVC::OBJITEM m_ObjItemSource;
		RVC::OBJITEM m_ObjItemOutput;
		STATFIELDLIST m_StatFieldList;
		UNSIGNEDSET m_ElemSetPolygons;
		UNSIGNEDSET m_ElemSetSource;
		RVC::GEOTABLE *m_pGeoTableSource;
		OUTPUT m_Output;
		OPERATION m_operation;
		ELEMSTATS m_ElemStat;
		FIELDSTATS m_FieldStat;
		ELEMTYPE m_SourceType;
		OUTPUTMODE m_OutputMode;
		bool m_bIncludeZeroCountTotal;
		bool m_bAllowSpaceInFieldName;

		ERRVALUE CheckSettings ();
		bool HasSeparateTablesByField () const;

		POLYGON_STATISTICS (const POLYGON_STATISTICS&);
		POLYGON_STATISTICS& operator= (const POLYGON_STATISTICS&);

		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(POLYGON_STATISTICS::OUTPUT)
DEFINE_ENUM_OP_BITWISE(POLYGON_STATISTICS::OPERATION)
DEFINE_ENUM_OP_BITWISE(POLYGON_STATISTICS::ELEMSTATS)
DEFINE_ENUM_OP_BITWISE(POLYGON_STATISTICS::FIELDSTATS)
#endif

//=====================================================================================================================

}  // end namespace OP
}  // end namespace RVC

#endif	// INC_RVC_OPPOLYGONSTATISTICS_H
