/**
 * \file rvc/opsurfacegeometricstatistics.h
 * \brief RVC::OP::SURFACE_GEOMETRIC_STATISTICS definition
 *
 * \if NODOC
 * $Id: opsurfacegeometricstatistics.h_v 1.4 2014/09/02 21:31:03 mju Exp $
 *
 * $Log: opsurfacegeometricstatistics.h_v $
 * Revision 1.4  2014/09/02 21:31:03  mju
 * Add elevationStdDev and areaRatio stats.
 *
 * Revision 1.3  2014/08/25 17:38:07  mju
 * Use slope not grade.
 *
 * Revision 1.2  2014/08/25 14:55:35  mju
 * Add methods to set units and elevation scale/offset.
 *
 * Revision 1.1  2014/08/22 21:30:25  mju
 * Initial revision
 *	\endif
**/

#ifndef	INC_RVC_SURFACEGEOMETRICSTATISTICS_H
#define	INC_RVC_SURFACEGEOMETRICSTATISTICS_H

#ifndef	INC_RVC_OBJITEM_H
	#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
	#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_UNSIGNEDSET_H
	#include <mi32/unsignedset.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
	#include <mi32/smlbase.h>
#endif

#ifndef	INC_MI32_UOM_H
	#include <mi32/uom.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
class MIJOB;
#endif

namespace RVC {
namespace OP {

//=====================================================================================================================

//! Compute raster properties for a geometric object.
class SURFACE_GEOMETRIC_STATISTICS {
	public:

		//! Statistics to record.
		enum STATS {
			STAT_None =							0,
			STAT_ElevationMin =				0x0000001,
			STAT_ElevationMax =				0x0000002,
			STAT_ElevationMean =				0x0000004,
			STAT_ElevationChange =			0x0000008,	//!< Difference from line start to end (EndZ - StartZ)
			STAT_SlopeMin =					0x0000010,	//!< Minimum grade (slope) %
			STAT_SlopeMax =					0x0000020,	//!< Maximum grade (slope) %
			STAT_SlopeMean =					0x0000040,	//!< Mean grade %
			STAT_AreaOrLength =				0x0000080,	//!< Surface area or length
			STAT_Coverage =					0x0000100,
			STAT_VolumeBelowRef =			0x0000200,	//!< Volume below reference level and (above) surface within area
			STAT_VolumeAboveRef =			0x0000400,	//!< Volume above reference level and (below) surface within area
			STAT_AspectMean =					0x0000800,	//!< Mean Aspect for area
			STAT_AreaRatio =					0x0001000,	//!< Ratio of 3D area to flat area
			STAT_ElevationStdDev =			0x0002000,	//!< Elevation standard deviation
			STAT_BorderElevationMin =		0x0010000,
			STAT_BorderElevationMax =		0x0020000,
			STAT_BorderElevationMean =		0x0040000,
			STAT_BorderSlopeMin =			0x0080000,
			STAT_BorderSlopeMax =			0x0100000,
			STAT_BorderSlopeMean =			0x0200000,
			STAT_BorderLength =				0x0400000,
			STAT_BorderCoverage =			0x0800000,
			STAT_MASK_Area =					0x0003FF7,
			STAT_MASK_Border =				0x0FF0000,
			STAT_MASK_Line =					0x00001FF,
			STAT_MASK_UseParm_Line =		0,
			STAT_MASK_UseParm_Area =		STAT_VolumeBelowRef | STAT_VolumeAboveRef,
			};

		//! Install SML interface.
		static void SML_Install (
			SML::CONTEXT *context
			);

		static MISTRING GetStatName (
			STATS stat,
			ELEMTYPE ElemType = ELEMTYPE_Empty
			);

		//! Constructor.
		SURFACE_GEOMETRIC_STATISTICS ();

		//! Destructor.
		~SURFACE_GEOMETRIC_STATISTICS ();

		//! Read all parameters from job file.
		ERRVALUE JobRead (
			MIJOB& job
			);

		//! Write parameters to job file.
		ERRVALUE JobWrite (
			MIJOB& job
			);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set whether to allow spaces in field names.
		void SetAllowSpaceInFieldName (
			bool bAllowSpaceInFieldName
			) { m_bAllowSpaceInFieldName = bAllowSpaceInFieldName; }

		//! Set field for element id.
		void SetElementIdField (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_ElementIdTableName = TableName; m_ElementIdFieldName = FieldName; m_bElementIdUseElemNum = false; }

		//! Set element id to use element number.
		void SetElementIdUseElemNum ()
			{ m_bElementIdUseElemNum = true; }

		//! Set elevation scale/offset.
		//! If not specified, will be determined based on IMAGESCALE data unit settings.
		void SetElevationScaleOffset (
			double ElevationScale,				//!< Scale factor to convert raster values to meters
			double ElevationOffset = 0			//!< Offset
			) { m_ElevationScale = ElevationScale; m_ElevationOffset = ElevationOffset; }

		//! Set to copy geometric object to specified location before adding tables.
		void SetObjItemCopyTo (
			const OBJITEM& ObjItemCopyTo
			) { m_ObjItemCopyTo = ObjItemCopyTo; m_Target = TARGET_Copy; }

		//! Set geometric object and general element type to use.
		ERRVALUE SetObjItemGeometric (
			const OBJITEM& ObjItem,				//!< Geometric object location
			ELEMTYPE ElemType,					//!< General element type (Line or Polygon)
			const UNSIGNEDSET *pElemSet = 0	//!< Set of elements to process, NULL for all
			);

		//! Set surface object to use.
		ERRVALUE SetObjItemSurface (
			const OBJITEM& ObjItem				//!< Surface object location
			);

		//! Set output table name / location with fields for each raster when summarize by attribute.
		void SetOutputTableByAttribute (
			const OBJITEM& objitem				//!< Location of table
			) { m_ObjItemTableByAttribute = objitem; }

		//! Set output table name / location with fields for each raster when summarize by element.
		void SetOutputTableByElement (
			const OBJITEM& objitem				//!< Location of table
			) { m_ObjItemTableByElement = objitem; }

		//! Set output table name / location for settings.
		void SetOutputTableParameters (
			const OBJITEM& objitem				//!< Location of table
			) { m_ObjItemTableParameters = objitem; }

		//! Set which statistics to compute by attribute.
		void SetStatsByAttribute (
			STATS stats								//!< Statistics to compute, STAT_None if not computing by attribute
			) { m_StatsByAttribute = stats; }

		//! Set which statistics to compute by element.
		void SetStatsByElement (
			STATS stats								//!< Statistics to compute, STAT_None if not computing by element
			) { m_StatsByElement = stats; }

		//! Set field for summary by attribute.
		void SetSummaryAttribute (
			const MISTRING& TableName,
			const MISTRING& FieldName
			) { m_SummaryTableName = TableName; m_SummaryFieldName = FieldName; }

		//! Set unit for area.
		void SetUnitArea (
			UOM::IDNUM unit						//!< Unit IDNUM
			) { m_UnitIdArea = unit; }

		//! Set unit for elevation.
		void SetUnitElevation (
			UOM::IDNUM unit						//!< Unit IDNUM
			) { m_UnitIdElevation = unit; }

		//! Set unit for grade slope calculation.
		void SetUnitSlope (
			UOM::IDNUM unit						//!< Angle or Scale unit
			) { m_UnitIdSlope = unit; }

		//! Set unit for length.
		void SetUnitLength (
			UOM::IDNUM unit						//!< Unit IDNUM
			) { m_UnitIdLength = unit; }

		//! Set unit for volume calculation.
		void SetUnitVolume (
			UOM::IDNUM unit						//!< Unit IDNUM
			) { m_UnitIdVolume = unit; }

		//! Set reference elevation for volume calculation.
		void SetVolumeRefElev (
			double VolumeRefElev					//!< Reference elevation (per specified elevation units)
			) { m_VolumeRefElev = VolumeRefElev; }

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		enum TARGET {
			TARGET_Existing,
			TARGET_Copy,
			TARGET_Text
			};

		OBJITEM m_ObjItemGeometric;
		OBJITEM m_ObjItemCopyTo;
		OBJITEM m_ObjItemSurface;
		UNSIGNEDSET m_ElemSetGeometric;
		MISTRING m_ElementIdTableName;
		MISTRING m_ElementIdFieldName;
		MISTRING m_SummaryTableName;
		MISTRING m_SummaryFieldName;
		OBJITEM m_ObjItemTableByElement;
		OBJITEM m_ObjItemTableByAttribute;
		OBJITEM m_ObjItemTableParameters;
		double m_ElevationScale;
		double m_ElevationOffset;
		double m_VolumeRefElev;			// Reference elevation for volume
		ELEMTYPE m_ElemType;
		TARGET m_Target;
		STATS m_StatsByElement;
		STATS m_StatsByAttribute;
		UOM::IDNUM m_UnitIdElevation;
		UOM::IDNUM m_UnitIdLength;
		UOM::IDNUM m_UnitIdSlope;
		UOM::IDNUM m_UnitIdArea;
		UOM::IDNUM m_UnitIdVolume;
		bool m_bAllowSpaceInFieldName;
		bool m_bElementIdUseElemNum;
	
		static void SML_Compute (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);

		ERRVALUE CheckSettings ();
		ERRVALUE RunSML (SML::CONTEXT *context, SML::ARG *args, int numargs);

		SURFACE_GEOMETRIC_STATISTICS (const SURFACE_GEOMETRIC_STATISTICS&);
		SURFACE_GEOMETRIC_STATISTICS& operator= (const SURFACE_GEOMETRIC_STATISTICS&);
		#endif // GENERATING_DOXYGEN_OUTPUT
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SURFACE_GEOMETRIC_STATISTICS::STATS)
#endif

//=====================================================================================================================

} // End OP namespace
} // End RVC namespace

#endif	// INC_RVC_SURFACEGEOMETRICSTATISTICS_H
