/**
 * \file rvc/opsurfacetogeometric.h
 * \brief RVC::OP::SURFACE_TO_GEOMETRIC definitions
 *
 * \if NODOC
 * $Id: opsurfacetogeometric.h_v 1.1 2014/11/05 22:35:02 mju Exp $
 *
 * $Log: opsurfacetogeometric.h_v $
 * Revision 1.1  2014/11/05 22:35:02  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_RVC_OPSURFACETOGEOMETRICPOINTS_H
#define	INC_RVC_OPSURFACETOGEOMETRICPOINTS_H

#ifndef	INC_RVC_OBJITEM_H
#include <rvc/objitem.h>
#endif

#ifndef	INC_MI32_ELEMTYPE_H
#include <mi32/elemtype.h>
#endif

#ifndef	INC_MI32_SMLBASE_H
#include <mi32/smlbase.h>
#endif


#ifndef GENERATING_DOXYGEN_OUTPUT
class MIJOB;
#endif

//=====================================================================================================================

namespace RVC {
namespace OP {

//! Convert surface raster to geometric points or mesh lines.
class SURFACE_TO_GEOMETRIC {
	public:

		//! Point types to generate.
		enum POINTTYPE {
			POINTTYPE_None =					0,
			POINTTYPE_LocalMaximum =		0x0001,
			POINTTYPE_LocalMinimum =		0x0002,
			POINTTYPE_Saddle =				0x0004,
			POINTTYPE_SaddleX =				0x0008,
			POINTTYPE_SubExtremum1 =		0x0010,
			POINTTYPE_SubExtremum2 =		0x0020,
			POINTTYPE_SubExtremum3 =		0x0040,
			POINTTYPE_Regular =				0x1000	//!< Output regular points or line mesh, cannot combine with other values
			};

		//! Get label for point type.
		//! Will return empty string for multiple types.
		static MISTRING GetLabel (POINTTYPE type);

		//! Install SML interface.
		static void SML_Install (SML::CONTEXT *context);

		//! Constructor.
		SURFACE_TO_GEOMETRIC ();

		//! Destructor.
		~SURFACE_TO_GEOMETRIC ();

		//! Read all parameters from job file.
		ERRVALUE JobRead (MIJOB &job);

		//! Write parameters to job file.
		ERRVALUE JobWrite (MIJOB &job);

		//! Run the operation.
		ERRVALUE Run ();

		//! Set elevation scale/offset.
		//! If not specified, will be determined based on IMAGESCALE data unit settings.
		void SetElevationScaleOffset (
			double ElevationScale,				//!< Scale factor to convert raster values to meters
			double ElevationOffset = 0			//!< Offset
			) { m_ElevationScale = ElevationScale; m_ElevationOffset = ElevationOffset; }

		//! Set mesh sampling.
		//! Will compute mesh size based on sampling intervals.
		//! By default if neither mesh size nor sampling is specified all cells will be used.
		void SetMeshSampling (
			INT32 ColumnSample,						//!< Column sampling
			INT32 RowSample							//!< Row sampling
			);

		//! Set mesh size.
		//! Will compute sampling based on size relative to surface raster dimensions.
		//! By default if neither mesh size nor sampling is specified all cells will be used.
		void SetMeshSize (
			INT32 MeshColumns,						//!< Mesh columns, cannot exceed surface raster columns
			INT32 MeshRows								//!< Mesh rows, cannot exceed surface raster rows
			);

		//! Set output geometric object location and point types to generate.
		ERRVALUE SetObjGeometric (
			const RVC::OBJITEM &ObjItem,			//!< Geometric object info
			POINTTYPE PointTypes,					//!< Point types
			ELEMTYPE ElemType = ELEMTYPE_Point	//!< ELEMTYPE_Point for points, ELEMTYPE_Line for mesh
			);

		//! Set object for surface.
		ERRVALUE SetObjSurface (
			const RVC::OBJITEM &ObjItemSurface	//!< Surface raster object location
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class ENGINE;

		RVC::OBJITEM m_ObjItemSurface;
		RVC::OBJITEM m_ObjItemGeometric;
		double m_ElevationScale;
		double m_ElevationOffset;
		INT32 m_MeshCols;
		INT32 m_MeshRows;
		INT32 m_ColSample;
		INT32 m_RowSample;
		POINTTYPE m_PointTypes;
		ELEMTYPE m_ElemType;

		static void SML_ComputeRegularMesh (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);
		static void SML_ComputeRegularPoints (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);
		static void SML_ComputeVariablePoints (int numargs, SML::ARG *args, UINT8 *argtypes, SML::CONTEXT *context);

		ERRVALUE CheckSettings ();
		ERRVALUE RunSML (ELEMTYPE ElemType, SML::CONTEXT *context, SML::ARG *args, int numargs);

		SURFACE_TO_GEOMETRIC (const SURFACE_TO_GEOMETRIC&);
		SURFACE_TO_GEOMETRIC& operator= (const SURFACE_TO_GEOMETRIC&);
		#endif
	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(SURFACE_TO_GEOMETRIC::POINTTYPE)
#endif

//=====================================================================================================================

}	// End namespace OP
}	// End namespace RVC

#endif	// INC_RVC_OPSURFACETOGEOMETRICPOINTS_H
