/**
 * \file rvc/uifiltergetobjectimage.h
 * \brief RVC::UI::FILTER_GETOBJECT_IMAGE definitions
 *
 * \if NODOC
 * $Id: uifiltergetobjectimage.h_v 1.7 2015/08/17 17:50:28 mju Exp $
 *
 * $Log: uifiltergetobjectimage.h_v $
 * Revision 1.7  2015/08/17 17:50:28  mju
 * Add getNumColumns/Rows.
 *
 * Revision 1.6  2013/09/30 14:35:09  mju
 * Add setDimensions.
 *
 * Revision 1.5  2013/07/30 13:52:02  mju
 * Add getCellTypesAllow/Deny.
 *
 * Revision 1.4  2013/05/08 15:47:39  mju
 * Allow specify cell types to allow in ctors.
 *
 * Revision 1.3  2013/05/08 12:44:21  mju
 * Use geometric filter as base class.
 *
 * Revision 1.2  2013/05/07 19:59:45  mju
 * Add baseclass typedef.
 *
 * Revision 1.1  2013/05/07 15:35:05  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFILTERGETOBJECTIMAGE_H
#define	INC_RVC_UIFILTERGETOBJECTIMAGE_H

#ifndef	INC_RVC_UIFILTERGETOBJECTGEOSPATIAL_H
#include <rvc/uifiltergetobjectgeospatial.h>
#endif

#ifndef	INC_RVC_IMAGE_H
#include <rvc/image.h>
#endif


namespace RVC {
namespace UI {

//=====================================================================================================================

//! Filter for selecting an RVC "image" object.
class FILTER_GETOBJECT_IMAGE : public FILTER_GETOBJECT_GEOSPATIAL {
	public:

		//! Default constructor, allows all "image" object types (Raster, Hyperspectral).
		explicit FILTER_GETOBJECT_IMAGE (
			RVC::IMAGE::CELLTYPE CellTypesAllow = RVC::IMAGE::CELLTYPE_All		//!< Cell types to allow
			);

		//! Construct with specific set of allowed object types.
		//! If any non-image types are allowed they will always be included by this class.
		explicit FILTER_GETOBJECT_IMAGE (
			const RVC::OBJTYPESET& ObjTypeSetSelectable,		//!< Set of allowed object types
			RVC::IMAGE::CELLTYPE CellTypesAllow = RVC::IMAGE::CELLTYPE_All		//!< Cell types to allow
			);

		//! Destructor.
		virtual ~FILTER_GETOBJECT_IMAGE ();

		//! Get cell types to allow.
		RVC::IMAGE::CELLTYPE GetCellTypesAllow () const
			{ return (m_CellTypesAllow); }

		//! Get cell types to deny.
		RVC::IMAGE::CELLTYPE GetCellTypesDeny () const
			{ return (m_CellTypesDeny); }

		//! Get number of columns required.
		//! @return Number required or 0 if not specific.
		UINT32 GetNumColumns () const
			{ return (m_NumColumns); }

		//! Get number of rows required.
		//! @return Number required or 0 if not specific.
		UINT32 GetNumRows () const
			{ return (m_NumRows); }

		//! Set cell types to allow.
		void SetCellTypesAllow (
			RVC::IMAGE::CELLTYPE CellTypesAllow
			) { m_CellTypesAllow = CellTypesAllow; }

		//! Set cell types to deny.
		//! Default is to not deny any cell types.
		void SetCellTypesDeny (
			RVC::IMAGE::CELLTYPE CellTypesDeny
			) { m_CellTypesAllow -= CellTypesDeny; }

		//! Set required dimensions.
		//! Default is to allow any image dimensions.
		void SetDimensions (
			UINT32 NumColumns,
			UINT32 NumRows
			) { m_NumColumns = NumColumns; m_NumRows = NumRows; }

		//! Set whether all images must have the same cell type.
		void SetNeedSameCellType (
			bool bNeedSameCellType = true
			) { m_bNeedSameCellType = bNeedSameCellType; };

		//! Set whether all images must have the same dimensions.
		void SetNeedSameDimensions (
			bool bNeedSameDimensions = true
			) { m_bNeedSameDimensions = bNeedSameDimensions; };

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		typedef FILTER_GETOBJECT_GEOSPATIAL BASECLASS;
		typedef FILTER_GETOBJECT_IMAGE THISCLASS;

		UINT32 m_NumColumns;
		UINT32 m_NumRows;
		RVC::IMAGE::CELLTYPE m_CellTypesAllow;
		RVC::IMAGE::CELLTYPE m_CellTypesDeny;
		bool m_bNeedSameCellType;
		bool m_bNeedSameDimensions;

		int IsImageSelectable (const RVC::OBJITEM& objitem, MISTRING& msg, int position) const;
		int IsImageListValid (const RVC::OBJITEMLIST& objlist, MISTRING& msg) const;
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFILTERGETOBJECTIMAGE_H
