/**
 * \file rvc/uiformeditstyle.h
 * \brief RVC::UI::FORM_EDIT_STYLE definitions
 *
 * \if NODOC
 * $Id: uiformeditstyle.h_v 1.5 2015/03/20 22:44:47 bcolombe Exp $
 *
 * $Log: uiformeditstyle.h_v $
 * Revision 1.5  2015/03/20 22:44:47  bcolombe
 * Allow setactivestylenum to force single selection
 *
 * Revision 1.4  2014/01/16 20:25:50  mju
 * Fix dox.
 *
 * Revision 1.3  2013/12/05 16:29:19  mju
 * Add setReadOnly.
 *
 * Revision 1.2  2013/11/15 22:05:32  mju
 * Add delegates for notifying of style deletion.
 *
 * Revision 1.1  2013/11/15 17:24:05  mju
 * Initial revision
 * \endif
**/

#ifndef	INC_RVC_UIFORMEDITSTYLE_H
#define	INC_RVC_UIFORMEDITSTYLE_H

#ifndef	INC_MGUI_FORM_H
#include <mgui/form.h>
#endif

#ifndef	INC_RVC_UISTYLE_H
#include <rvc/uistyle.h>
#endif

#ifndef	INC_RVC_OBJECT_H
#include <rvc/object.h>
#endif

#ifndef GENERATING_DOXYGEN_OUTPUT
// Forward declarations.
namespace RVC {
	class STYLE;
	}
#endif

namespace RVC {
namespace UI {

//=====================================================================================================================

//! User Interface form for editing style definition.
class FORM_EDIT_STYLE : public MGUI::FORM_COMPOSITE {
	public:

		//! Delegate for when style deleted.
		//! Parameters are index of style that was deleted and index of style it was replaced with.
		//! When styles are deleted, the last style is moved into the place of the deleted one and the number
		//! of styles is reduced by one.
		typedef fastdelegate::FastDelegate<void(INT32,INT32)> DELEGATE_STYLE_DELETED;

		//! Constructor.
		FORM_EDIT_STYLE ();

		//! Destructor.
		virtual ~FORM_EDIT_STYLE ();

		//! Commit changes made by user to file.
		ERRVALUE CommitChanges ();

		//! Create the form.
		ERRVALUE Create (
			MGUI::LAYOUT_PANE_BASE& ParentPane,
			RVC::STYLETYPE StyleType,
			EDITSTYLEFLAGS FormStyle = EDITSTYLEFLAG_None
			);

		//! Get active style number.
		//! @return Active style number, -1 for "default" style, < -1 if error.
		INT32 GetActiveStyleNum () const;

		//! Get active style number for specified type.
		//! @return Active style number, -1 for "default" style, < -1 if error.
		INT32 GetActiveStyleNum (
			RVC::STYLETYPE StyleType
			) const;

		//! Get active style type.
		RVC::STYLETYPE GetActiveStyleType () const;

		//! Get current scale.
		double GetCurrentScale () const;

		//! Get default POINTSTYLE.
		void GetDefaultStyle (
			POINTSTYLE& PointStyle
			) const;

		//! Get default LINESTYLE.
		void GetDefaultStyle (
			LINESTYLE& LineStyle
			) const;

		//! Get default POLYSTYLE.
		void GetDefaultStyle (
			POLYSTYLE& PolyStyle
			) const;

		//! Get default TEXTSTYLE.
		void GetDefaultStyle (
			TEXTSTYLE& TextStyle
			) const;

		//! Get default MULTISTYLE.
		void GetDefaultStyle (
			MULTISTYLE& MultiStyle
			) const;

		//! Get current style object item.
		void GetStyleObjItem (
			RVC::OBJITEM& ObjItem
			) const;

		//! Determine if RVC style object has been modified by user edits.
		bool IsObjectModified () const;

		//! Reset 'modified' status.
		void ResetModified ();

		//! Set active style number for specified style type.
		//! This does not change the style type being "edited", use SetStyleType() to do that.
		ERRVALUE SetActiveStyleNum (
			RVC::STYLETYPE StyleType,
			INT32 StyleNum,
			bool bSingleSelect = false	//!< Force to only style selected in list
			);

		//! Set current 'map scale' for style samples and 'Current Scale' button.
		void SetCurrentMapScale (
			double CurMapScale
			);

		//! Set default LINE style.
		void SetDefaultStyle (
			const LINESTYLE& LineStyle
			);

		//! Set default POINT style.
		void SetDefaultStyle (
			const POINTSTYLE& PointStyle
			);

		//! Set default POLYGON style.
		void SetDefaultStyle (
			const POLYSTYLE& PolygonStyle
			);

		//! Set default TEXT style.
		void SetDefaultStyle (
			const TEXTSTYLE& TextStyle
			);

		//! Set default MULTISTYLE style.
		void SetDefaultStyle (
			const MULTISTYLE& MultiStyle
			);

		//! Set delegate for obtaining current map scale.
		void SetDelegateGetCurrentScale (
			DELEGATE_DOUBLE_NOPARMS delegate
			);

		//! Set delegate to call when style edited.
		void SetDelegateNotifyModified (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set delegate to call when style deleted.
		void SetDelegateOnStyleDeleted (
			DELEGATE_STYLE_DELETED delegate
			);

		//! Set delegate to call when all styles (of current type) deleted.
		void SetDelegateOnStyleDeleteAll (
			DELEGATE_VOID_NOPARMS delegate
			);

		//! Set hardcopy pixel spacing in "pixels per inch".
		//! This value is used when user chooses units to view style elements in.
		void SetHardcopyDPI (
			double HardcopyDPI
			);

		// Set whether read-only or not.
		void SetReadOnly (
			bool bReadOnly = true
			);

		//! Set style object to edit or select styles from.
		ERRVALUE SetStyleObject (
			const RVC::OBJITEM& ObjItem,
			RVC::OBJECT::OPENMODE OpenMode = RVC::OBJECT::OPENMODE_Read
			);

		//! Set type of style being edited.
		void SetStyleType (
			RVC::STYLETYPE StyleType
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class PRIV;
		PRIV *m_pPriv;

		FORM_EDIT_STYLE (const FORM_EDIT_STYLE&);
		FORM_EDIT_STYLE& operator= (const FORM_EDIT_STYLE&);
		#endif
	};

//=====================================================================================================================

}	// End namespace UI
}	// End namespace RVC

#endif	// INC_RVC_UIFORMEDITSTYLE_H
