/**
 * \file spatmod/imagecache.h
 * \brief SPATMOD::IMAGE::CACHE definitions
 *
 * \if NODOC
 * $Id$
 *
 * $Log$
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGECACHE_H
#define	INC_SPATMOD_IMAGECACHE_H

#ifndef	INC_SPATMOD_DATATYPE_H
#include <spatmod/datatype.h>
#endif

namespace SPATMOD {
namespace IMAGE {

class STAGE;

//=====================================================================================================================

//! Base for image cache.
class CACHE {
	public:

		//! Attach image stage to cache.
		ERRVALUE AttachStage (
			STAGE *pStage,
			bool TakeOwnership
			);

		//! Determine if specified cell is valid
		bool IsValidCell (
			INT32 row,					//!< Row relative to total image
			INT32 col					//!< Column relative to total image
			);

		//! Set active rectangle to cache.
		//! If rectangle is locked, all cells outside will be considered to be invalid.  This can be useful
		//! if the process can efficiently determine an area currently being processed.
		ERRVALUE SetActiveRect (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			bool lock					//!< Lock rectangle such that all cells outside are considered invalid
			);

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CACHE (
			DATATYPE DataType,
			bool AllowMultiSamples
			);

		~CACHE ();

		#endif

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		STAGE *m_pStage;
		DATATYPE m_DataType;
		LRECT2D m_RectCache;
		bool m_bIsStageOwner;
		bool m_AllowMultiSamples;
		bool m_bRectLocked;

		CACHE (const CACHE&);
		CACHE& operator= (const CACHE&);
		#endif
	};

//=====================================================================================================================

//! Base for single-sample image cache.
class CACHE_ONESAMPLE : public CACHE {
	public:

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CACHE_ONESAMPLE (
			DATATYPE DataType
			):
			CACHE(DataType,false)
			{ }

		//! Get data value at specified cell.
		ERRVALUE BaseGetDataForCell (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			void *pValue,				//!< Value returned
			bool &validity				//!< Validity returned
			);

		//! Get data values for specified rectangle, returning result in 1D arrays.
		ERRVALUE BaseGetDataForRect1D (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			void *pValues,				//!< Values returned, grouped by row
			bool *validity				//!< Validity returned, grouped by row
			);

		//! Get data values for specified rectangle, returning result in ND arrays.
		ERRVALUE BaseGetDataForRectND (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			void **ppValues,			//!< Values returned, indexed by [rectrow][rectcol]
			bool **validity			//!< Validity returned, indexed by [rectrow][rectcol]
			);

		//! Get data value for specified cell, returning validity
		//! @return true if value is valid, false if not
		bool BaseGetValue (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			void *pValue				//!< Value returned
			);

		#endif
	};

//=====================================================================================================================

//! Cache template for single-sample images.
template <typename _ValueType> class CACHE_ONESAMPLE_T : public CACHE_ONESAMPLE {
	public:

		//! Constructor.
		CACHE_ONESAMPLE_T ()
			: CACHE_ONESAMPLE(DataTypeOf(_ValueType()))
			{ }

		//! Get data value at specified cell.
		ERRVALUE GetDataForCell (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			_ValueType &value,		//!< Value returned
			bool &validity				//!< Validity returned
			) { return (BaseGetDataForCell(row,col,&value,validity)); }

		//! Get data values for specified rectangle, returning result in 1D arrays.
		ERRVALUE GetDataForRect1D (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			_ValueType *values,		//!< Values returned, grouped by row
			bool *validity				//!< Validity returned, grouped by row
			) { return (BaseGetDataForRect1D(rect,values,validity)); }

		//! Get data values for specified rectangle, returning result in ND arrays.
		ERRVALUE GetDataForRectND (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			_ValueType **values,		//!< Values returned, indexed by [rectrow][rectcol]
			bool **validity			//!< Validity returned, indexed by [rectrow][rectcol]
			) { return (BaseGetDataForRectND(rect,values,validity)); }

		//! Get data value for specified cell, returning validity.
		//! @return true if value is valid, false if not
		bool GetValue (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			_ValueType &value			//!< Value returned
			) { return (BaseGetValue(row,col,&value)); }

	};

//=====================================================================================================================

//! Base for single-sample image cache.
class CACHE_MULTISAMPLE : public CACHE {
	public:

	protected:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		CACHE_MULTISAMPLE (
			DATATYPE DataType
			):
			CACHE(DataType,true)
			{ }

		//! Get data values at specified cell.
		ERRVALUE BaseGetDataForCell (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			void *pValues,				//!< Values returned for each image sample
			bool &validity				//!< Validity returned
			);

		//! Get data values for specified rectangle, returning result in 1D arrays.
		ERRVALUE BaseGetDataForRect1D (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			void *values,				//!< Values returned packed into 1D array, grouped by sample, then row, 
			bool *validity				//!< Validity returned packed into 1D array, grouped by row
			);

		//! Get data values for specified rectangle, returning result in ND arrays.
		ERRVALUE BaseGetDataForRectND (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			void ***values,			//!< Values returned, indexed by [sample][rectrow][rectcol]
			bool **validity			//!< Validity returned, indexed by [rectrow][rectcol]
			);

		#endif

	};

//=====================================================================================================================

//! Cache template for multi-sample image.
template <typename _ValueType> class CACHE_MULTISAMPLE_T : public CACHE_MULTISAMPLE {
	public:

		//! Constructor.
		CACHE_MULTISAMPLE_T ()
			: CACHE_MULTISAMPLE(DataTypeOf(_ValueType()))
			{ }

		//! Get data values at specified cell.
		ERRVALUE GetDataForCell (
			INT32 row,					//!< Row relative to total image
			INT32 col,					//!< Column relative to total image
			_ValueType *pValues,		//!< Values returned for each image sample
			bool &validity				//!< Validity returned
			) { return (BaseGetDataForCell(row,col,pValues,validity)); }

		//! Get data values for specified rectangle, returning result in 1D arrays.
		ERRVALUE GetDataForRect1D (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			_ValueType *values,		//!< Values returned packed into 1D array, grouped by sample, then row, 
			bool *validity				//!< Validity returned packed into 1D array, grouped by row
			) { return (BaseGetDataForRect1D(rect,values,validity)); }

		//! Get data values for specified rectangle, returning result in ND arrays.
		ERRVALUE GetDataForRectND (
			const LRECT2D& rect,		//!< Rectangle relative to total image
			_ValueType ***values,	//!< Values returned, indexed by [sample][rectrow][rectcol]
			bool **validity			//!< Validity returned, indexed by [rectrow][rectcol]
			) { return (BaseGetDataForRectND(rect,values,validity)); }

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGECACHE_H
