/**
 * \file spatmod/imagefilterdarkcurrent.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_DARKCURRENT definition.
 *
 * \if NODOC
 * $Id: imagefilterdarkcurrent.h_v 1.1 2015/08/19 15:10:15 mju Exp $
 *
 * $Log: imagefilterdarkcurrent.h_v $
 * Revision 1.1  2015/08/19 15:10:15  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERDARKCURRENT_H
#define	INC_SPATMOD_IMAGEFILTERDARKCURRENT_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform dark current subtraction.
//! Dark Current levels may be specified for the 4 corners of the image. A bilinear function will be used to model
//! the levels across the image. The resulting image will have the same pixel properties as the input. Opacity samples
//! will be left unchanged unless levels are specifically set.
class FILTER_DARKCURRENT : public FILTER {
	public:

		//! Constructor.
		//! If dark current levels are specified, they will apply to all (non-opacity) samples.
		FILTER_DARKCURRENT (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool bTakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const double *levels = 0				//!< 4 element array of corner dark current levels (top left, top right, bottom left, bottom right)
			);

		//! Destructor.
		virtual ~FILTER_DARKCURRENT ();

		//! Set dark current levels to subtract.
		void SetLevels (
			const double *levels,					//!< 4 element array of corner dark current levels (top left, top right, bottom left, bottom right)
			UINT16 SampleNum = UINT16_MAX			//!< Sample number, UINT16_MAX for all non-opacity samples
			);

		//! Set dark current levels to subtract.
		void SetLevels (
			double TopLeft,							//! Top-Left level to subtract
			double TopRight,							//! Top-Right level to subtract
			double BottomLeft,						//! Bottom-Left level to subtract
			double BottomRight,						//! Bottom-Right level to subtract
			UINT16 SampleNum = UINT16_MAX			//!< Sample number, UINT16_MAX for all non-opacity samples
			);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		struct LEVELS {
			double m_TopLeft;
			double m_TopRight;
			double m_BottomLeft;
			double m_BottomRight;
			bool m_bSet;

			void Set (
				double TopLeft,
				double TopRight,
				double BottomLeft,
				double BottomRight
				) { m_TopLeft = TopLeft; m_TopRight = TopRight; m_BottomLeft = BottomLeft; m_BottomRight = BottomRight; m_bSet = true; }
			};

		DOUBLE_ARRAY<LEVELS> m_Levels;

		// STAGE overrides.
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual ERRVALUE v_Initialize ();

		// Unimplemented
		FILTER_DARKCURRENT (const FILTER_DARKCURRENT&);
		FILTER_DARKCURRENT& operator= (const FILTER_DARKCURRENT&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERDARKCURRENT_H

