/**
 * \file spatmod/imagefilterfocaltopographic.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_FOCALTOPOGRAPHIC definition.
 *
 * \if NODOC
 * $Id: imagefilterfocaltopographic.h_v 1.13 2017/03/31 13:49:33 mju Exp $
 *
 * $Log: imagefilterfocaltopographic.h_v $
 * Revision 1.13  2017/03/31 13:49:33  mju
 * Add m_bcalcPlanValidity.
 *
 * Revision 1.12  2016/11/22 19:54:06  mju
 * Add members to avoid recalculation.
 *
 * Revision 1.11  2015/11/20 23:19:15  bcolombe
 * Add IncidenceAngle method that doesn't apply scaling
 *
 * Revision 1.9  2007/10/02 14:47:55  mju
 * Add control for whether to output separate validity for plan curvature.
 *
 * Revision 1.8  2007/09/25 17:57:30  mju
 * Change computePlan to set validity also.
 *
 * Revision 1.7  2007/09/10 14:41:37  mju
 * Specify values for enums when saved as settings.
 *
 * Revision 1.2  2006/11/30 17:33:01  mju
 * Init surface fit member of settings.
 * Document units for sun direction/elevation.
 *
 * Revision 1.1  2006/11/14 23:43:38  drb
 * Initial revision
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERFOCALTOPOGRAPHIC_H
#define	INC_SPATMOD_IMAGEFILTERFOCALTOPOGRAPHIC_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Purpose of this filter is to compute topographic properties from an input DEM raster
class FILTER_FOCALTOPOGRAPHIC : public FILTER {
	public:

		//! Surface fitting method to use
		enum SURFACE_FITTING_METHOD {
			SURFACE_FITTING_METHOD_FourNearest =					0,
			SURFACE_FITTING_METHOD_QLSF =								1,	// Quadratic surface, least-squares fit
			SURFACE_FITTING_METHOD_QLSFOverDistance =				2,
			SURFACE_FITTING_METHOD_QLSFOverDistanceSquared =	3,
			SURFACE_FITTING_METHOD_QLSFMatchCentral =				4
			// Don't change numbers as saved as settings.
			};

		//! Units to use for computing the slope
		enum SLOPE_UNITS {
			SLOPE_UNITS_Percent,
			SLOPE_UNITS_Degrees
			};

		//! Method to use for creating shading raster
		enum SHADING_METHOD {
			SHADING_METHOD_HighContrast =		0,
			SHADING_METHOD_Display =			1,
			SHADING_METHOD_IncidenceAngle =		2
			// Don't change numbers as saved as settings.
			};

		//! Units to use for computing cuvature rasters
		enum CURVATURE_UNITS {
			CURVATURE_UNITS_RadiansPerMeter,
			CURVATURE_UNITS_RadiansPer100Meter
			};

		//! Settings class for the FILTER_FOCALTOPOGRAPHIC class
		class SETTINGS {
			public:

				//! Default constructor.
				SETTINGS ( ) :
					m_SurfaceFitMethod(SURFACE_FITTING_METHOD_FourNearest),
					m_DoSlope(false),
					m_SlopeDataType(SPATMOD::DATATYPE_FLOAT32),
					m_SlopeUnits(FILTER_FOCALTOPOGRAPHIC::SLOPE_UNITS_Degrees),
					m_DoAspect(false),
					m_AspectDataType(SPATMOD::DATATYPE_SINT16),
					m_DoShading(false),
					m_ShadingDataType(SPATMOD::DATATYPE_UINT8),
					m_ShadingMethod(FILTER_FOCALTOPOGRAPHIC::SHADING_METHOD_Display),
					m_DoProfile(false),
					m_DoPlan(false),
					m_CurvatureDataType(SPATMOD::DATATYPE_FLOAT32),
					m_CurvatureUnits(FILTER_FOCALTOPOGRAPHIC::CURVATURE_UNITS_RadiansPerMeter),
					m_XCellSize(0.0),
					m_YCellSize(0.0),
					m_ElevationScale(1.0),
					m_ElevationAngleOfSun(45.0*DEGTORAD),
					m_DirectionOfSun(50.0*DEGTORAD)
					{ }

				~SETTINGS () { }

				//! Set the surface fitting method to use
				void SetSurfaceFittingMethod ( SURFACE_FITTING_METHOD method ) { m_SurfaceFitMethod = method; }

				//! Set whether to compute a slope raster and what parameters to use
				void SetSlopeParms ( 
					bool DoSlope, 
					SPATMOD::DATATYPE type = SPATMOD::DATATYPE_SINT16, 
					SLOPE_UNITS units = FILTER_FOCALTOPOGRAPHIC::SLOPE_UNITS_Degrees
					) {	
					SetDoSlope(DoSlope);
					SetSlopeDataType(type);
					SetSlopeUnits(units);
					}
				
				//! Set whether to compute the slope raster
				void SetDoSlope ( bool DoSlope ) { m_DoSlope = DoSlope; };

				//! Set DataType of the raster
				void SetSlopeDataType ( SPATMOD::DATATYPE type ) { m_SlopeDataType = type; };

				//! Set units for the slope raster
				void SetSlopeUnits ( SLOPE_UNITS units ) { m_SlopeUnits = units; };

				//! Set whether to compute a aspect raster and what parameters to use
				void SetAspectParms (
					bool DoAspect, 
					SPATMOD::DATATYPE type = SPATMOD::DATATYPE_FLOAT32
					) {	
					SetDoAspect(DoAspect);
					SetAspectDataType(type);
					}

				//! Set whether to compute the aspect raster
				void SetDoAspect ( bool DoAspect ) { m_DoAspect = DoAspect; };

				//! Set DataType of the raster
				void SetAspectDataType ( SPATMOD::DATATYPE type ) { m_AspectDataType = type; };

				//! Set whether to compute a shading raster and what parameters to use
				void SetShadingParms ( 
					bool DoShading, 
					SPATMOD::DATATYPE type = SPATMOD::DATATYPE_UINT8, 
					SHADING_METHOD method = FILTER_FOCALTOPOGRAPHIC::SHADING_METHOD_Display
					) {	
					SetDoShading(DoShading);
					SetShadingDataType(type);
					SetShadingMethod(method);
					}

				//! Set whether to compute the shading raster
				void SetDoShading ( bool DoShading ) { m_DoShading = DoShading; };

				//! Set DataType of the raster
				void SetShadingDataType ( SPATMOD::DATATYPE type ) { m_ShadingDataType = type; };

				void SetShadingMethod ( SHADING_METHOD method ) { m_ShadingMethod = method; };

				//! Set whether to compute the profile or plan curvature rasters and what parameters to use
				void SetCurvatureParms ( 
					bool DoProfile, 
					bool DoPlan, 
					SPATMOD::DATATYPE type = SPATMOD::DATATYPE_SINT16, 
					CURVATURE_UNITS units = FILTER_FOCALTOPOGRAPHIC::CURVATURE_UNITS_RadiansPerMeter
					) {	
					SetDoProfile(DoProfile);
					SetDoPlan(DoPlan);
					SetCurvatureDataType(type);
					SetCurvatureUnits(units);
					}

				//! Set whether to compute the profile curvature raster
				void SetDoProfile ( bool DoProfile ) { m_DoProfile = DoProfile; };

				//! Set whether to compute the plan curvature raster
				void SetDoPlan ( bool DoPlan ) { m_DoPlan = DoPlan; };

				//! Set DataType of the rasters
				void SetCurvatureDataType ( SPATMOD::DATATYPE type	) { m_CurvatureDataType = type; };

				//! Set units for the curvature rasters
				void SetCurvatureUnits ( CURVATURE_UNITS units ) { m_CurvatureUnits = units; };

				//! Set the X cell size to use for the input raster
				void SetXCellSize ( double scale ) { m_XCellSize = scale; };

				//! Get the X cell size set to use for the input raster
				double GetXCellSize ( ) { return (m_XCellSize); };

				//! Set the Y cell size to use for the input raster
				void SetYCellSize ( double scale ) { m_YCellSize = scale; };

				//! Get the T cell size set to use for the input raster
				double GetYCellSize ( ) { return (m_YCellSize); };

				//! Set the elevation scale to use for the input raster
				void SetElevationScale ( double scale ) { m_ElevationScale = scale; };

				//! Set the Elevation Angle of the Sun (in degrees)
				void SetElevationAngleOfSun (
					double angle							//!< Elevation angle (degrees)
					) { m_ElevationAngleOfSun = angle * DEGTORAD; };

				//! Set the Direction of the Sun (in degrees), where north is 0 going clockwise (azimuth)
				void SetDirectionOfSun (
					double direction						//!< Azimuth angle (degrees)
					) { m_DirectionOfSun = direction * DEGTORAD; };

			private:
				#ifndef GENERATING_DOXYGEN_OUTPUT
				SURFACE_FITTING_METHOD m_SurfaceFitMethod;

				bool m_DoSlope;
				SPATMOD::DATATYPE m_SlopeDataType;
				SLOPE_UNITS m_SlopeUnits;

				bool m_DoAspect;
				SPATMOD::DATATYPE m_AspectDataType;
	
				bool m_DoShading;
				SPATMOD::DATATYPE m_ShadingDataType;
				SHADING_METHOD m_ShadingMethod;

				bool m_DoProfile;
				bool m_DoPlan;
				SPATMOD::DATATYPE m_CurvatureDataType;
				CURVATURE_UNITS m_CurvatureUnits;
	
				double m_XCellSize;
				double m_YCellSize;
				double m_ElevationScale;
				double m_ElevationAngleOfSun;
				double m_DirectionOfSun;
	
				friend class FILTER_FOCALTOPOGRAPHIC;
				#endif // GENERATING_DOXYGEN_OUTPUT
			};

		//! Constructor
		FILTER_FOCALTOPOGRAPHIC (
			STAGE *pPreviousStage,
			bool TakeOwnership, 
			const SETTINGS &settings
			);

		//! Destructor.
		virtual ~FILTER_FOCALTOPOGRAPHIC ();

		//! Set to output separate validity band for plan curvature.
		void SetSeparatePlanValidity (
			bool SeparatePlanValidity
			) { m_SeparatePlanValidity = SeparatePlanValidity; }

	protected:

		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR& iterator);
		virtual ERRVALUE v_PullSubSection (const SECTION& section, BUFFER& target);

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT

		SETTINGS m_Settings;
		SIMPLE_ARRAY<UINT8> m_gradgray;
		SIMPLE_ARRAY<UINT8*> m_pValidInput;
		SAMPLEITERATOR *m_itInput;
		DPOINT3D m_SunVector;
		double m_xgradscale;
		double m_ygradscale;
		double m_CellSizeX2;
		double m_CellSizeY2;
		double m_z[10];
		double m_a;
		double m_b;
		double m_c;
		double m_d;
		double m_e;
		double m_d2;
		double m_e2;
		double m_ProfileDenom;
		double m_PlanDenom;
		UINT16 m_NumOutputs;
		UINT16 m_IdxPlanValidity;
		bool m_SeparatePlanValidity;
		bool m_bNeedDiagCells;
		bool m_bCalcPlanValidity;

		ERRVALUE ComputeCoefficients ();
		ERRVALUE ComputeDisplayShadingLookupTable ();
		void ComputeSunVector ();

		bool ReadKernel ();
		bool IsKernalLevel ();

		void ComputeSlope (SAMPLEITERATOR &Tgt);
		void ComputeAspect (SAMPLEITERATOR &Tgt);
		void ComputeShading (SAMPLEITERATOR &Tgt);
		void ComputeCurvatureProfile (SAMPLEITERATOR &Tgt);
		void ComputeCurvaturePlan (SAMPLEITERATOR &TgtResult,	UINT8 *pTgtValidity = 0);

		// Unimplemented
		FILTER_FOCALTOPOGRAPHIC (const FILTER_FOCALTOPOGRAPHIC&);
		FILTER_FOCALTOPOGRAPHIC& operator= (const FILTER_FOCALTOPOGRAPHIC&);

		#endif // GENERATING_DOXYGEN_OUTPUT
	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERFOCALTOPOGRAPHIC_H

