/**
 * \file spatmod/imagefiltermosaiccamera.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_MOSAIC_CAMERA definition.
 *
 * \if NODOC
 * $Id: imagefiltermosaiccamera.h_v 1.1 2016/01/21 22:02:03 mju Exp $
 *
 * $Log: imagefiltermosaiccamera.h_v $
 * Revision 1.1  2016/01/21 22:02:03  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERMOSAICCAMERA_H
#define	INC_SPATMOD_IMAGEFILTERMOSAICCAMERA_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEBUFFER_H
#include <spatmod/imagebuffer.h>
#endif

#ifndef	INC_SPATMOD_IMAGEMRUSTAGELIST_H
#include <spatmod/imagemrustagelist.h>
#endif

#ifndef	INC_SPATMOD_IMAGEMOSAICENUMS_H
#include <spatmod/imagemosaicenums.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

#ifndef	INC_MI32P_CAMERA3D_H
#include <mi32p/camera3d.h>
#endif

#ifndef	INC_MI32_DOUBLEAR_H
#include <mi32/doublear.h>
#endif

#include <vector>

namespace SPATMOD {
namespace IMAGE {

class SAMPLEITERATOR;
class SAMPLEITERATOR_CONST;

//=====================================================================================================================

//! Filter to mosaic images with camera parameters.
//! The coordinate transformation between resulting image and each input stage is computed using the CAMERA3D values
//! specified with each input image.
//! Note that source images will not be processed "sequentially", and the sections specified
//! in PullBegin for the source will overlap.  It is recommended that a suitable caching filter be placed ahead
//! of this filter if the input requires significant computation or sequential iteration.
class FILTER_MOSAIC_CAMERA : public FILTER {
	public:

		//! Construct with specified dimensions and resulting georeference.
		FILTER_MOSAIC_CAMERA (
			STAGE *pStageTerrain,					//!< Pointer to terrain stage
			bool TakeOwnershipTerrain,				//!< Filter takes ownership of terrain stage and will delete when needed
			STAGE_ARRAY &InputStages,				//!< Input stages
			bool TakeOwnershipInput,				//!< Filter takes ownership of input stages and will delete when needed
			std::vector<CAMERA3D> &cameras,		//!< Camera parameters for each input stage
			const DIMENSIONS &Dimensions,			//!< Resulting image dimensions
			const GEOREFERENCE &Georeference,	//!< Georeference for resulting image
			RESAMPLEMETHOD ResampleMethod,		//!< Resampling method
			MOSAIC_OVERLAP OverlapMethod = MOSAIC_OVERLAP_ClosestToSourceCenter		//!< Overlap control method
			);

		// Destructor.
		virtual ~FILTER_MOSAIC_CAMERA ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		class INPUTLIST;

		static const unsigned int STAGEIDX_Terrain = 0;
		std::vector<CAMERA3D> m_Cameras;
		RESAMPLEMETHOD m_ResampleMethod;
		MOSAIC_OVERLAP m_OverlapMethod;
		double m_Accuracy;
		INT32 m_MaxColumnsPull;
		INT32 m_MaxRowsPull;
		DRECT2D *m_ExtentsInputInOutput;
		SAMPLEITERATOR_CONST *m_itInput;
		SAMPLEITERATOR *m_itOutput;
		double m_ElevMin;
		double m_ElevMax;
		MRUSTAGELIST m_mrulist;
		BUFFER m_BufferTerrain;
		BUFFER m_BufferInput;
		BUFFER m_BufferInputDistance;
		BUFFER m_BufferBestDistance;
		HELPER_RESAMPLE m_Resampler;
		DOUBLE_ARRAY<DPOINT2D> m_SrcCoord;
		DOUBLE_ARRAY<DPOINT2D> m_TgtCoord;
		bool m_bUseDistance;

		ERRVALUE ComputeSrcRect (UINT32 StageNum, const DRECT2D &RectTgt, DRECT2D &RectSrc, double ElevMin, double ElevMax) const;
		ERRVALUE ProcessOverlap (BUFFER &BufferOutput, BUFFER &BufferBestDistance, const BUFFER &BufferInput, const BUFFER &BufferInputDistance) const;
		ERRVALUE PullInputSection (UINT32 StageNum, const SECTION &TgtSection, const BUFFER &BufferTerrain, BUFFER &TgtBufferInput, BUFFER &TgtBufferDistance);
		ERRVALUE SetupTerrainStage ();

		// BASE/STAGE/FILTER overrides.
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_MOSAIC_CAMERA (const FILTER_MOSAIC_CAMERA&);
		FILTER_MOSAIC_CAMERA& operator= (const FILTER_MOSAIC_CAMERA&);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERMOSAICCAMERA_H

