/**
 * \file spatmod/imagefilterpolarlog.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_POLARLOG definition.
 *
 * \if NODOC
 * $Id: imagefilterpolarlog.h_v 1.1 2015/09/08 19:01:19 mju Exp $
 *
 * $Log: imagefilterpolarlog.h_v $
 * Revision 1.1  2015/09/08 19:01:19  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERPOLARLOG_H
#define	INC_SPATMOD_IMAGEFILTERPOLARLOG_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

#ifndef	INC_SPATMOD_IMAGEHELPERRESAMPLE_H
#include <spatmod/imagehelperresample.h>
#endif

#ifndef	INC_MI32_POINT_H
#include <mi32/point.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to transform image to polar-logarithmic form.
//! This filter is primarily used in the spectral domain as part of image matching when determining the rotation and
//! scaling between a pair of images.
class FILTER_POLARLOG : public FILTER {
	public:

		enum FLAGS {
			FLAG_None =						0,
			FLAG_UseHanningWindow =		0x01,		//!< Apply Hanning window to values based on radius
			};

		//! Construct.
		FILTER_POLARLOG (
			STAGE *pPreviousStage,					//!< Pointer to previous stage
			bool TakeOwnership,						//!< Filter takes ownership of previous stage and will delete when needed
			const DPOINT2D &center,					//!< Center coordinate in source image, typically center of cell
			double RadiusMax,							//!< Maximum radius in source image
			double AngleMin,							//!< Minimum angle in radians
			double AngleMax,							//!< Maximum angle in radians
			FLAGS flags = FLAG_None,
			RESAMPLEMETHOD ResampleMethod = RESAMPLEMETHOD_Bilinear,		//!< Resampling method
			UINT32 NumColumns = 256,				//!< Number of target columns, typically a power of 2 for FFT, represents phase angle
			UINT32 NumRows = 256						//!< Number of target rows, typically a power of 2 for FFT, represents logarithmic radius
			);

		// Destructor.
		virtual ~FILTER_POLARLOG ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		HELPER_RESAMPLE m_Resampler;
		DPOINT2D m_Center;
		double m_RadiusMax;
		double m_AngleMin;
		double m_AngleMax;
		double *m_CosTable;
		double *m_SinTable;
		double *m_RadTable;
		DOUBLE_ARRAY<DPOINT2D> m_SrcCoord;
		FLAGS m_Flags;
		RESAMPLEMETHOD m_ResampleMethod;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullBegin (const SECTION_ITERATOR &iterator);
		virtual ERRVALUE v_PullEnd ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_POLARLOG (const FILTER_POLARLOG &);
		FILTER_POLARLOG& operator= (const FILTER_POLARLOG &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERPOLARLOG_H

