/**
 * \file spatmod/imagefilterspomf.h
 * \brief Spatial Modelling (SPATMOD) IMAGE::FILTER_SPOMF definition.
 *
 * \if NODOC
 * $Id: imagefilterspomf.h_v 1.3 2015/09/16 21:35:14 mju Exp $
 *
 * $Log: imagefilterspomf.h_v $
 * Revision 1.3  2015/09/16 21:35:14  mju
 * Change flags so default is full fft.
 *
 * Revision 1.2  2015/09/11 21:03:58  mju
 * Add flag for creating 180 rotated match.
 *
 * Revision 1.1  2015/09/08 19:01:58  mju
 * Initial revision
 *
 * \endif
**/

#ifndef	INC_SPATMOD_IMAGEFILTERSPOMF_H
#define	INC_SPATMOD_IMAGEFILTERSPOMF_H

#ifndef	INC_SPATMOD_IMAGEFILTER_H
#include <spatmod/imagefilter.h>
#endif

namespace SPATMOD {
namespace IMAGE {

//=====================================================================================================================

//! Filter to perform Symmetric Phase-Only Match
//! This filter is primarily used in the spectral domain as part of image matching when estimating the translation,
//! scaling and rotation between a pair of images. The source images should not have null cells as this will affect
//! the Fourier transform results and usually will result in no match being found.
class FILTER_SPOMF : public FILTER {
	public:

		enum FLAGS {
			FLAG_None =			0,
			FLAG_Dual180 =		0x01,		//!< Produce both normal and 180 degree rotated output
			FLAG_HalfFFT =		0x02		//!< Perform half FFT Real-to-Complex conversion (default is full)
			};

		//! Construct.
		//! The pixel properties and dimensions of the two input stages must be the same. Any opacity/validity components
		//! will be ignored. This filter will insert additional stages to perform fourier transforms on the input unless
		//! the input stages contain "complex" components, in which case they are assumed to already be forward fourier
		//! transformed and will be used directly.
		FILTER_SPOMF (
			STAGE *pPreviousStage1,			//!< Pointer to previous stage
			bool TakeOwnership1,				//!< Filter takes ownership of previous stage and will delete when needed
			STAGE *pPreviousStage2,			//!< Pointer to previous stage
			bool TakeOwnership2,				//!< Filter takes ownership of previous stage and will delete when needed
			FLAGS flags = FLAG_None			//!< Flags
			);

		// Destructor.
		virtual ~FILTER_SPOMF ();

	private:
		#ifndef GENERATING_DOXYGEN_OUTPUT
		FLAGS m_flags;

		// BASE/STAGE/FILTER overrides.
		virtual const char* v_GetTypeName () const;
		virtual ERRVALUE v_Initialize ();
		virtual ERRVALUE v_PullSubSection (const SECTION &SubSection, BUFFER &buffer);
		virtual float v_QueryIterator (const SECTION_ITERATOR &iterator) const;

		// Unimplemented
		FILTER_SPOMF (const FILTER_SPOMF &);
		FILTER_SPOMF& operator= (const FILTER_SPOMF &);
		#endif // GENERATING_DOXYGEN_OUTPUT

	};

#ifndef GENERATING_DOXYGEN_OUTPUT
DEFINE_ENUM_OP_BITWISE(FILTER_SPOMF::FLAGS)
#endif

//=====================================================================================================================

}	// End namespace IMAGE
}	// End namespace SPATMOD

#endif	// INC_SPATMOD_IMAGEFILTERSPOMF_H

