namespace SHAPE {

class ELEMENT {
	public:
		enum TYPE {
			TYPE_BlockInsertion,
			TYPE_Point,
			TYPE_Rectangle,
			TYPE_Box,
			TYPE_Polyline,
			TYPE_Polygon,
			TYPE_Circle,
			TYPE_CircularArc,
			TYPE_CircularArcChord,
			TYPE_CircularArcWedge,
			TYPE_Ellipse,
			TYPE_EllipticalArc,
			TYPE_EllipticalArcChord,
			TYPE_EllipticalArcWedge,
			TYPE_Text,
			TYPE_Polynomial,
			TYPE_Mesh2D,
			TYPE_Mesh3D,
			TYPE_
			};
		//! Flags describing element formatting.
		enum FLAGS {
			FLAG_3D =					0x01,		//!< All points are stored as 3D (X,Y,Z)
			FLAG_Multi =				0x02,		//!< "Multi" element, count follows header
			FLAG_MultiSameHdr =		0x04,		//!< Multi element has same header for all repeated instances (circle/ellipse/arc/box only)
			};
		//! Indication of which "extra" values are stored.
		enum EXTRAVALUES {
			EXTRAVALUE_None =					0,
			EXTRAVALUE_Z =					0x01,		//!< Single Z value for entire element, all points stored as 2D
			EXTRAVALUE_ExtrusionZ =		0x02,		//!< Z extrusion value (mutually exclusive with Extrusion3D)
			EXTRAVALUE_Extrusion3D =	0x04,		//!< Full 3D extrusion vector (mutually exclusive with ExtrusionZ)
			EXTRAVALUE_Normal =			0x08,		//!< Normal vector (for orientation in 3D)
			EXTRAVALUE_CurveFit =		0x10,		//!< Curve fit method where supported (polyline/polygon)
			};
		//! Indication of which "style" values are stored.
		enum STYLES {
			STYLE_None =			0,
			STYLE_MapScale =		0x0001,
			STYLE_StyleFlags =	0x0002,
			STYLE_Color =			0x0004,
			STYLE_FillColor =		0x0008,
			STYLE_Symbol =			0x0010,
			STYLE_FillSymbol =	0x0020,
			STYLE_Angle =			0x0040,
			STYLE_Size =			0x0080,
			STYLE_SizeUnits =		0x0100,
			};
	private:
		TYPE m_type;
		FLAGS m_flags;
		UINT32 m_count;
	};

// Element data storage structure:
//		ELEMHDR (6 bytes)
//		[count, UINT32, if FLAG_Multi]
//		[ELEMEXTRA values, based on EXTRAVALUES flags, in order of ascending bit value]
//		[ELEMSTYLE values, based on STYLEVALUES flags, in order of ascending bit value]
//		[ELEM_XXX based on TYPE, repeated "count" times per FLAG values]
//		points or other TYPE-specific data

struct ELEMHDR {
	UINT8 m_type;
	UINT8 m_flags;
	UINT16 m_extravalues;
	UINT16 m_styles;
	};

struct ELEMEXTRA {
	DPOINT3D m_normal;
	DPOINT3D m_extrusion;
	double m_z;
	UINT16 m_CurveFit;
	};

struct ELEMSTYLE {
	double m_MapScale;
	UINT32 m_StyleFlags;
	UINT32 m_color;
	UINT32 m_fillcolor;
	INT32 m_symbol;
	INT32 m_fillsymbol;
	float m_symbolangle;
	float m_size;
	UINT8 m_sizeunits;
	};


struct ELEM_CIRCLE {
	double radius;						//!< Radius of circle 
	};		// Followed by center point(s)

struct ELEM_CIRCULARARC {
	double radius;						//!< Radius of arc 
	double anglestart;				//!< Start angle of arc 
	double angleend;					//!< End angle of arc 
	};		// Followed by center point(s)

struct ELEM_ELLIPSE {
	double sizeX;						//!< Size in direction of X axis
	double sizeY;						//!< Size in direction of Y axis
	double rotate;						//!< Rotation with respect to the world X axis 
	};		// Followed by center point or multiple points if FLAG_MultiSameHdr specified

struct ELEM_ELLIPTICALARC {
	double sizeX;						//!< Size in direction of X axis
	double sizeY;						//!< Size in direction of Y axis
	double rotate;						//!< Rotation with respect to the world X axis
	double anglestart;				//!< Start angle of elliptical arc 
	double angleend;					//!< End angle of elliptical arc 
	};		// Followed by center point or multiple points if FLAG_MultiSameHdr specified

struct ELEM_BOX {
	double rotate;						//!< Rotation in XY plane
	};

struct ELEM_POLYLINE {
	UINT32 NumPoints;					//!< Number of points in polyline 
	};		// Followed by points


struct ELEM_POLYGON {
	UINT32 NumPoints;					//!< Number of points in first polygon. 
	UINT32 NumIslands;				//!< Number of islands in polygon.  0 if none. 
	};										// Followed by array of number of points per island if any islands present
											// Then followed by point data for polygon and each island

struct ELEM_MESH {
	UINT16 NumVertices1;				//!< Number of vertices for first index
	UINT16 NumVertices1;				//!< Number of vertices for second index
	UINT16 NumVertices1;				//!< Number of vertices for third index (for TYPE_Mesh3D)
	UINT16 flags;						//!< 1 = Closed on 1st index, 2, closed on 2nd index, 4 closed on 3rd index
	};										// Followed by the point data in the form of array[NumVertices1][NumVertices2][NumVertices3]




struct ELEM_POLYNOM {		/* Size = 40 */
	DPOINT2D_OLD minval;				//!< Maximum x, y range of polynomial 
	DPOINT2D_OLD maxval;				//!< Minimum x, y range of polynomial 
	UINT32 NumCoeff;					//!< Number of coefficients in polynomial 
	UINT32 space;
	};										/* Followed by the coefficients of the polynomial */

struct ELEM_INSERTION {
	DPOINT3D_OLD InsertPt;			//!< Point where to insert the block 
	INT32 BlockNum;					//!< Index of block to insert 
	double rotate;						//!< Rotation angle 
	double xscale;						//!< X Scale factor				(default 1.0) 
	double yscale;						//!< Y Scale factor				(default 1.0) 
	double zscale;						//!< Z Scale factor				(default 1.0) 
	UINT16 numlins;					//!< Number of lines			(default 1) 
	UINT16 numcols;					//!< Number of columns			(default 1) 
	float linspace;					//!< Spacing between lines 
	float colspace;					//!< Spacing between columns 
	UINT32 space;
	};

struct ELEM_TEXT {			/* Size = 128 */
	UINT32 NumPoints;					//!< Number of points 
	UINT16 NumChars;					//!< Number of charaters in text string 
	UINT16 CharSize;					//!< Number of bytes per character (1 or 2) 
	float height;						//!< Height of text in ELEM_ coordinates 
	double rotate;						//!< Rotation angle of string 
	float shear;						//!< Shear angle 
	float textrotate;					//!< Rotation angle of each character in the string 
	float xscale;						//!< X Scale factor relative to height (default 1.0) 
	INT32 font;							//!< Font number 
	UINT32 FontFlags;					//!< Text style bits 
	UINT8 m_ShadowRelative;			//!< true if shadow angle relative to baseline angle
	UINT8 m_BaselineFlags;			//!< Baseline flags
	UINT8 m_bCartoonBalloonLeaders;
	UINT8 m_Predefined;					//!< Predefined label styles
	
	//!< From TEXTSTYLE
	float m_LineSpace;				//!< Line spacing in ELEM_ units
	float m_StrokeWidth;				//!< Stroke width in % of glyph height (1.0 = 100%)
	float m_EnhanceWidth;			//!< Enhanced width in % of glyph height (1.0 = 100%)
	float m_BoldnessWidth;			//!< Boldness width in % of glyph height (1.0 = 100%) Ignored unless FONTSTYLE_Bold is set
	float m_UnderlineWidth;			//!< Underline with in % of glyph height (1.0 = 100%)
	float m_UnderlineOffset;		//!< Underline Offset from baseline in % (1.0 = 100%)
	float m_ShadowOffset;			//!< Drop shadow offset in % (1.0 = 100%) Ignored unless FONTSTYLE_Shadow is set
	float m_ShadowAngle;				//!< Drop shadow angle in radians
	float m_ClipWidth;				//!< Clip extended area around text in TextAddToClip()
	
	//!< From LABELFRAMEPARMS
	float m_Margin;					//!< Units: % of font ascent 
	float m_TMargin;					//!< Units: % of font ascent
	float m_BMargin;					//!< Units: % of font ascent
	float m_LMargin;					//!< Units: % of font ascent
	float m_RMargin;					//!< Units: % of font ascent
	UINT32 m_FrameShape;
	UINT32 m_LineStyleFlags;
	UINT32 m_PolyStyleFlags;
	float m_LineScale;				//!< Width of line (usually in millimeters) or pattern zoom factor
	float m_PolyBorderScale;		//!< Width of border (usually in millimeters) or pattern zoom factor
	UINT32 m_Arrow;

	UINT32 m_NumLeaderLines;
	UINT8 spare2[4];
//	float m_HatchAngle;				//!< Use ELEM_ELEMOPT::Angle: Rotation angle for hatch patterns (was spare[8])
//	INT32 PatternNum;					//!< Use ELEM_ELEMOPT::LinePatt
//	INT32 FillPatternNum;			//!< Use ELEM_ELEMOPT::FillPatt
//	INT32 BorderPatternNum;			//!< Use ELEM_ELEMOPT::Symbol
	};										/* Followed by the actual points then the number of characters */

#define ELEM_BASELINEFLAG_Straight 	0x00
#define ELEM_BASELINEFLAG_Exact			0x01
#define ELEM_BASELINEFLAG_Spline		0x02
#define ELEM_BASELINEFLAG_ClipUnder	0x04


}
