/**
 * CADTOVEC - Convert a CAD object into a vector object
**/

#define RCSID "$Id: cadtovec.c_v 1.15 2013/08/14 13:54:39 mju Exp $"

#include <mi32/stddefns.h>				// For common definitions and declarations, normally first
#include <mi32/textid.h>				// For TEXTID... values
#include <mi32/helpid.h>				// For HELPID... values
#include <mi32/mistatus.h>				// For MISTATUSCONTEXT
#include <mi32/simplear.h>				// For SIMPLE_ARRAY<>

#include <rvc/combine.h>				// For RVC::COMBINE::ExtractToVector()

#include <mgui/appmain.h>				// For MGUI::APP_MAIN
#include <mgui/topshell.h>				// For MGUI::TOPSHELL
#include <mgui/ctrl.h>					// For MGUI::CTRL_TOGGLEBUTTON
#include <mgui/combobox.h>				// For MGUI::FORM_COMBOBOX
#include <mgui/dlgstatus.h>			// For MGUI::DLG_STATUS
#include <mgui/dlggetobject.h>		// For MGUI::DlgGetObjectSet()

#include <rvc/uiformobjectlist.h>		// For RVC::UI::FORM_OBJECT_LIST
#include <rvc/uifiltergetobjectgeometric.h>	// For RVC::UI::FILTER_GETOBJECT_GEOMETRIC

namespace CADTOVECTOR {

//=====================================================================================================================
//		MAINWND - Begin

class MAINWND : public MGUI::TOPSHELL {
	public:

		MAINWND ();
			
		~MAINWND (
			) { }
			
	private:
		RVC::UI::FORM_OBJECT_LIST m_ObjectList;
		MGUI::CTRL_TOGGLEBUTTON m_Optimize;
		MGUI::FORM_COMBOBOX m_TopoType;
		
		virtual ERRVALUE v_CreateContent ();
		virtual void v_OnCloseRequest ();
		virtual void v_OnRun ();

		ERRVALUE ExecProcess ();
		void OnObjectListChanged ();
		
	};

// Constructor.
MAINWND::MAINWND (
	) : 
	TOPSHELL(HELPID_ObjectToVectorConversion)
	{ }

// Create the MGUI FORM's and CTRL's for the dialog
ERRVALUE MAINWND::v_CreateContent (
	) {
	MGUI::DLG_GETOBJECT_FILTER *pFilter = new RVC::UI::FILTER_GETOBJECT_GEOMETRIC(RVC::OBJTYPE_CAD);
	m_ObjectList.Create(GetMainPane(), TEXTID_Prompt_Objects_ToProcess, pFilter);
	m_ObjectList.SetDelegateOnListChanged(DELEGATE_VOID_NOPARMS(this,&MAINWND::OnObjectListChanged));
	m_TopoType.Create(GetMainPane(), TEXTID_Generic_TopologyLevel_C);
	m_TopoType.AddItem(RVC::VECTOR::TOPOLOGYTYPE_Polygonal, TEXTID_Generic_Polygonal);
	m_TopoType.AddItem(RVC::VECTOR::TOPOLOGYTYPE_Planar, TEXTID_Generic_Planar);
	m_TopoType.AddItem(RVC::VECTOR::TOPOLOGYTYPE_Network, TEXTID_Generic_Network);
	m_TopoType.SetSelectedItemID(RVC::VECTOR::TOPOLOGYTYPE_Polygonal);
	m_Optimize.Create(GetMainPane(), TEXTID_TNTedit_Vector_OptimizeVectorWhenSaving, MGUI::CTRL_TOGGLEBUTTON::STYLE_Check, true);
	
	CreateButtons();
	SetRunEnabled(false);		// Disable the "Run" button until we have objects to process, see OnObjectListChanged()
	return (1);						// Have the dialog show up immediately
	}

ERRVALUE MAINWND::ExecProcess (
	) {
	const RVC::OBJITEMLIST& SrcList = m_ObjectList.GetObjectList();
	RVC::OBJITEMLIST OutList;
	RVC::OBJITEM::SetupOutputList(OutList, SrcList);

	MGUI::DLG_GETOBJECT_FILTER filter(RVC::OBJTYPE_Vector);
	ERRVALUE err = MGUI::DlgGetObjectSet(GetDlgParent(), TEXTID_Prompt_Vectors_Output, filter, OutList, MGUI::DLG_GETOBJECT_STYLE_OnlyNew);
	if (err < 0 && err != R_NewRequest) return (SetErrPosn(err));

	MGUI::DLG_STATUS sd;
	sd.Create(GetDlgParent());
		{
		RVC::COMBINE::COMBINEFLAGS CombineFlags = RVC::COMBINE::COMBINEFLAG_None;
		if (m_Optimize.GetValue()) CombineFlags |= RVC::COMBINE::COMBINEFLAG_Optimize;

		RVC::OBJITEMLIST::CONST_ITERATOR cit;
		RVC::OBJITEMLIST::ITERATOR OutIt;
		MISTRING msg;
		int objnum;
		for (cit = SrcList.Begin(), OutIt = OutList.Begin(), objnum = 0;(cit != SrcList.End());++cit, ++OutIt, ++objnum) {
			msg << STR_Clear << TEXTID_Processing_ObjectXofN << (objnum+1) << OutList.GetNumItems();
			MISTATUSCONTEXT sc(&sd, msg);
			if (sc.IsCancelled()) return (EUserCancel);

			RVC::GEOMETRIC::ELEMSELECTPARMS SelectParms;
			SelectParms.SetSelectAll();
			TRANS2D_MAPGEN SrcToDest;
			if ((err = RVC::COMBINE::ExtractToVector(*cit, SelectParms, SrcToDest, *OutIt, CombineFlags, static_cast<RVC::VECTOR::TOPOLOGYTYPE>(m_TopoType.GetSelectedItemID()))) < 0) return (SetErrPosn(err));
			}
		}
	sd.ReportDone(TEXTID_TimeTo_ConvertToVector);
	return (0);
	}

// Called when "Run" is pressed
void MAINWND::v_OnRun (
	) {
	ERRVALUE err;
	if ((err = ExecProcess()) < 0) {
		MdlgShowErrorCode(GetDlgParent(), err);
		}
	else {
		m_ObjectList.Clear(true);
		}
	return;
	}
	
// Called when the "Exit" button or the "X" icon is pressed
void MAINWND::v_OnCloseRequest (
	) {
	Destroy();
	MGUI::APP_MAIN::Exit();
	return;
	}

// Called when changes to the CAD object list are made, used here to enable the "Run" button
void MAINWND::OnObjectListChanged (
	) {
	SetRunEnabled(!m_ObjectList.IsEmpty());
	return;
	}

//		MAINWND - End
//=====================================================================================================================
//		APP_CADMTOVEC - Begin

class APP_CADTOVEC : public MGUI::APP_BASE {
	public:
		APP_CADTOVEC (
			) { }
	private:
		MAINWND pdp;
		// Overrides from APP_BASE.
		virtual const char* v_GetProcessName () const;
		virtual void v_ExitInstance ();
		virtual bool v_InitInstance ();
	};

const char* APP_CADTOVEC::v_GetProcessName (
	) const {
	return ("cadtovec");
	}

void APP_CADTOVEC::v_ExitInstance (
	) {
	return;
	}

bool APP_CADTOVEC::v_InitInstance (
	) {
	pdp.Create("Main", TEXTID_Title_GeometricToVectorConversion);
	return (true);
	}

//		APP_CADTOVEC - End
//=====================================================================================================================
//		MAIN application instance class

class MAIN : public MGUI::APP_MAIN {
	public:
		MAIN () :m_pApp(new APP_CADTOVEC) { }
	private:
		APP_CADTOVEC *m_pApp;
	};

static MAIN MainApp;

//=====================================================================================================================

}		// End namespace CADTOVECTOR
